@@ -1,5 +1,5 @@
 /*
-$Id: owfs.c,v 1.84 2011/05/11 01:53:29 alfille Exp $
+$Id: owfs.c,v 1.88 2013/01/01 16:03:36 alfille Exp $
      OW -- One-Wire filesystem
     version 0.4 7/2/2003
 
@@ -34,11 +34,10 @@ char *fuse_open_opt = NULL;
 int main(int argc, char *argv[])
 {
 	int c;
-	int allow_other = 0;
 	struct Fuse_option fuse_options;
 
 	/* Set up owlib */
-	LibSetup(opt_owfs);
+	LibSetup(program_type_filesystem);
 
 	/* grab our executable name */
 	if (argc > 0) {
@@ -69,9 +68,6 @@ int main(int argc, char *argv[])
 				ow_exit(0);
 			}
 			break;
-		case e_allow_other:	/* allow_other */
-			allow_other = 1;
-			break;
 		}
 		if ( BAD( owopt(c, optarg) ) ) {
 			ow_exit(0);			/* rest of message */
@@ -125,22 +121,13 @@ int main(int argc, char *argv[])
 	LEVEL_DEBUG("fuse_mnt_opt=[%s]", fuse_mnt_opt);
 	Fuse_parse(fuse_open_opt, &fuse_options);
 	LEVEL_DEBUG("fuse_open_opt=[%s]", fuse_open_opt);
-	if (allow_other) {
+	if (Globals.allow_other) {
 		Fuse_add("-o", &fuse_options);	// add "-o allow_other" to permit other users access
 		Fuse_add("allow_other", &fuse_options);
 	}
-#if OW_MT == 0
-	Fuse_add("-s", &fuse_options);	// single threaded
-#endif							/* OW_MT */
+
 	Globals.now_background = Globals.want_background;	// tell "error" that we are background
-#if 0
-	{
-		int i;
-		for (i = 0; i < fuse_options.argc; i++) {
-			LEVEL_DEBUG("fuse_options.argv[%d]=[%s]", i, fuse_options.argv[i]);
-		}
-	}
-#endif
+
 #if FUSE_VERSION > 25
 	fuse_main(fuse_options.argc, fuse_options.argv, &owfs_oper, NULL);
 #else							/* FUSE_VERSION <= 25 */
@@ -148,6 +135,7 @@ int main(int argc, char *argv[])
 #endif							/* FUSE_VERSION <= 25 */
 	Fuse_cleanup(&fuse_options);
 #endif
+
 	ow_exit(0);
 	return 0;
 }
