@@ -1,12 +1,12 @@
 /*
-$Id: owfs.h,v 1.12 2007/01/30 05:19:34 alfille Exp $
+$Id$
    OWFS and OWHTTPD
    one-wire file system and
    one-wire web server
 
     By Paul H Alfille
     {c} 2003 GPL
-    palfille@earthlink.net
+    paul.alfille@gmail.com
 */
 
 /* OWFS - specific header */
