@@ -1,5 +1,6 @@
-/* 
- * $Id: ftp_session.c,v 1.67 2012/03/13 00:06:35 alfille Exp $
+/*
+ * part of owftpd By Paul H Alfille
+ * The whole is GPLv2 licenced though the ftp code was more liberally licenced when first used.
  */
 
 #include "owftpd.h"
@@ -14,16 +15,16 @@
 static int invariant(const struct ftp_session_s *f);
 static void reply(struct ftp_session_s *f, int code, const char *fmt, ...);
 static void change_dir(struct ftp_session_s *f, const char *new_dir);
-static int open_connection(struct ftp_session_s *f);
-static int write_fully(int file_descriptor, const char *buf, int buflen);
+static FILE_DESCRIPTOR_OR_ERROR open_connection(struct ftp_session_s *f);
+static int write_fully(FILE_DESCRIPTOR_OR_ERROR file_descriptor, const char *buf, int buflen);
 static void init_passive_port(void);
 static int get_passive_port(void);
 static int convert_newlines(char *dst, const char *src, int srclen);
 static void get_addr_str(const sockaddr_storage_t * s, char *buf, int bufsiz);
 static void send_readme(const struct ftp_session_s *f, int code);
-static void netscape_hack(int file_descriptor);
+static void netscape_hack(FILE_DESCRIPTOR_OR_ERROR file_descriptor);
 static void set_port(struct ftp_session_s *f, const sockaddr_storage_t * host_port);
-static int set_pasv(struct ftp_session_s *f, sockaddr_storage_t * host_port);
+static FILE_DESCRIPTOR_OR_ERROR set_pasv(struct ftp_session_s *f, sockaddr_storage_t * host_port);
 //static int ip_equal(const sockaddr_storage_t *a, const sockaddr_storage_t *b);
 static int ip_equal(const sockaddr_storage_t * a, const sockaddr_storage_t * b);
 static void get_absolute_fname(char *fname, size_t fname_len, const char *dir, const char *file);
@@ -153,7 +154,7 @@ int ftp_session_init(struct ftp_session_s *f,
 
 	f->data_channel = DATA_PORT;
 	f->data_port = *client_addr;
-	f->server_fd = -1;
+	f->server_fd = FILE_DESCRIPTOR_BAD;
 
 	daemon_assert(invariant(f));
 
@@ -207,8 +208,9 @@ void ftp_session_run(struct ftp_session_s *f, struct watched_s *watched)
 			reply(f, 500, "Command line too long.");
 			while (telnet_session_readln(f->telnet_session, buf, sizeof(buf))) {
 				len = strlen(buf);
-				if (buf[len - 1] == '\n')
+				if (buf[len - 1] == '\n') {
 					break;
+				}
 			}
 			goto next_command;
 		}
@@ -243,10 +245,7 @@ void ftp_session_destroy(struct ftp_session_s *f)
 {
 	daemon_assert(invariant(f));
 
-	if (f->server_fd != -1) {
-		close(f->server_fd);
-		f->server_fd = -1;
-	}
+	Test_and_Close( & f->server_fd ) ;
 }
 
 #ifndef NDEBUG
@@ -288,12 +287,12 @@ static int invariant(const struct ftp_session_s *f)
 		if (!ip_equal(&f->client_addr, &f->data_port)) {
 			return 0;
 		}
-		if (f->server_fd != -1) {
+		if ( FILE_DESCRIPTOR_NOT_VALID(f->server_fd)) {
 			return 0;
 		}
 		break;
 	case DATA_PASSIVE:
-		if (f->server_fd < 0) {
+		if ( FILE_DESCRIPTOR_NOT_VALID(f->server_fd)) {
 			return 0;
 		}
 		break;
@@ -439,7 +438,7 @@ static void change_dir(struct ftp_session_s *f, const char *new_dir)
 	struct cd_parse_s cps;
 
 	strcpy(cps.buffer, (ASCII *) f->dir);
-	strcpy(cps.rest, new_dir);
+	cps.rest = new_dir;
 	cps.pse = parse_status_init;
 
 	LEVEL_DEBUG("CD dir=%s, file=%s", SAFESTRING(cps.buffer), SAFESTRING(new_dir));
@@ -460,7 +459,8 @@ static void change_dir(struct ftp_session_s *f, const char *new_dir)
 	if (cps.ret) {
 		reply(f, 550, "Error changing directory. %s", strerror(-cps.ret));
 	} else {
-		strcpy(f->dir, cps.dir);
+		f->dir[PATH_MAX] = '\0' ; // trailing null
+		strncpy(f->dir, cps.dir, PATH_MAX);
 		reply(f, 250, "Directory change successful.");
 	}
 
@@ -499,8 +499,7 @@ static void set_port(struct ftp_session_s *f, const sockaddr_storage_t * host_po
 	} else {
 		/* close any outstanding PASSIVE port */
 		if (f->data_channel == DATA_PASSIVE) {
-			close(f->server_fd);
-			f->server_fd = -1;
+			Test_and_Close( & f->server_fd ) ;
 		}
 
 		f->data_channel = DATA_PORT;
@@ -574,20 +573,20 @@ static void do_eprt(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 /* support for the various pasv setting functions */
 /* returns the file descriptor of the bound port, or -1 on error */
 /* note: the "host_port" parameter will be modified, having its port set */
-static int set_pasv(struct ftp_session_s *f, sockaddr_storage_t * bind_addr)
+static FILE_DESCRIPTOR_OR_ERROR set_pasv(struct ftp_session_s *f, sockaddr_storage_t * bind_addr)
 {
-	int socket_fd;
+	FILE_DESCRIPTOR_OR_ERROR socket_fd;
 	int port;
 
 	daemon_assert(invariant(f));
 	daemon_assert(bind_addr != NULL);
 
 	socket_fd = socket(SSFAM(bind_addr), SOCK_STREAM, 0);
-	if (socket_fd == -1) {
+	if (FILE_DESCRIPTOR_NOT_VALID(socket_fd)) {
 		reply(f, 500, "Error creating server socket; %s.", strerror(errno));
-		return -1;
+		return FILE_DESCRIPTOR_BAD;
 	}
-	fcntl (socket_fd, F_SETFD, FD_CLOEXEC); // for safe forking
+//	fcntl (socket_fd, F_SETFD, FD_CLOEXEC); // for safe forking
 
 	for (;;) {
 		port = get_passive_port();
@@ -597,15 +596,15 @@ static int set_pasv(struct ftp_session_s *f, sockaddr_storage_t * bind_addr)
 		}
 		if (errno != EADDRINUSE) {
 			reply(f, 500, "Error binding server port; %s.", strerror(errno));
-			close(socket_fd);
-			return -1;
+			Test_and_Close(&socket_fd);
+			return FILE_DESCRIPTOR_BAD;
 		}
 	}
 
 	if (listen(socket_fd, 1) != 0) {
 		reply(f, 500, "Error listening on server port; %s.", strerror(errno));
-		close(socket_fd);
-		return -1;
+		Test_and_Close(&socket_fd);
+		return FILE_DESCRIPTOR_BAD;
 	}
 
 	return socket_fd;
@@ -614,7 +613,7 @@ static int set_pasv(struct ftp_session_s *f, sockaddr_storage_t * bind_addr)
 /* pick a server port to listen for connection on */
 static void do_pasv(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 {
-	int socket_fd;
+	FILE_DESCRIPTOR_OR_ERROR socket_fd;
 	unsigned int addr;
 	int port;
 
@@ -628,7 +627,7 @@ static void do_pasv(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	}
 
 	socket_fd = set_pasv(f, &f->server_ipv4_addr);
-	if (socket_fd == -1) {
+	if (FILE_DESCRIPTOR_NOT_VALID(socket_fd)) {
 		goto exit_pasv;
 	}
 
@@ -640,7 +639,7 @@ static void do_pasv(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 
 	/* close any outstanding PASSIVE port */
 	if (f->data_channel == DATA_PASSIVE) {
-		close(f->server_fd);
+		Test_and_Close( & f->server_fd);
 	}
 	f->data_channel = DATA_PASSIVE;
 	f->server_fd = socket_fd;
@@ -652,7 +651,7 @@ static void do_pasv(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 /* pick a server port to listen for connection on, including IPv6 */
 static void do_lpsv(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 {
-	int socket_fd;
+	FILE_DESCRIPTOR_OR_ERROR socket_fd;
 	char addr[80];
 	uint8_t *a;
 	uint8_t *p;
@@ -667,7 +666,7 @@ static void do_lpsv(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	}
 
 	socket_fd = set_pasv(f, &f->server_addr);
-	if (socket_fd == -1) {
+	if (FILE_DESCRIPTOR_NOT_VALID(socket_fd)) {
 		goto exit_lpsv;
 	}
 
@@ -691,7 +690,7 @@ static void do_lpsv(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 
 	/* close any outstanding PASSIVE port */
 	if (f->data_channel == DATA_PASSIVE) {
-		close(f->server_fd);
+		Test_and_Close( & f->server_fd);
 	}
 	f->data_channel = DATA_PASSIVE;
 	f->server_fd = socket_fd;
@@ -703,7 +702,7 @@ static void do_lpsv(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 /* pick a server port to listen for connection on, new IPv6 method */
 static void do_epsv(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 {
-	int socket_fd;
+	FILE_DESCRIPTOR_OR_ERROR socket_fd;
 	sockaddr_storage_t *addr;
 
 	daemon_assert(invariant(f));
@@ -743,7 +742,7 @@ static void do_epsv(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 
 	/* bind port and so on */
 	socket_fd = set_pasv(f, addr);
-	if (socket_fd == -1) {
+	if (FILE_DESCRIPTOR_NOT_VALID(socket_fd)) {
 		goto exit_epsv;
 	}
 
@@ -752,7 +751,7 @@ static void do_epsv(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 
 	/* close any outstanding PASSIVE port */
 	if (f->data_channel == DATA_PASSIVE) {
-		close(f->server_fd);
+		Test_and_Close( & f->server_fd ) ;
 	}
 	f->data_channel = DATA_PASSIVE;
 	f->server_fd = socket_fd;
@@ -909,8 +908,7 @@ static void get_absolute_fname(char *fname, size_t fname_len, const char *dir, c
 static void do_retr(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 {
 	const char *file_name;
-	int file_fd;
-	int socket_fd;
+	FILE_DESCRIPTOR_OR_ERROR socket_fd;
 	ASCII *buf2 = NULL;
 	ASCII *bufwrite;
 	struct timeval start_timestamp;
@@ -926,8 +924,7 @@ static void do_retr(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	daemon_assert(cmd->num_arg == 1);
 
 	/* set up for exit */
-	file_fd = -1;
-	socket_fd = -1;
+	socket_fd = FILE_DESCRIPTOR_BAD;
 
 	/* create an absolute name for our file */
 	file_name = cmd->arg[0].string;
@@ -996,8 +993,9 @@ static void do_retr(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 
 	/* open data path */
 	socket_fd = open_connection(f);
-	if (socket_fd == -1)
+	if (FILE_DESCRIPTOR_NOT_VALID(socket_fd)) {
 		goto exit_retr;
+	}
 
 	/* we're golden, send the file */
 	if (write_fully(socket_fd, bufwrite, size_write) == -1) {
@@ -1008,8 +1006,7 @@ static void do_retr(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	watchdog_defer_watched(f->watched);
 
 	/* disconnect */
-	close(socket_fd);
-	socket_fd = -1;
+	Test_and_Close(&socket_fd);
 
 	/* hey, it worked, let the other side know */
 	reply(f, 226, "File transfer complete.");
@@ -1030,20 +1027,14 @@ static void do_retr(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 		free(buf2);
 	}
 	f->file_offset = 0;
-	if (socket_fd != -1) {
-		close(socket_fd);
-	}
-	if (file_fd != -1) {
-		close(file_fd);
-	}
+	Test_and_Close(&socket_fd) ;
 	daemon_assert(invariant(f));
 }
 
 /* Write to 1-wire device */
 static void do_stor(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 {
-	int file_fd;
-	int socket_fd;
+	FILE_DESCRIPTOR_OR_ERROR socket_fd;
 	struct timeval start_timestamp;
 	struct timeval end_timestamp;
 	struct timeval transfer_time;
@@ -1051,7 +1042,7 @@ static void do_stor(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 
 	size_t size_read;
 	off_t offset = 0 ;
-	char * data_in ;
+	char * data_in = NULL ;
 	struct parsedname *pn;
 	OWQ_allocate_struct_and_pointer(owq);
 
@@ -1062,8 +1053,7 @@ static void do_stor(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	daemon_assert(cmd->num_arg == 1);
 
 	/* set up for exit */
-	file_fd = -1;
-	socket_fd = -1;
+	socket_fd = FILE_DESCRIPTOR_BAD;
 
 	/* create an absolute name for our file */
 	if ( BAD( OWQ_create_plus(f->dir, cmd->arg[0].string, owq) ) ) {
@@ -1101,7 +1091,7 @@ static void do_stor(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 
 	/* open data path */
 	socket_fd = open_connection(f);
-	if (socket_fd == -1) {
+	if ( FILE_DESCRIPTOR_NOT_VALID(socket_fd) ) {
 		goto exit_stor;
 	}
 
@@ -1119,18 +1109,15 @@ static void do_stor(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 			goto exit_stor;
 		}
 		if ( BAD( OWQ_allocate_write_buffer( data_in, size_actual, offset, owq )) ) {
-			owfree(data_in) ;
 			reply(f, 550, "Out of memory.");
 			goto exit_stor;
 		}
-		owfree(data_in) ;
 	}
 
 	watchdog_defer_watched(f->watched);
 
 	/* disconnect */
-	close(socket_fd);
-	socket_fd = -1;
+	Test_and_Close(&socket_fd) ;
 
 	/* hey, it worked, let the other side know */
 	reply(f, 226, "File transfer complete.");
@@ -1154,21 +1141,27 @@ static void do_stor(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	LEVEL_DATA("%s stored \"%s\", %ld bytes in "TVformat,
 			   f->client_addr_str, pn->path, OWQ_size(owq), TVvar(&transfer_time) );
 
+
+	/* Fall through */
   exit_stor:
 	OWQ_destroy(owq);
+
 	f->file_offset = 0;
-	if (socket_fd != -1) {
-		close(socket_fd);
-	}
-	if (file_fd != -1) {
-		close(file_fd);
-	}
+
+	/* Free memory */
+	if ( data_in != NULL ) {
+		owfree(data_in) ;
+	} 
+
+	/* disconnect */
+	Test_and_Close(&socket_fd) ;
+
 	daemon_assert(invariant(f));
 }
 
-static int open_connection(struct ftp_session_s *f)
+static FILE_DESCRIPTOR_OR_ERROR open_connection(struct ftp_session_s *f)
 {
-	int socket_fd;
+	FILE_DESCRIPTOR_OR_ERROR socket_fd;
 	struct sockaddr_in addr;
 	unsigned addr_len;
 
@@ -1176,24 +1169,24 @@ static int open_connection(struct ftp_session_s *f)
 
 	if (f->data_channel == DATA_PORT) {
 		socket_fd = socket(SSFAM(&f->data_port), SOCK_STREAM, 0);
-		if (socket_fd == -1) {
+		if ( FILE_DESCRIPTOR_NOT_VALID(socket_fd) ) {
 			reply(f, 425, "Error creating socket; %s.", strerror(errno));
-			return -1;
+			return FILE_DESCRIPTOR_BAD ;
 		}
-		fcntl (socket_fd, F_SETFD, FD_CLOEXEC); // for safe forking
+//		fcntl (socket_fd, F_SETFD, FD_CLOEXEC); // for safe forking
 		if (connect(socket_fd, (struct sockaddr *) &f->data_port, sizeof(sockaddr_storage_t)) != 0) {
 			reply(f, 425, "Error connecting; %s.", strerror(errno));
-			close(socket_fd);
-			return -1;
+			Test_and_Close(&socket_fd);
+			return FILE_DESCRIPTOR_BAD;
 		}
 	} else {
 		daemon_assert(f->data_channel == DATA_PASSIVE);
 
 		addr_len = sizeof(struct sockaddr_in);
 		socket_fd = accept(f->server_fd, (struct sockaddr *) &addr, &addr_len);
-		if (socket_fd == -1) {
+		if ( FILE_DESCRIPTOR_NOT_VALID(socket_fd) ) {
 			reply(f, 425, "Error accepting connection; %s.", strerror(errno));
-			return -1;
+			return FILE_DESCRIPTOR_BAD;
 		}
 #ifdef INET6
 		/* in IPv6, the client can connect to a channel using a different */
@@ -1202,15 +1195,15 @@ static int open_connection(struct ftp_session_s *f)
 		if (SAFAM(addr) == SSFAM(&f->client_addr)) {
 			if (memcmp(&SINADDR(&f->client_addr), &SINADDR(&addr), sizeof(SINADDR(&addr)))) {
 				reply(f, 425, "Error accepting connection; connection from invalid IP.");
-				close(socket_fd);
-				return -1;
+				Test_and_Close(&socket_fd);
+				return FILE_DESCRIPTOR_BAD;
 			}
 		}
 #else
 		if (memcmp(&f->client_addr.sin_addr, &addr.sin_addr, sizeof(struct in_addr))) {
 			reply(f, 425, "Error accepting connection; connection from invalid IP.");
-			close(socket_fd);
-			return -1;
+			Test_and_Close(&socket_fd);
+			return FILE_DESCRIPTOR_BAD;
 		}
 #endif
 	}
@@ -1237,7 +1230,7 @@ static int convert_newlines(char *dst, const char *src, int srclen)
 	return dstlen;
 }
 
-static int write_fully(int file_descriptor, const char *buf, int buflen)
+static int write_fully(FILE_DESCRIPTOR_OR_ERROR file_descriptor, const char *buf, int buflen)
 {
 	int amt_written;
 	int write_ret;
@@ -1272,7 +1265,7 @@ static void both_list(struct ftp_session_s *f, const struct ftp_command_s *cmd,
 	fps.rest = NULL;
 	fps.pse = parse_status_init;
 	fps.fle = fle;
-	fps.out = -1;
+	fps.out = FILE_DESCRIPTOR_BAD;
 
 
 	daemon_assert(invariant(f));
@@ -1295,7 +1288,7 @@ static void both_list(struct ftp_session_s *f, const struct ftp_command_s *cmd,
 
 	/* open our data connection */
 	fps.out = open_connection(f);
-	if (fps.out == -1) {
+	if (FILE_DESCRIPTOR_NOT_VALID(fps.out)) {
 		goto exit_blst;
 	}
 
@@ -1313,8 +1306,7 @@ static void both_list(struct ftp_session_s *f, const struct ftp_command_s *cmd,
 
 	/* clean up and exit */
   exit_blst:
-	if (fps.out != -1)
-		close(fps.out);
+	Test_and_Close( & fps.out ) ;
 	daemon_assert(invariant(f));
 }
 
@@ -1472,14 +1464,14 @@ static void send_readme(const struct ftp_session_s *f, int code)
 }
 
 /* hack which prevents Netscape error in file list */
-static void netscape_hack(int file_descriptor)
+static void netscape_hack(FILE_DESCRIPTOR_OR_ERROR file_descriptor)
 {
 	fd_set readfds;
 	struct timeval ns_timeout = {15, 0 } ; // 15 seconds
 	int select_ret;
 	char c;
 
-	daemon_assert(file_descriptor >= 0);
+	daemon_assert(FILE_DESCRIPTOR_VALID(file_descriptor));
 
 	shutdown(file_descriptor, 1);
 	FD_ZERO(&readfds);
