@@ -1,5 +1,5 @@
 /*
-$Id: owhttpd_handler.c,v 1.98 2011/12/20 03:40:28 alfille Exp $
+$Id$
  * http.c for owhttpd (1-wire web server)
  * By Paul Alfille 2003, using libow
  * offshoot of the owfs ( 1wire file system )
@@ -31,8 +31,9 @@ struct urlparse {
 enum http_return { http_ok, http_dir, http_icon, http_400, http_404 } ;
 
 	/* Error page functions */
-static void Bad400(FILE * out, struct parsedname * pn);
-static void Bad404(FILE * out, struct parsedname * pn);
+enum content_type PoorMansParser( char * bad_url ) ;
+static void Bad400(FILE * out, const enum content_type ct);
+static void Bad404(FILE * out, const enum content_type ct);
 
 /* URL parsing function */
 static void URLparse(struct urlparse *up);
@@ -48,7 +49,8 @@ static char * GetPostPath( FILE * out ) ;
 /* Main handler for a web page */
 int handle_socket(FILE * out)
 {
-	enum http_return http_code = http_ok ;
+	enum http_return http_code ;
+	enum content_type pmp = ct_html;
 
 	struct urlparse up;
 
@@ -117,6 +119,17 @@ int handle_socket(FILE * out)
 			ReadToCRLF(out) ;
 			http_code = http_400 ;
 		}
+		switch ( http_code ) {
+			case http_400:
+			case http_404:
+				// need to call this before freeing up.file
+				pmp = PoorMansParser(up.file) ;
+				break ;
+			default:
+				// not an error
+				break ;
+		}
+		// allocated by getline
 		free(up.line) ;
 	} else {
 		LEVEL_DEBUG("No http data.");
@@ -132,10 +145,10 @@ int handle_socket(FILE * out)
 			Favicon(out);
 			break ;
 		case http_400:
-			Bad400(out,pn);
+			Bad400(out,pmp);
 			break ;
 		case http_404:
-			Bad404(out,pn);
+			Bad404(out,pmp);
 			break ;
 		case http_dir:
 			ShowDir(out, pn);
@@ -208,7 +221,7 @@ static enum http_return handle_GET(FILE * out, struct urlparse * up)
 /* The HTTP request is a POST message */
 static enum http_return handle_POST(FILE * out, struct urlparse * up)
 {
-	enum http_return http_code ;
+	enum http_return http_code = http_404 ; // default error mode
 
 	char * boundary = NULL ;
 	size_t boundary_length ;
@@ -234,23 +247,18 @@ static enum http_return handle_POST(FILE * out, struct urlparse * up)
 					if ( GOOD( OWQ_allocate_write_buffer( (char *) MemblobData(&mb), MemblobLength(&mb), 0, owq)) ) {
 						PostData(owq);
 						http_code = http_ok ;
-					} else {
-						http_code = http_404 ;
 					}
 					OWQ_destroy(owq) ;
 				} else {
 					LEVEL_DEBUG("Can't create %s",post_path);
-					http_code = http_404 ;
 				}
 			} else {
 				LEVEL_DEBUG("Can't read full binary data from file upload");
-				http_code = http_404 ;
 			}
 			MemblobClear( &mb ) ;
 			owfree(post_path ) ;
 		} else {
 			LEVEL_DEBUG("Can't read property name from file upload");
-			http_code = http_404 ;
 		}
 	}
 	if ( boundary ) {
@@ -335,49 +343,49 @@ static void URLparse(struct urlparse *up)
 }
 
 
-static void Bad400(FILE * out, struct parsedname *pn)
+static void Bad400(FILE * out, const enum content_type ct)
 {
 	LEVEL_CALL("Return a 400 HTTP error code");
-	if ( pn == NULL ) {
-		HTTPstart(out, "400 Bad Request", ct_text);
-		fprintf(out, "null");
-	} else if ( pn->state & ePS_text ) {
-		HTTPstart(out, "400 Bad Request", ct_text);
-		fprintf(out, "400 Bad request");
-	} else if ( pn->state & ePS_json ) {
-		HTTPstart(out, "400 Bad Request", ct_text);
-		fprintf(out, "null");
-	} else {
-		HTTPstart(out, "400 Bad Request", ct_html);
-		HTTPtitle(out, "Error 400 -- Bad request");
-		HTTPheader(out, "Unrecognized Request");
-		fprintf(out, "<P>The 1-wire web server is carefully constrained for security and stability. Your requested web page is not recognized.</P>");
-		fprintf(out, "<P>Navigate from the <A HREF=\"/\">Main page</A> for best results.</P>");
+	switch( ct ) {
+		case ct_text:
+			HTTPstart(out, "400 Bad Request", ct_text);
+			fprintf(out, "400 Bad request");
+			break ;
+		case ct_json:
+			HTTPstart(out, "400 Bad Request", ct_json);
+			fprintf(out, "{}");
+			break ;
+		default:
+			HTTPstart(out, "400 Bad Request", ct_html);
+			HTTPtitle(out, "Error 400 -- Bad request");
+			HTTPheader(out, "Unrecognized Request");
+			fprintf(out, "<P>The 1-wire web server is carefully constrained for security and stability. Your requested web page is not recognized.</P>");
+			fprintf(out, "<P>Navigate from the <A HREF=\"/\">Main page</A> for best results.</P>");
+			HTTPfoot(out);
 	}
-	HTTPfoot(out);
 }
 
-static void Bad404(FILE * out, struct parsedname *pn)
+static void Bad404(FILE * out, const enum content_type ct)
 {
 	LEVEL_CALL("Return a 404 HTTP error code");
-	if ( pn == NULL ) {
-		HTTPstart(out, "404 Not Found", ct_text);
-		fprintf(out, "null");
-	} else if ( pn->state & ePS_text ) {
-		HTTPstart(out, "404 Not Found", ct_text);
-		fprintf(out, "404 Not Found");
-	} else if ( pn->state & ePS_json ) {
-		HTTPstart(out, "404 Not Found", ct_text);
-		fprintf(out, "null");
-	} else {
-		HTTPstart(out, "404 Not Found", ct_html);
-		HTTPtitle(out, "Error 404 -- Item doesn't exist");
-		HTTPheader(out, "Nonexistent Device");
-		fprintf(out, "<P>The 1-wire web server is carefully constrained for security and stability. Your requested device is not recognized.</P>");
-		fprintf(out, "<P>Navigate from the <A HREF=\"/\">Main page</A> for best results.</P>");
-	}
-	HTTPfoot(out);
-}
+	switch( ct ) {
+		case ct_text:
+			HTTPstart(out, "404 Not Found", ct_text);
+			fprintf(out, "404 Not Found");
+			break ;
+		case ct_json:
+			HTTPstart(out, "404 Not Found", ct_json);
+			fprintf(out, "{}");
+			break ;
+		default:
+			HTTPstart(out, "404 Not Found", ct_html);
+			HTTPtitle(out, "Error 404 -- Item doesn't exist");
+			HTTPheader(out, "Nonexistent Device");
+			fprintf(out, "<P>The 1-wire web server is carefully constrained for security and stability. Your requested device is not recognized.</P>");
+			fprintf(out, "<P>Navigate from the <A HREF=\"/\">Main page</A> for best results.</P>");
+			HTTPfoot(out);
+}	}
+
 
 static void ReadToCRLF( FILE * out )
 {
@@ -480,3 +488,27 @@ static int GetPostData( char * boundary, struct memblob * mb, FILE * out )
 	LEVEL_DEBUG("HTTP error -- no ending MIME boundary");
 	return 1 ;
 }
+
+// Parse the line for just the text or json key since there was an error and Parsedname is null
+enum content_type PoorMansParser( char * bad_url )
+{
+	if ( bad_url == NULL ) {
+		LEVEL_DEBUG("Error on http request assume html");
+		return ct_html ;
+	} else if ( strstr( bad_url, "json" ) != NULL ) {
+		LEVEL_DEBUG("Error on http request <%s> assume json",bad_url);
+		return ct_json ;
+	} else if ( strstr( bad_url, "JSON" ) != NULL ) {
+		LEVEL_DEBUG("Error on http request <%s> assume json",bad_url);
+		return ct_json ;
+	} else if ( strstr( bad_url, "text" ) != NULL ) {
+		LEVEL_DEBUG("Error on http request <%s> assume text",bad_url);
+		return ct_text ;
+	} else if ( strstr( bad_url, "TEXT" ) != NULL ) {
+		LEVEL_DEBUG("Error on http request <%s> assume text",bad_url);
+		return ct_text ;
+	}
+	LEVEL_DEBUG("Error on http request <%s> assume html",bad_url);
+	return ct_html ;
+}
+			
