@@ -1,9 +1,8 @@
 /*
-$Id: error.c,v 1.39 2011/11/20 21:44:34 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -30,7 +29,6 @@ $Id: error.c,v 1.39 2011/11/20 21:44:34 alfille Exp $
 
 /* module/ownet/c/src/c/error.c & module/owlib/src/c/error.c are identical */
 
-#if OW_MT
 const char mutex_init_failed[] = "mutex_init failed rc=%d [%s]\n";
 const char mutex_destroy_failed[] = "mutex_destroy failed rc=%d [%s]\n";
 const char mutex_lock_failed[] = "mutex_lock failed rc=%d [%s]\n";
@@ -43,10 +41,16 @@ const char rwlock_read_lock_failed[] = "rwlock_read_lock failed rc=%d [%s]\n";
 const char rwlock_read_unlock_failed[] = "rwlock_read_unlock failed rc=%d [%s]\n";
 const char cond_timedwait_failed[] = "cond_timedwait failed rc=%d [%s]\n";
 const char cond_signal_failed[] = "cond_signal failed rc=%d [%s]\n";
+const char cond_broadcast_failed[] = "cond_broadcast failed rc=%d [%s]\n";
 const char cond_wait_failed[] = "cond_wait failed rc=%d [%s]\n";
 const char cond_init_failed[] = "cond_init failed rc=%d [%s]\n";
 const char cond_destroy_failed[] = "cond_destroy failed rc=%d [%s]\n";
-#endif
+const char sem_init_failed[] = "sem_init failed rc=%d [%s]\n";
+const char sem_post_failed[] = "sem_post failed rc=%d [%s]\n";
+const char sem_wait_failed[] = "sem_wait failed rc=%d [%s]\n";
+const char sem_trywait_failed[] = "sem_trywait failed rc=%d [%s]\n";
+const char sem_timedwait_failed[] = "sem_timedwait failed rc=%d [%s]\n";
+const char sem_destroy_failed[] = "sem_destroy failed rc=%d [%s]\n";
 
 static void err_format(char * format, int errno_save, const char * level_string, const char * file, int line, const char * func, const char * fmt);
 static void hex_print( const char * buf, int length ) ;
@@ -63,6 +67,32 @@ int log_available = 0;
  * Caller specifies "errnoflag" and "level" */
 #define MAXLINE     1023
 
+void print_timestamp_(const char * file, int line, const char * func, const char *fmt, ...)
+{
+	struct timeval tv;
+	char buf[MAXLINE + 3];
+	char format[MAXLINE + 3];
+	va_list va;
+
+	gettimeofday(&tv, NULL);
+	
+	// Add line and file to message
+	snprintf(format, MAXLINE, "%s:%s(%d) %s", file,func,line,fmt);  /* safe */
+
+	va_start( va, fmt );
+	// Add print format arguemnts (variable number)
+#ifdef HAVE_VSNPRINTF
+	vsnprintf(buf, MAXLINE, format, va);    /* safe */
+#else
+	vsprintf(buf, format, va);      /* not safe */
+#endif
+	va_end( va ) ;
+
+	//` Now print it out
+	fprintf(stderr, "%ld DEFAULT: %s %ld.%06ld\n", time(NULL), buf, tv.tv_sec, tv.tv_usec);
+	fflush(stderr);
+}
+
 void err_msg(enum e_err_type errnoflag, enum e_err_level level, const char * file, int line, const char * func, const char *fmt, ...)
 {
 	int errno_save = (errnoflag==e_err_type_error)?errno:0;		/* value caller might want printed */
@@ -97,7 +127,15 @@ void err_msg(enum e_err_type errnoflag, enum e_err_level level, const char * fil
 	/* Print where? */
 	switch (Globals.error_print) {
 	case e_err_print_mixed:
-		sl = Globals.now_background ? e_err_print_syslog : e_err_print_console;
+		switch (Globals.daemon_status) {
+			case e_daemon_want_bg:
+			case e_daemon_bg:
+				sl = e_err_print_syslog ;
+				break ;
+			default:
+				sl = e_err_print_console;
+				break ;
+		}		
 		break;
 	case e_err_print_syslog:
 		sl = e_err_print_syslog;
@@ -161,6 +199,7 @@ void _Debug_Bytes(const char *title, const unsigned char *buf, int length)
 	ascii_print( (const char *) buf, length ) ;
 }
 
+/* calls exit() so never returns */
 void fatal_error(const char * file, int line, const char * func, const char *fmt, ...)
 {
 	va_list ap;
@@ -179,7 +218,7 @@ void fatal_error(const char * file, int line, const char * func, const char *fmt
 #else
 		vsprintf(buf, fmt, ap);
 #endif
-		fprintf(stderr, buf);
+		fprintf(stderr, "%s", buf);
 	}
 #else /* OWNETC_OW_DEBUG */
 	if(Globals.fatal_debug) {
@@ -192,7 +231,15 @@ void fatal_error(const char * file, int line, const char * func, const char *fmt
 		/* Print where? */
 		switch (Globals.error_print) {
 			case e_err_print_mixed:
-				sl = Globals.now_background ? e_err_print_syslog : e_err_print_console;
+				switch (Globals.daemon_status) {
+					case e_daemon_want_bg:
+					case e_daemon_bg:
+						sl = e_err_print_syslog ;
+						break ;
+					default:
+						sl = e_err_print_console;
+						break ;
+				}		
 				break;
 			case e_err_print_syslog:
 				sl = e_err_print_syslog;
@@ -201,6 +248,7 @@ void fatal_error(const char * file, int line, const char * func, const char *fmt
 				sl = e_err_print_console;
 				break;
 			default:
+				va_end(ap);
 				return;
 		}
 
@@ -237,6 +285,8 @@ void fatal_error(const char * file, int line, const char * func, const char *fmt
 	}
 #endif /* OWNETC_OW_DEBUG */
 	va_end(ap);
+	debug_crash(); // Core-dump to make it possible to trace down the problem!
+	//exit(EXIT_FAILURE) ;
 }
 
 static void err_format(char * format, int errno_save, const char * level_string, const char * file, int line, const char * func, const char * fmt)
