@@ -1,9 +1,8 @@
 /*
-$Id: globals.c,v 1.66 2011/05/15 23:30:25 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -16,7 +15,6 @@ $Id: globals.c,v 1.66 2011/05/15 23:30:25 alfille Exp $
 
 /* Globals for port and bus communication */
 /* connections globals stored in ow_connect.c */
-/* i.e. connection_in * Inbound_Control.head ...         */
 
 /* char * pid_file in ow_opt.c */
 
@@ -35,8 +33,9 @@ int32_t LocalControlFlags ;
 struct global Globals = {
 	.announce_off = 0,
 	.announce_name = NULL,
-	.opt = opt_swig,
+	.program_type = program_type_swig,
 	.progname = NULL,			// "One Wire File System" , Can't allocate here since it's freed
+	.allow_other = 0 , // for fuse
 
 	.temp_scale = temp_celsius,
 	.pressure_scale = pressure_mbar,
@@ -45,12 +44,12 @@ struct global Globals = {
 	.uncached = 0,
 	.unaliased = 0,
 	
-	.want_background = 1,
-	.now_background = 0,
-
+	.daemon_status = e_daemon_want_bg ,
+	
 	.error_level = e_err_default,
+	.error_level_restore = e_err_default,
 	.error_print = e_err_print_mixed,
-	.fatal_debug = 0,
+	.fatal_debug = 1,
 	.fatal_debug_file = NULL,
 
 	.concurrent_connections = 10,
@@ -84,19 +83,24 @@ struct global Globals = {
 	.clients_persistent_high = 20,
 
 	.usb_scan_interval = DEFAULT_USB_SCAN_INTERVAL,
+	.enet_scan_interval = DEFAULT_ENET_SCAN_INTERVAL,
 
 	.pingcrazy = 0,
 	.no_dirall = 0,
 	.no_get = 0,
 	.no_persistence = 0,
 	.eightbit_serial = 0,
+	.trim = 0, // don't whitespace trim results by default
 	.zero = zero_unknown ,
 	.i2c_APU = 1 ,
-	.i2c_PPM = 1 ,
+	.i2c_PPM = 0 , // to prevent confusing the DS2483
 	.baud = B9600 ,
 
 	.templow = GLOBAL_UNTOUCHED_TEMP_LIMIT,
 	.temphigh = GLOBAL_UNTOUCHED_TEMP_LIMIT,
+	
+//	.allow_external = 1 , // for testing
+	.allow_external = 0 , // unless program == owexternal
 };
 
 // generic value for ignorable function returns 
@@ -121,6 +125,8 @@ void SetLocalControlFlags( void )
 	LocalControlFlags |= Globals.uncached ? UNCACHED : 0 ;
 	// Unaliased
 	LocalControlFlags |= Globals.unaliased ? 0 : ALIAS_REQUEST ;
+	// Trim
+	LocalControlFlags |= Globals.trim ? 0 : TRIM ;
 	// OWNet flag or Presence check
 	LocalControlFlags |= OWNET ;
 	CONTROLFLAGSUNLOCK;
