@@ -1,9 +1,8 @@
 /*
-$Id: ow_2408.c,v 1.73 2011/07/27 12:15:07 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -70,6 +69,7 @@ READ_FUNCTION(FS_r_pio);
 WRITE_FUNCTION(FS_w_pio);
 READ_FUNCTION(FS_sense);
 READ_FUNCTION(FS_power);
+WRITE_FUNCTION(FS_out_of_testmode);
 READ_FUNCTION(FS_r_latch);
 WRITE_FUNCTION(FS_w_latch);
 READ_FUNCTION(FS_r_s_alarm);
@@ -86,33 +86,41 @@ WRITE_FUNCTION(FS_Hscreen);
 WRITE_FUNCTION(FS_Hscreenyx);
 WRITE_FUNCTION(FS_Hmessage);
 WRITE_FUNCTION(FS_Honoff);
+WRITE_FUNCTION(FS_redefchar);
+WRITE_FUNCTION(FS_redefchar_hex);
 
 #define PROPERTY_LENGTH_LCD_MESSAGE   128
+#define LCD_REDEFCHAR_LENGTH 8
+#define LCD_REDEFCHAR_LENGTH_HEX (LCD_REDEFCHAR_LENGTH*2)
 
 /* ------- Structures ----------- */
 
-struct aggregate A2408 = { 8, ag_numbers, ag_aggregate, };
-struct filetype DS2408[] = {
+static struct aggregate A2408 = { 8, ag_numbers, ag_aggregate, };
+static struct aggregate A2408c = { 8, ag_numbers, ag_separate, };
+static struct filetype DS2408[] = {
 	F_STANDARD,
-	{"power", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_volatile, FS_power, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA,},
-	{"PIO", PROPERTY_LENGTH_BITFIELD, &A2408, ft_bitfield, fc_stable, FS_r_pio, FS_w_pio, VISIBLE, NO_FILETYPE_DATA,},
-	{"sensed", PROPERTY_LENGTH_BITFIELD, &A2408, ft_bitfield, fc_volatile, FS_sense, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA,},
-	{"latch", PROPERTY_LENGTH_BITFIELD, &A2408, ft_bitfield, fc_volatile, FS_r_latch, FS_w_latch, VISIBLE, NO_FILETYPE_DATA,},
-	{"strobe", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, FS_r_strobe, FS_w_strobe, VISIBLE, NO_FILETYPE_DATA,},
-	{"set_alarm", PROPERTY_LENGTH_UNSIGNED, NON_AGGREGATE, ft_unsigned, fc_stable, FS_r_s_alarm, FS_w_s_alarm, VISIBLE, NO_FILETYPE_DATA,},
-	{"por", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, FS_r_por, FS_w_por, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_M", PROPERTY_LENGTH_SUBDIR, NON_AGGREGATE, ft_subdir, fc_subdir, NO_READ_FUNCTION, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_M/clear", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, NO_READ_FUNCTION, FS_Mclear, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_M/home", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, NO_READ_FUNCTION, FS_Mhome, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_M/screen", PROPERTY_LENGTH_LCD_MESSAGE, NON_AGGREGATE, ft_ascii, fc_stable, NO_READ_FUNCTION, FS_Mscreen, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_M/message", PROPERTY_LENGTH_LCD_MESSAGE, NON_AGGREGATE, ft_ascii, fc_stable, NO_READ_FUNCTION, FS_Mmessage, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_H", PROPERTY_LENGTH_SUBDIR, NON_AGGREGATE, ft_subdir, fc_subdir, NO_READ_FUNCTION, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_H/clear", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, NO_READ_FUNCTION, FS_Hclear, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_H/home", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, NO_READ_FUNCTION, FS_Hhome, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_H/screen", PROPERTY_LENGTH_LCD_MESSAGE, NON_AGGREGATE, ft_ascii, fc_stable, NO_READ_FUNCTION, FS_Hscreen, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_H/screenyx", PROPERTY_LENGTH_LCD_MESSAGE, NON_AGGREGATE, ft_ascii, fc_stable, NO_READ_FUNCTION, FS_Hscreenyx, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_H/message", PROPERTY_LENGTH_LCD_MESSAGE, NON_AGGREGATE, ft_ascii, fc_stable, NO_READ_FUNCTION, FS_Hmessage, VISIBLE, NO_FILETYPE_DATA,},
-	{"LCD_H/onoff", PROPERTY_LENGTH_UNSIGNED, NON_AGGREGATE, ft_unsigned, fc_stable, NO_READ_FUNCTION, FS_Honoff, VISIBLE, NO_FILETYPE_DATA,},
+	{"power", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_volatile, FS_power, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA, },
+	{"out_of_testmode", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_volatile, NO_READ_FUNCTION,  FS_out_of_testmode, VISIBLE, NO_FILETYPE_DATA, },
+	{"PIO", PROPERTY_LENGTH_BITFIELD, &A2408, ft_bitfield, fc_stable, FS_r_pio, FS_w_pio, VISIBLE, NO_FILETYPE_DATA, },
+	{"sensed", PROPERTY_LENGTH_BITFIELD, &A2408, ft_bitfield, fc_volatile, FS_sense, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA, },
+	{"latch", PROPERTY_LENGTH_BITFIELD, &A2408, ft_bitfield, fc_volatile, FS_r_latch, FS_w_latch, VISIBLE, NO_FILETYPE_DATA, },
+	{"strobe", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, FS_r_strobe, FS_w_strobe, VISIBLE, NO_FILETYPE_DATA, },
+	{"set_alarm", PROPERTY_LENGTH_UNSIGNED, NON_AGGREGATE, ft_unsigned, fc_stable, FS_r_s_alarm, FS_w_s_alarm, VISIBLE, NO_FILETYPE_DATA, },
+	{"por", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, FS_r_por, FS_w_por, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_M", PROPERTY_LENGTH_SUBDIR, NON_AGGREGATE, ft_subdir, fc_subdir, NO_READ_FUNCTION, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_M/clear", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, NO_READ_FUNCTION, FS_Mclear, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_M/home", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, NO_READ_FUNCTION, FS_Mhome, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_M/screen", PROPERTY_LENGTH_LCD_MESSAGE, NON_AGGREGATE, ft_ascii, fc_stable, NO_READ_FUNCTION, FS_Mscreen, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_M/message", PROPERTY_LENGTH_LCD_MESSAGE, NON_AGGREGATE, ft_ascii, fc_stable, NO_READ_FUNCTION, FS_Mmessage, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_H", PROPERTY_LENGTH_SUBDIR, NON_AGGREGATE, ft_subdir, fc_subdir, NO_READ_FUNCTION, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_H/clear", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, NO_READ_FUNCTION, FS_Hclear, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_H/home", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_stable, NO_READ_FUNCTION, FS_Hhome, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_H/screen", PROPERTY_LENGTH_LCD_MESSAGE, NON_AGGREGATE, ft_ascii, fc_stable, NO_READ_FUNCTION, FS_Hscreen, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_H/screenyx", PROPERTY_LENGTH_LCD_MESSAGE, NON_AGGREGATE, ft_ascii, fc_stable, NO_READ_FUNCTION, FS_Hscreenyx, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_H/message", PROPERTY_LENGTH_LCD_MESSAGE, NON_AGGREGATE, ft_ascii, fc_stable, NO_READ_FUNCTION, FS_Hmessage, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_H/onoff", PROPERTY_LENGTH_UNSIGNED, NON_AGGREGATE, ft_unsigned, fc_stable, NO_READ_FUNCTION, FS_Honoff, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_H/redefchar", LCD_REDEFCHAR_LENGTH, &A2408c, ft_binary, fc_stable, NO_READ_FUNCTION, FS_redefchar, VISIBLE, NO_FILETYPE_DATA, },
+	{"LCD_H/redefchar_hex", LCD_REDEFCHAR_LENGTH, &A2408c, ft_binary, fc_stable, NO_READ_FUNCTION, FS_redefchar_hex, VISIBLE, NO_FILETYPE_DATA, },
 };
 
 DeviceEntryExtended(29, DS2408, DEV_alarm | DEV_resume | DEV_ovdr, NO_GENERIC_READ, NO_GENERIC_WRITE);
@@ -131,9 +139,18 @@ DeviceEntryExtended(29, DS2408, DEV_alarm | DEV_resume | DEV_ovdr, NO_GENERIC_RE
 Make_SlaveSpecificTag(INI, fc_stable);	// LCD screen initialized?
 
 /* Nibbles for LCD controller */
+/* From Klaus Der Tiger:
+ * Since at least 2.7p6 there has been a bug causing all three buttons of
+ * the Hobbyboards LCD module (using a DS2408) to malfunction. This is due
+ * to the fact, that the three lowest bits of the port register are set to
+ * 0 during initialization of the display module, turning the output
+ * transistors on and by that taking the sensing voltage away from the
+ * buttons.
+* */
 #define LCD_DATA_FLAG       0x08
-#define NIBBLE_ONE(x)       ((x)&0xF0)
-#define NIBBLE_TWO(x)       (((x)<<4)&0xF0)
+#define LCD_BUTTON_MASK		0x07
+#define NIBBLE_ONE(x)       ( ((x)&0xF0) | LCD_BUTTON_MASK )
+#define NIBBLE_TWO(x)       ( (((x)<<4)&0xF0) | LCD_BUTTON_MASK )
 #define NIBBLE_CTRL( x )    NIBBLE_ONE(x)               , NIBBLE_TWO(x)
 #define NIBBLE_DATA( x )    NIBBLE_ONE(x)|LCD_DATA_FLAG , NIBBLE_TWO(x)|LCD_DATA_FLAG
 
@@ -146,6 +163,8 @@ static GOOD_OR_BAD OW_w_pio(const BYTE data, const struct parsedname *pn);
 static GOOD_OR_BAD OW_r_reg(BYTE * data, const struct parsedname *pn);
 static GOOD_OR_BAD OW_w_s_alarm(const BYTE * data, const struct parsedname *pn);
 static GOOD_OR_BAD OW_w_pios(const BYTE * data, const size_t size, const struct parsedname *pn);
+static GOOD_OR_BAD OW_redefchar(ASCII * pattern, struct parsedname * pn);
+static GOOD_OR_BAD OW_out_of_test_mode( const struct parsedname * pn ) ;
 
 /* 2408 switch */
 /* 2408 switch -- is Vcc powered?*/
@@ -157,6 +176,14 @@ static ZERO_OR_ERROR FS_power(struct one_wire_query *owq)
 	return 0;
 }
 
+static ZERO_OR_ERROR FS_out_of_testmode(struct one_wire_query *owq)
+{
+	if ( OWQ_Y(owq) ) {
+		RETURN_ERROR_IF_BAD( OW_out_of_test_mode(PN(owq) ) );
+	}
+	return 0;
+}
+
 static ZERO_OR_ERROR FS_r_strobe(struct one_wire_query *owq)
 {
 	BYTE data[6];
@@ -291,7 +318,7 @@ static ZERO_OR_ERROR FS_r_s_alarm(struct one_wire_query *owq)
 	U = (d[5] & 0x03) * 100000000;
 	/* registers 0x8B and 0x8C */
 	for (i = 0, p = 1; i < 8; ++i, p *= 10) {
-		U += UT_getbit(&d[3], i) | (UT_getbit(&d[4], i) << 1) * p;
+		U += (UT_getbit(&d[4], i) | (UT_getbit(&d[3], i) << 1)) * p;
 	}
 	OWQ_U(owq) = U;
 	return 0;
@@ -304,7 +331,7 @@ static ZERO_OR_ERROR FS_r_s_alarm(struct one_wire_query *owq)
 /* data[0] selection  */
 static ZERO_OR_ERROR FS_w_s_alarm(struct one_wire_query *owq)
 {
-	BYTE data[3];
+	BYTE data[3] = { 0, 0, 0, }; // coverity likes this initialized
 	int i;
 	UINT p;
 	UINT U = OWQ_U(owq);
@@ -384,11 +411,11 @@ static GOOD_OR_BAD OW_Hinit(struct parsedname * pn)
 {
 	int init = 1;
 	// clear, display on, mode
-	BYTE start[] = { LCD_COMMAND_ATTENTION, };
+	BYTE start[] = { NIBBLE_ONE(LCD_COMMAND_ATTENTION), };
 	BYTE next[] = {
-		LCD_COMMAND_ATTENTION,
-		LCD_COMMAND_ATTENTION,
-		LCD_COMMAND_4_BIT,
+		NIBBLE_ONE(LCD_COMMAND_ATTENTION),
+		NIBBLE_ONE(LCD_COMMAND_ATTENTION),
+		NIBBLE_ONE(LCD_COMMAND_4_BIT),
 		NIBBLE_CTRL(LCD_COMMAND_4_BIT_2_LINES),
 	};
 	BYTE data[6];
@@ -583,6 +610,36 @@ static ZERO_OR_ERROR FS_Honoff(struct one_wire_query *owq)
 	return 0;
 }
 
+static ZERO_OR_ERROR FS_redefchar(struct one_wire_query *owq)
+{
+	struct parsedname *pn = PN(owq);
+	
+	if ( OWQ_size(owq) != LCD_REDEFCHAR_LENGTH ) {
+		return -ERANGE ;
+	}
+	if ( OWQ_offset(owq) != 0 ) {
+		return -ERANGE ;
+	}
+		
+	return GB_to_Z_OR_E( OW_redefchar( OWQ_buffer(owq), pn ) ) ;
+}
+
+static ZERO_OR_ERROR FS_redefchar_hex(struct one_wire_query *owq)
+{
+	struct parsedname *pn = PN(owq);
+	BYTE data[LCD_REDEFCHAR_LENGTH] ;
+	
+	if ( OWQ_size(owq) != LCD_REDEFCHAR_LENGTH_HEX ) {
+		return -ERANGE ;
+	}
+	if ( OWQ_offset(owq) != 0 ) {
+		return -ERANGE ;
+	}
+	string2bytes( OWQ_buffer(owq), data, LCD_REDEFCHAR_LENGTH ) ;
+		
+	return GB_to_Z_OR_E( OW_redefchar( (ASCII *) data, pn ) ) ;
+}
+
 /* Read 6 bytes --
    0x88 PIO logic State
    0x89 PIO output Latch state
@@ -620,7 +677,12 @@ static GOOD_OR_BAD OW_w_pio(const BYTE data, const struct parsedname *pn)
 		TRXN_END,
 	};
 
-	RETURN_BAD_IF_BAD(BUS_transaction(t, pn)) ;
+	if ( BAD(BUS_transaction(t, pn)) ) {
+		// may be in test mode, which causes Channel Access Write to fail
+		// fix now, but need another attempt to see if will work
+		OW_out_of_test_mode(pn) ;
+		return gbBAD ;
+	}
 
 	if (read_back[0] != 0xAA) {
 		return gbBAD;
@@ -653,7 +715,14 @@ static GOOD_OR_BAD OW_w_pios(const BYTE * data, const size_t size, const struct
 		formatted_data[formatted_data_index + 2] = 0xFF;
 		formatted_data[formatted_data_index + 3] = 0xFF;
 	}
-	RETURN_BAD_IF_BAD(BUS_transaction(t, pn)) ;
+	
+	if ( BAD(BUS_transaction(t, pn)) ) {
+		// may be in test mode, which causes Channel Access Write to fail
+		// fix now, but need another attempt to see if will work
+		OW_out_of_test_mode(pn) ;
+		return gbBAD ;
+	}
+	
 	for (i = 0; i < size; ++i) {
 		int formatted_data_index = 4 * i;
 		BYTE rdata = ((BYTE)~data[i]);  // get rid of warning: comparison of promoted ~unsigned with unsigned
@@ -747,3 +816,42 @@ static GOOD_OR_BAD OW_w_s_alarm(const BYTE * data, const struct parsedname *pn)
 	return (data[0] != new_register[3]) || (data[1] != new_register[4])
 		|| (control_value[0] != (new_register[5] & 0x0F)) ? gbBAD : gbGOOD;
 }
+
+// very strange command to get out of test mode.
+// Uses a different 1-wire command 
+static GOOD_OR_BAD OW_out_of_test_mode( const struct parsedname * pn )
+{
+	BYTE out_of_test[] = { 0x96, SNvar(pn->sn), 0x3C, } ;
+	struct transaction_log t[] = {
+		TRXN_RESET,
+		TRXN_WRITE(out_of_test, 1 + SERIAL_NUMBER_SIZE + 1 ),
+		TRXN_END,
+	};
+	return BUS_transaction( t, pn ) ;
+}	
+
+/* Redefine a character */
+static GOOD_OR_BAD OW_redefchar(ASCII * pattern, struct parsedname * pn)
+{
+	int i ;
+	int j = 0;
+	int char_num = pn->extension * 8 + 0x40 ;
+	int datalength = 2 * (LCD_REDEFCHAR_LENGTH+1) ; // nibbles for data plus index
+	BYTE data[ datalength ] ;
+
+	RETURN_BAD_IF_BAD( OW_Hinit(pn) ) ;
+
+	i = j = 0;
+	
+	// Start with char num
+	data[j++] = (char_num & 0xF0);
+	data[j++] = (char_num << 4) & 0xF0;
+
+	// Add pattern
+	for ( i = 0 ; i < LCD_REDEFCHAR_LENGTH ; ++i ) {
+		data[j++] = NIBBLE_ONE(pattern[i]) | LCD_DATA_FLAG;
+		data[j++] = NIBBLE_TWO(pattern[i]) | LCD_DATA_FLAG;
+	}
+
+	return OW_w_pios(data, datalength, pn) ;
+}
