@@ -1,5 +1,5 @@
 /*
-$Id: ow_add_inflight.c,v 1.9 2010/12/20 03:29:54 alfille Exp $
+$Id: ow_add_inflight.c,v 1.13 2012/10/01 14:16:36 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -17,27 +17,27 @@ $Id: ow_add_inflight.c,v 1.9 2010/12/20 03:29:54 alfille Exp $
 /* This allows adding new Bus Masters while program is running
  * The master is usually only read-locked for normal operation
  * write lock is done in a separate thread when no requests are being processed */
-void Add_InFlight( GOOD_OR_BAD (*nomatch)(struct connection_in * trial,struct connection_in * existing), struct connection_in * new_in )
+void Add_InFlight( GOOD_OR_BAD (*nomatch)(struct port_in * trial,struct port_in * existing), struct port_in * new_pin )
 {
-	if ( new_in == NO_CONNECTION ) {
+	if ( new_pin == NULL ) {
 		return ;
 	}
 
-	LEVEL_DEBUG("Request master be added: %s", SOC(new_in)->devicename);
+	LEVEL_DEBUG("Request master be added: %s", DEVICENAME(new_pin->first));
 
 	CONNIN_WLOCK ;
-	if ( nomatch != NO_CONNECTION ) {
-		struct connection_in * in ;
-		for ( in = Inbound_Control.head ; in != NO_CONNECTION ; in = in->next ) {
-			if ( GOOD( nomatch( new_in, in )) ) {
+	if ( nomatch != NULL ) {
+		struct port_in * pin ;
+		for ( pin = Inbound_Control.head_port ; pin != NULL ; pin = pin->next ) {
+			if ( GOOD( nomatch( new_pin, pin )) ) {
 				continue ;
 			}
-			LEVEL_DEBUG("Already exists as index=%d",in->index) ;
+			LEVEL_DEBUG("Already exists as index=%d",pin->first->index) ;
 			CONNIN_WUNLOCK ;
-			RemoveIn( new_in ) ;
+			RemovePort( new_pin ) ;
 			return ;
 		}
 	}
-	LinkIn(new_in);
+	LinkPort(new_pin);
 	CONNIN_WUNLOCK ;
 }
