@@ -1,5 +1,5 @@
 /*
-$Id: ow_avahi_browse.c,v 1.20 2010/11/19 02:05:30 alfille Exp $
+$Id: ow_avahi_browse.c,v 1.24 2013/01/21 21:57:16 alfille Exp $
 OWFS -- One-Wire filesystem
 OWHTTPD -- One-Wire Web Server
 Written 2003 Paul H Alfille
@@ -17,7 +17,7 @@ See the header file: ow.h for full attribution
 #include "stdlib.h"  // need this for NULL
 #endif /* HAVE_STDLIB_H */
 
-#if OW_ZERO && OW_MT && ( ! OW_CYGWIN )
+#if OW_ZERO && ! OW_CYGWIN && ! OW_DARWIN
 
 #include "ow_connection.h"
 
@@ -214,7 +214,7 @@ void * OW_Avahi_Browse(void * v)
 
 	/* Check whether creating the loop object succeeded */
 	if (in->master.browse.avahi_poll!=NULL) {
-		AvahiPoll * simple_poll = avahi_simple_poll_get(in->master.browse.avahi_poll) ;
+		const AvahiPoll * simple_poll = avahi_simple_poll_get(in->master.browse.avahi_poll) ;
 		
 		/* Create a new client */
 		in->master.browse.avahi_client = avahi_client_new( simple_poll, AVAHI_CLIENT_NO_FAIL, client_callback, (void *) in, &error );
@@ -255,9 +255,4 @@ void * OW_Avahi_Browse(void * v)
 	return VOID_RETURN;
 }
 
-#else /* OW_ZERO  && OW_MT && ! OW_CYGWIN */
-void * OW_Avahi_Browse(void * v) {
-	(void) v ;
-	return VOID_RETURN ;
-}
-#endif  /* OW_ZERO  && OW_MT && ! OW_CYGWIN */
+#endif  /* OW_ZERO && ! OW_CYGWIN && ! OW_DARWIN */
