@@ -1,9 +1,9 @@
 /*
-$Id: ow_badadapter.c,v 1.39 2010/12/20 03:29:54 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -23,10 +23,11 @@ static void BadAdapter_close(struct connection_in *in);
 /* Device-specific functions */
 /* Note, the "Bad"adapter" ha not function, and returns "-ENOTSUP" (not supported) for most functions */
 /* It does call lower level functions for higher ones, which of course is pointless since the lower ones don't work either */
-GOOD_OR_BAD BadAdapter_detect(struct connection_in *in)
+GOOD_OR_BAD BadAdapter_detect(struct port_in *pin)
 {
-	SOC(in)->type = ct_none ;
-	SOC(in)->file_descriptor = FILE_DESCRIPTOR_BAD ;
+	struct connection_in * in = pin->first ;
+	pin->type = ct_none ;
+	pin->file_descriptor = FILE_DESCRIPTOR_BAD ;
 	in->Adapter = adapter_Bad;	/* OWFS assigned value */
 	in->iroutines.reset = BadAdapter_reset;
 	in->iroutines.next_both = NO_NEXT_BOTH_ROUTINE;
@@ -36,13 +37,15 @@ GOOD_OR_BAD BadAdapter_detect(struct connection_in *in)
 	in->iroutines.sendback_bits = BadAdapter_sendback_bits;
 	in->iroutines.select = NO_SELECT_ROUTINE;
 	in->iroutines.select_and_sendback = NO_SELECTANDSENDBACK_ROUTINE;
+	in->iroutines.set_config = NO_SET_CONFIG_ROUTINE;
+	in->iroutines.get_config = NO_GET_CONFIG_ROUTINE;
 	in->iroutines.reconnect = NO_RECONNECT_ROUTINE;
 	in->iroutines.close = BadAdapter_close;
 	in->iroutines.flags = ADAP_FLAG_sham;
 	in->adapter_name = "Bad Adapter";
-	SAFEFREE( SOC(in)->devicename ) ;
-	SOC(in)->devicename = owstrdup("None") ;
-	in->busmode = bus_bad ;
+	SAFEFREE( DEVICENAME(in) ) ;
+	DEVICENAME(in) = owstrdup("None") ;
+	pin->busmode = bus_bad ;
 	return gbGOOD;
 }
 
