@@ -1,9 +1,9 @@
 /*
-$Id: ow_bus.c,v 1.102 2010/09/23 03:17:16 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -13,11 +13,11 @@ $Id: ow_bus.c,v 1.102 2010/09/23 03:17:16 alfille Exp $
 #include "owfs_config.h"
 #include "ow_connection.h"
 
-GOOD_OR_BAD BUS_detect( struct connection_in * in )
+GOOD_OR_BAD BUS_detect( struct port_in * pin )
 {
-	GOOD_OR_BAD (*detect) (struct connection_in * in) = in->iroutines.detect ;
+	GOOD_OR_BAD (*detect) (struct port_in * pin) = pin->first->iroutines.detect ;
 	if ( detect != NO_DETECT_ROUTINE ) {
-		return (detect)(in) ;
+		return (detect)(pin) ;
 	}
 	return gbBAD ;
 }
