@@ -1,9 +1,9 @@
 /*
-$Id: ow_byte.c,v 1.3 2010/09/24 21:04:24 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -24,7 +24,7 @@ struct one_wire_query * ALLtoBYTE(struct one_wire_query *owq_all)
 	}
 
 	for ( extension = 0 ; extension < elements ; ++extension ) {
-		UT_setbit( (BYTE *) &OWQ_U(owq_byte), extension, OWQ_array_Y(owq_all,extension) ) ;
+		UT_setbit_U( &OWQ_U(owq_byte), extension, OWQ_array_Y(owq_all,extension) ) ;
 	}
 	return owq_byte ;
 }	 
@@ -40,7 +40,7 @@ struct one_wire_query * BYTEtoALL(struct one_wire_query *owq_byte)
 	}
 
 	for ( extension = 0 ; extension < elements ; ++extension ) {
-		OWQ_array_Y(owq_all,extension) = UT_getbit( (BYTE *) &OWQ_U(owq_byte), extension ) ;
+		OWQ_array_Y(owq_all,extension) = UT_getbit_U( OWQ_U(owq_byte), extension ) ;
 	}
 	return owq_all ;
 }	 
