@@ -1,9 +1,8 @@
 /*
-$Id: ow_cache.c,v 1.138 2011/07/28 10:57:18 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -19,7 +18,6 @@ $Id: ow_cache.c,v 1.138 2011/07/28 10:57:18 alfille Exp $
 
 //#define CACHE_DEBUG
 
-#if OW_CACHE
 #include <limits.h>
 
 #define EXTENSION_INTERNAL  -2
@@ -44,10 +42,6 @@ void * Device_Marker = &DevMarkerLoc ;
 int AliasMarkerLoc ;
 void * Alias_Marker = &AliasMarkerLoc ;
 
-// Simultaneous Dir are bus-based
-// generic unique address for Simultaneous.
-int SimulMarkerLoc[simul_end] ;
-void * Simul_Marker[] = { &SimulMarkerLoc[simul_temp], &SimulMarkerLoc[simul_volt], } ;
 
 /* Put the globals into a struct to declutter the namespace */
 struct cache_data {
@@ -146,7 +140,7 @@ static enum cache_task_return Cache_Get_Common(void *data, size_t * dsize, time_
 static enum cache_task_return Cache_Get_Common_Dir(struct dirblob *db, time_t * duration, const struct tree_node *tn);
 static enum cache_task_return Cache_Get_Persistent(void *data, size_t * dsize, time_t * duration, const struct tree_node *tn);
 
-static GOOD_OR_BAD Cache_Get_Simultaneous(enum simul_type type, struct one_wire_query *owq) ;
+static GOOD_OR_BAD Cache_Get_Simultaneous(const struct internal_prop *ip, struct one_wire_query *owq) ;
 static GOOD_OR_BAD Cache_Get_Internal(void *data, size_t * dsize, const struct internal_prop *ip, const struct parsedname *pn);
 static GOOD_OR_BAD Cache_Get_Strict(void *data, size_t dsize, const struct parsedname *pn);
 
@@ -451,7 +445,7 @@ GOOD_OR_BAD Cache_Add_Dir(const struct dirblob *db, const struct parsedname *pn)
 		return gbGOOD;				// do check here to avoid needless processing
 	}
 	
-	switch ( pn->selected_connection->busmode ) {
+	switch ( get_busmode(pn->selected_connection) ) {
 		case bus_fake:
 		case bus_tester:
 		case bus_mock:
@@ -496,27 +490,18 @@ GOOD_OR_BAD Cache_Add_Dir(const struct dirblob *db, const struct parsedname *pn)
 
 /* Add a Simultaneous entry to the cache */
 /* return 0 if good, 1 if not */
-GOOD_OR_BAD Cache_Add_Simul(const enum simul_type type, const struct parsedname *pn)
+GOOD_OR_BAD Cache_Add_Simul(const struct internal_prop *ip, const struct parsedname *pn)
 {
 	// Note: pn already points to directory
-	time_t duration = TimeOut(fc_volatile);
+	time_t duration = TimeOut(ip->change);
 	struct tree_node *tn;
 
 	if (pn==NO_PARSEDNAME || pn->selected_connection==NO_CONNECTION) {
 		return gbGOOD;				// do check here to avoid needless processing
 	}
 	
-	switch ( pn->selected_connection->busmode ) {
-		case bus_fake:
-		case bus_tester:
-		case bus_mock:
-		case bus_bad:
-		case bus_unknown:
-			return gbGOOD ;
-		default:
-			break ;
-	}
-	
+	// Already removed inappropriate busmodes (like fake and usb_monitor)
+	// Now test requested time
 	if (duration <= 0) {
 		return gbGOOD;				/* in case timeout set to 0 */
 	}
@@ -531,8 +516,8 @@ GOOD_OR_BAD Cache_Add_Simul(const enum simul_type type, const struct parsedname
 	LEVEL_DEBUG(SNformat, SNvar(pn->sn));
 	
 	// populate node with directory name and dirblob
-	LoadTK( pn->sn, Simul_Marker[type], pn->selected_connection->index, tn) ;
-	LEVEL_DEBUG("Simultaneous add type=%d",type);
+	LoadTK( pn->sn, ip->name, 0, tn) ;
+	LEVEL_DEBUG("Simultaneous add type=%s",ip->name);
 	tn->expires = duration + NOW_TIME;
 	tn->dsize = 0;
 	return Add_Stat(&cache_dir, Cache_Add_Common(tn));
@@ -771,9 +756,9 @@ GOOD_OR_BAD OWQ_Cache_Get(struct one_wire_query *owq)
 
 	switch (pn->selected_filetype->change) {
 	case fc_simultaneous_temperature:
-		return Cache_Get_Simultaneous(simul_temp, owq) ;
+		return Cache_Get_Simultaneous(SlaveSpecificTag(S_T), owq) ;
 	case fc_simultaneous_voltage:
-		return Cache_Get_Simultaneous(simul_volt, owq) ;
+		return Cache_Get_Simultaneous(SlaveSpecificTag(S_T), owq) ;
 	default:
 		break ;
 	}
@@ -968,16 +953,15 @@ If the property isn't independently cached, return false (1)
 If the simultaneous conversion is more recent, return false (1)
 Else return the cached value and true (0)
 */
-GOOD_OR_BAD Cache_Get_Simul_Time(enum simul_type type, time_t * dwell_time, const struct parsedname * pn)
+GOOD_OR_BAD Cache_Get_Simul_Time(const struct internal_prop *ip, time_t * dwell_time, const struct parsedname * pn)
 {
 	// valid cached primary data -- see if a simultaneous conversion should be used instead
 	struct tree_node tn;
 	time_t duration ;
-	time_t time_left ;
 	size_t dsize_simul = 0 ;
 	struct parsedname pn_directory ;
 
-	duration = TimeOut(ipSimul[type].change); // time allocated this conversion
+	duration = TimeOut(ip->change); // time allocated this conversion
 	if ( duration <= 0) {
 		// uncachable
 		return gbBAD;
@@ -986,14 +970,14 @@ GOOD_OR_BAD Cache_Get_Simul_Time(enum simul_type type, time_t * dwell_time, cons
 	LEVEL_DEBUG("Looking for conversion time "SNformat, SNvar(pn->sn));
 	
 	FS_LoadDirectoryOnly(&pn_directory, pn);
-	LoadTK(pn_directory.sn, Simul_Marker[type], pn->selected_connection->index, &tn ) ;
-	if ( Get_Stat(&cache_int, Cache_Get_Common(NULL, &dsize_simul, &time_left, &tn)) ) {
+	LoadTK(pn_directory.sn, ip->name, 0, &tn ) ;
+	if ( Get_Stat(&cache_int, Cache_Get_Common(NULL, &dsize_simul, &duration, &tn)) ) {
 		return gbBAD ;
 	}
 	// duration_simul is time left
 	// duration is time allocated
 	// back-compute dwell time
-	dwell_time[0] = duration - time_left ;
+	dwell_time[0] = TimeOut(ip->change) - duration ;
 	return gbGOOD ;
 }
 
@@ -1001,7 +985,7 @@ GOOD_OR_BAD Cache_Get_Simul_Time(enum simul_type type, time_t * dwell_time, cons
  * return true if simultaneous is the prefered method
  * bad if no simultaneous, or it's not the best
 */
-static GOOD_OR_BAD Cache_Get_Simultaneous(enum simul_type type, struct one_wire_query *owq)
+static GOOD_OR_BAD Cache_Get_Simultaneous(const struct internal_prop *ip, struct one_wire_query *owq)
 {
 	struct tree_node tn;
 	time_t duration ;
@@ -1010,7 +994,7 @@ static GOOD_OR_BAD Cache_Get_Simultaneous(enum simul_type type, struct one_wire_
 	struct parsedname * pn = PN(owq) ;
 	size_t dsize = sizeof(union value_object) ;
 	
-	duration = TimeOut(pn->selected_filetype->change);
+	time_left = duration = TimeOut(pn->selected_filetype->change);
 	if (duration <= 0) {
 		// probably "uncached" requested
 		return gbBAD;
@@ -1022,7 +1006,7 @@ static GOOD_OR_BAD Cache_Get_Simultaneous(enum simul_type type, struct one_wire_
 		// valid cached primary data -- see if a simultaneous conversion should be used instead
 		time_t dwell_time_data = duration - time_left ;
 		
-		if ( BAD( Cache_Get_Simul_Time( type, &dwell_time_simul, pn)) ) {
+		if ( BAD( Cache_Get_Simul_Time( ip, &dwell_time_simul, pn)) ) {
 			// Simul not found or timed out
 			LEVEL_DEBUG("Simultaneous conversion not found.") ;
 			OWQ_SIMUL_CLR(owq) ;
@@ -1038,7 +1022,7 @@ static GOOD_OR_BAD Cache_Get_Simultaneous(enum simul_type type, struct one_wire_
 		return gbGOOD ;
 	}
 	// fall through -- no cached primary data
-	if ( BAD( Cache_Get_Simul_Time( type, &dwell_time_simul, pn)) ) {
+	if ( BAD( Cache_Get_Simul_Time( ip, &dwell_time_simul, pn)) ) {
 		// no simultaneous either
 		OWQ_SIMUL_CLR(owq) ;
 		return gbBAD ;
@@ -1072,8 +1056,11 @@ ASCII * Cache_Get_Alias(const BYTE * sn)
 	return alias_name ;
 }
 
-/* Look in caches, 0=found and valid, 1=not or uncachable in the first place */
+/* Look in caches */
 /* duration is time left */
+/* inputs: dsize, duration, tn
+ * outputs: return value, data, dsize (updated), duration (updated)
+ * */
 static enum cache_task_return Cache_Get_Common(void *data, size_t * dsize, time_t * duration, const struct tree_node *tn)
 {
 	enum cache_task_return ctr_ret;
@@ -1320,13 +1307,13 @@ void Cache_Del_Dir(const struct parsedname *pn)
 	Del_Stat(&cache_dir, Cache_Del_Common(&tn));
 }
 
-void Cache_Del_Simul(enum simul_type type, const struct parsedname *pn)
+void Cache_Del_Simul(const struct internal_prop *ip, const struct parsedname *pn)
 {
 	struct tree_node tn;
 	struct parsedname pn_directory;
 	
 	FS_LoadDirectoryOnly(&pn_directory, pn);
-	LoadTK(pn_directory.sn, Simul_Marker[type], pn->selected_connection->index, &tn );
+	LoadTK(pn_directory.sn, ip->name, 0, &tn );
 	Del_Stat(&cache_dir, Cache_Del_Common(&tn));
 }
 
@@ -1499,7 +1486,6 @@ void Cache_Add_Alias_Bus(const ASCII * alias_name, INDEX_OR_ERROR bus)
 
 /* Add an item to the alias cache */
 /* retire the cache (flip) if too old, and start a new one (keep the old one for a while) */
-/* return 0 if good, 1 if not */
 static void Cache_Add_Alias_Common(struct alias_tree_node *atn)
 {
 	struct tree_opaque *opaque;
@@ -1703,6 +1689,3 @@ void Cache_Del_Alias_Bus(const ASCII * alias_name)
 	LEVEL_DEBUG("Hide %s",alias_name) ;
 	Cache_Add_Alias_Bus( alias_name, INDEX_BAD ) ;
 }
-
-
-#endif							/* OW_CACHE */
