@@ -1,9 +1,9 @@
 /*
-$Id: ow_com.c,v 1.54 2011/11/26 00:57:15 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-	email: palfille@earthlink.net
+	email: paul.alfille@gmail.com
 	Released under the GPL
 	See the header file: ow.h for full attribution
 	1wire/iButton system from Dallas Semiconductor
@@ -25,7 +25,7 @@ GOOD_OR_BAD COM_test( struct connection_in * connection )
 		return gbBAD;
 	}
 
-	switch ( SOC(connection)->type ) {
+	switch ( connection->pown->type ) {
 		case ct_unknown:
 		case ct_none:
 			LEVEL_DEBUG("ERROR!!! ----------- ERROR!");
@@ -42,9 +42,9 @@ GOOD_OR_BAD COM_test( struct connection_in * connection )
 			break ;
 	}
 
-	if ( SOC(connection)->state == cs_virgin ) {
-		LEVEL_DEBUG("Auto initialization of %s", SAFESTRING(SOC(connection)->devicename)) ;
-	} else if ( FILE_DESCRIPTOR_VALID( SOC(connection)->file_descriptor ) ) {
+	if ( connection->pown->state == cs_virgin ) {
+		LEVEL_DEBUG("Auto initialization of %s", SAFESTRING(DEVICENAME(connection))) ;
+	} else if ( FILE_DESCRIPTOR_VALID( connection->pown->file_descriptor ) ) {
 		return gbGOOD ;
 	}
 	return COM_open(connection) ;
@@ -57,7 +57,7 @@ void COM_flush( const struct connection_in *connection)
 		return ;
 	}
 
-	switch ( SOC(connection)->type ) {
+	switch ( connection->pown->type ) {
 		case ct_unknown:
 		case ct_none:
 			LEVEL_DEBUG("ERROR!!! ----------- ERROR!");
@@ -65,14 +65,14 @@ void COM_flush( const struct connection_in *connection)
 		case ct_netlink:
 		case ct_telnet:
 		case ct_tcp:
-			tcp_read_flush( SOC(connection)->file_descriptor) ;
+			tcp_read_flush( connection->pown->file_descriptor) ;
 			break ;
 		case ct_i2c:
 		case ct_usb:
 			LEVEL_DEBUG("Unimplemented!!!");
 			return ;
 		case ct_serial:
-			tcflush( SOC(connection)->file_descriptor, TCIOFLUSH);
+			tcflush( connection->pown->file_descriptor, TCIOFLUSH);
 			break ;
 	}
 }
@@ -88,7 +88,7 @@ void COM_break(struct connection_in *in)
 		return ;
 	}
 
-	switch ( SOC(in)->type ) {
+	switch ( in->pown->type ) {
 		case ct_unknown:
 		case ct_none:
 			LEVEL_DEBUG("ERROR!!! ----------- ERROR!");
@@ -103,7 +103,7 @@ void COM_break(struct connection_in *in)
 			LEVEL_DEBUG("Unimplemented!!!");
 			return ;
 		case ct_serial:
-			tcsendbreak(SOC(in)->file_descriptor, 0);
+			tcsendbreak(in->pown->file_descriptor, 0);
 			break ;
 	}
 }
