@@ -1,5 +1,5 @@
 /*
-$Id: ow_com_close.c,v 1.2 2011/01/20 02:14:47 alfille Exp $
+$Id: ow_com_close.c,v 1.7 2012/09/28 20:43:34 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -20,12 +20,15 @@ $Id: ow_com_close.c,v 1.2 2011/01/20 02:14:47 alfille Exp $
 
 void COM_close(struct connection_in *connection)
 {
+	struct port_in * pin ;
+	
 	if (connection == NO_CONNECTION) {
 		LEVEL_DEBUG("Attempt to close a NULL device");
 		return ;
 	}
+	pin = connection->pown ;
 
-	switch ( SOC(connection)->type ) {
+	switch ( pin->type ) {
 		case ct_unknown:
 		case ct_none:
 		case ct_usb:
@@ -42,12 +45,12 @@ void COM_close(struct connection_in *connection)
 			break ;
 	}
 
-	switch ( SOC(connection)->state ) {
+	switch ( pin->state ) {
 		case cs_virgin:
 			break ;
 		default:
 		case cs_deflowered:
-			Test_and_Close( &( SOC(connection)->file_descriptor) ) ;
+			Test_and_Close( &( pin->file_descriptor) ) ;
 			break ;
 	}
 }
