@@ -1,9 +1,9 @@
 /*
-$Id: ow_com_free.c,v 1.1 2010/12/22 02:27:09 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-	email: palfille@earthlink.net
+	email: paul.alfille@gmail.com
 	Released under the GPL
 	See the header file: ow.h for full attribution
 	1wire/iButton system from Dallas Semiconductor
@@ -25,7 +25,11 @@ void COM_free(struct connection_in *connection)
 		return ;
 	}
 
-	switch ( SOC(connection)->type ) {
+	if ( connection->pown->state == cs_virgin ) {
+		return ;
+	}
+
+	switch ( connection->pown->type ) {
 		case ct_unknown:
 		case ct_none:
 		case ct_usb:
@@ -42,6 +46,5 @@ void COM_free(struct connection_in *connection)
 			break ;
 	}
 
-	SAFEFREE(SOC(connection)->devicename) ;
-	SOC(connection)->state = cs_virgin ;
+	connection->pown->state = cs_virgin ;
 }
