@@ -1,9 +1,9 @@
 /*
-$Id: ow_com_read.c,v 1.22 2011/05/15 23:30:25 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-	email: palfille@earthlink.net
+	email: paul.alfille@gmail.com
 	Released under the GPL
 	See the header file: ow.h for full attribution
 	1wire/iButton system from Dallas Semiconductor
@@ -18,10 +18,15 @@ $Id: ow_com_read.c,v 1.22 2011/05/15 23:30:25 alfille Exp $
 #include <linux/limits.h>
 #endif
 
-SIZE_OR_ERROR COM_read_size_low( BYTE * data, size_t length, struct connection_in *connection ) ;
+static SIZE_OR_ERROR COM_read_get_size( BYTE * data, size_t length, struct connection_in *connection ) ;
 
+// read length bytes.
+// return BAD if any error or timeout before full length
+/* Called on head of multibus group */
 GOOD_OR_BAD COM_read( BYTE * data, size_t length, struct connection_in *connection)
 {
+	struct port_in * pin ;
+	
 	if ( length == 0 ) {
 		return gbGOOD ;
 	}
@@ -30,15 +35,16 @@ GOOD_OR_BAD COM_read( BYTE * data, size_t length, struct connection_in *connecti
 		// bad parameters
 		return gbBAD ;
 	}
+	pin = connection->pown ;
 
 	// unlike write or open, a closed connection isn't automatically opened.
 	// the reason is that reopening won't have the data waiting. We really need
 	// to restart the transaction from the "write" portion
-	if ( FILE_DESCRIPTOR_NOT_VALID( SOC(connection)->file_descriptor ) ) {
+	if ( FILE_DESCRIPTOR_NOT_VALID( pin->file_descriptor ) ) {
 		return gbBAD ;
 	}
 
-	switch ( SOC(connection)->type ) {
+	switch ( pin->type ) {
 		// test the type of connection
 		case ct_unknown:
 		case ct_none:
@@ -48,7 +54,7 @@ GOOD_OR_BAD COM_read( BYTE * data, size_t length, struct connection_in *connecti
 			return telnet_read( data, length, connection ) ;
 		case ct_tcp:
 			// network is ok
-			return COM_read_size_low( data, length, connection ) < 0 ? gbBAD : gbGOOD ;
+			return COM_read_get_size( data, length, connection ) == (ssize_t) length ? gbGOOD : gbBAD ;
 		case ct_i2c:
 		case ct_netlink:
 		case ct_usb:
@@ -56,13 +62,13 @@ GOOD_OR_BAD COM_read( BYTE * data, size_t length, struct connection_in *connecti
 			break ; 
 		case ct_serial:
 		// serial is ok
-		// printf("Serial read fd=%d length=%d\n",SOC(connection)->file_descriptor, (int) length);
+		// printf("Serial read fd=%d length=%d\n",pin->file_descriptor, (int) length);
 		{
-			ssize_t actual = COM_read_size_low( data, length, connection ) ;
-			if ( FILE_DESCRIPTOR_VALID( SOC(connection)->file_descriptor ) ) {
+			ssize_t actual = COM_read_get_size( data, length, connection ) ;
+			if ( FILE_DESCRIPTOR_VALID( pin->file_descriptor ) ) {
 				// tcdrain only works on serial conections
-				tcdrain( SOC(connection)->file_descriptor );
-				return actual < 0 ? gbBAD : gbGOOD ;
+				tcdrain( pin->file_descriptor );
+				return actual == (ssize_t) length ? gbGOOD : gbBAD ;
 			}
 			break ;
 		}
@@ -70,8 +76,13 @@ GOOD_OR_BAD COM_read( BYTE * data, size_t length, struct connection_in *connecti
 	return gbBAD ;
 }
 
-SIZE_OR_ERROR COM_read_size( BYTE * data, size_t length, struct connection_in *connection)
+// try to read bytes and return number actually read.
+// Timeout ok
+/* Called on head of multibus group */
+SIZE_OR_ERROR COM_read_with_timeout( BYTE * data, size_t length, struct connection_in *connection)
 {
+	struct port_in * pin ;
+	
 	if ( length == 0 ) {
 		return 0 ;
 	}
@@ -80,23 +91,36 @@ SIZE_OR_ERROR COM_read_size( BYTE * data, size_t length, struct connection_in *c
 		// bad parameters
 		return -EIO ;
 	}
+	pin = connection->pown ;
 
 	// unlike write or open, a closed connection isn't automatically opened.
 	// the reason is that reopening won't have the data waiting. We really need
 	// to restart the transaction from the "write" portion
-	if ( FILE_DESCRIPTOR_NOT_VALID( SOC(connection)->file_descriptor ) ) {
+	if ( FILE_DESCRIPTOR_NOT_VALID( pin->file_descriptor ) ) {
 		return -EBADF ;
-	}
+	} else {
+		size_t actual_size ;
+		ZERO_OR_ERROR zoe = tcp_read( pin->file_descriptor, data, length, &(pin->timeout), &actual_size ) ;
 
-	return COM_read_size_low( data, length, connection ) ;
+		if ( zoe == -EBADF ) {
+			COM_close(connection) ;
+			return zoe ;
+		} else {
+			return actual_size ;
+		}
+	}
 }
 
-SIZE_OR_ERROR COM_read_size_low( BYTE * data, size_t length, struct connection_in *connection )
+// Read only if no errors
+// Returns actual read size
+/* Called on head of multibus group */
+static SIZE_OR_ERROR COM_read_get_size( BYTE * data, size_t length, struct connection_in *connection )
 {
 	size_t actual_size ;
-	ZERO_OR_ERROR zoe = tcp_read( SOC(connection)->file_descriptor, data, length, &(SOC(connection)->timeout), &actual_size ) ;
+	struct port_in * pin = connection->pown ;
+	ZERO_OR_ERROR zoe = tcp_read( pin->file_descriptor, data, length, &(pin->timeout), &actual_size ) ;
 
-	if ( zoe == -EBADF ) {
+	if ( zoe < 0 ) {
 		COM_close(connection) ;
 		return zoe ;
 	} else {
