@@ -1,9 +1,9 @@
 /*
-$Id: ow_connect.c,v 1.141 2012/04/11 01:24:35 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -14,6 +14,8 @@ $Id: ow_connect.c,v 1.141 2012/04/11 01:24:35 alfille Exp $
 #include "ow.h"
 #include "ow_connection.h"
 
+static struct connection_in *AllocIn(const struct connection_in *old_in) ;
+
 /* Routines for handling a linked list of connections in*/
 /* typical connection in would be the serial port or USB */
 
@@ -21,7 +23,7 @@ $Id: ow_connect.c,v 1.141 2012/04/11 01:24:35 alfille Exp $
 struct inbound_control Inbound_Control = {
 	.active = 0,
 	.next_index = 0,
-	.head = NO_CONNECTION,
+	.head_port = NULL,
 	.next_fake = 0,
 	.next_tester = 0,
 	.next_mock = 0,
@@ -31,12 +33,14 @@ struct inbound_control Inbound_Control = {
 
 struct connection_in *find_connection_in(int bus_number)
 {
-	struct connection_in *c_in;
-	// step through inbound linked list
-
-	for (c_in = Inbound_Control.head; c_in != NO_CONNECTION; c_in = c_in->next) {
-		if (c_in->index == bus_number) {
-			return c_in;
+	struct port_in * pin ;
+	
+	for ( pin = Inbound_Control.head_port ; pin != NULL ; pin = pin->next ) {
+		struct connection_in *cin;
+		for ( cin = pin->first; cin != NO_CONNECTION; cin = cin->next) {
+			if (cin->index == bus_number) {
+				return cin;
+			}
 		}
 	}
 	LEVEL_DEBUG("Couldn't find bus number %d",bus_number);
@@ -55,12 +59,12 @@ int SetKnownBus( int bus_number, struct parsedname * pn)
 	return 0 ;
 }
 
-enum bus_mode get_busmode(struct connection_in *in)
+enum bus_mode get_busmode(const struct connection_in *in)
 {
 	if (in == NO_CONNECTION) {
 		return bus_unknown;
 	}
-	return in->busmode;
+	return in->pown->busmode;
 }
 
 // return true (non-zero) if the connection_in exists, and is remote
@@ -69,12 +73,18 @@ int BusIsServer(struct connection_in *in)
 	if ( in == NO_CONNECTION ) {
 		return 0 ;
 	}
-	return (in->busmode == bus_server) || (in->busmode == bus_zero);
+	switch ( get_busmode(in) ) {
+		case bus_server:
+		case bus_zero:
+			return 1 ;
+		default:
+			return 0 ;
+	}
 }
 
 /* Make a new connection_in entry, but DON'T place it in the chain (yet)*/
 /* Based on a shallow copy of "in" if not NULL */
-struct connection_in *AllocIn(const struct connection_in *old_in)
+static struct connection_in *AllocIn(const struct connection_in *old_in)
 {
 	size_t len = sizeof(struct connection_in);
 	struct connection_in *new_in = (struct connection_in *) owmalloc(len);
@@ -82,23 +92,16 @@ struct connection_in *AllocIn(const struct connection_in *old_in)
 		if (old_in == NO_CONNECTION) {
 			// Not a copy
 			memset(new_in, 0, len);
-			SOC(new_in)->file_descriptor = FILE_DESCRIPTOR_BAD ;
-			SOC(new_in)->type = ct_unknown ;
-			SOC(new_in)->state = cs_virgin ;
-			SOC(new_in)->devicename = NULL ;
-			/* Indirection to head of list for multichannel devices */
-			OW_channel_init(new_in) ;
+			DEVICENAME(new_in) = NULL ; 
 			/* Not yet a valid bus */
 			new_in->iroutines.flags = ADAP_FLAG_sham ;
 		} else {
 			// Copy of prior bus
 			memcpy(new_in, old_in, len);
-			if ( SOC(new_in)->devicename != NULL ) {
+			if ( DEVICENAME(new_in) != NULL ) {
 				// Don't make the name point to the same string, make a copy
-				SOC(new_in)->devicename = owstrdup( SOC(old_in)->devicename ) ;
+				DEVICENAME(new_in) = owstrdup( DEVICENAME(old_in) ) ;
 			}
-			/* Indirection to head of list for multichannel devices */
-			new_in->channel_info.head = old_in ;
 		}
 		
 		/* not yet linked */
@@ -114,83 +117,201 @@ struct connection_in *AllocIn(const struct connection_in *old_in)
 		/* Arbitrary guess at root directory size for allocating cache blob */
 		new_in->last_root_devs = 10;
 		new_in->AnyDevices = anydevices_unknown ;
+
+		++Inbound_Control.active ;
+		new_in->index = Inbound_Control.next_index++;
+		_MUTEX_INIT(new_in->bus_mutex);
+		_MUTEX_INIT(new_in->dev_mutex);
+		new_in->dev_db = NULL;
 	} else {
 		LEVEL_DEFAULT("Cannot allocate memory for bus master structure");
 	}
 	return new_in;
 }
 
-/* Place a new connection in the chain */
-struct connection_in *LinkIn(struct connection_in *now)
+struct port_in * AllocPort( const struct port_in * old_pin )
 {
-	if (now) {
+	size_t len = sizeof(struct port_in);
+	struct port_in *new_pin = (struct port_in *) owmalloc(len);
+	if ( new_pin != NULL ) {
+		if (old_pin == NULL) {
+			// Not a copy
+			memset(new_pin, 0, len);
+			new_pin->first = AllocIn(NO_CONNECTION) ;
+		} else {
+			// Copy of prior bus
+			memcpy(new_pin, old_pin, len);
+			new_pin->first = AllocIn(old_pin->first) ;
+			if ( old_pin->init_data != NULL ) {
+				new_pin->init_data = owstrdup( old_pin->init_data ) ;
+			}		
+		}
+		
+		// never copy file_descriptor (it's unique to port)
+		new_pin->file_descriptor = FILE_DESCRIPTOR_BAD ;
+		new_pin->type = ct_unknown ;
+		new_pin->state = cs_virgin ;
+
+		if ( new_pin->first == NO_CONNECTION ) {
+			LEVEL_DEBUG("Port creation incomplete");
+			owfree(new_pin) ;
+			return NULL ;
+		}
+		
+		new_pin->connections = 1 ;
+		new_pin->first->channel = 0 ;
+		
+		/* port not yet linked */
+		new_pin->next = NULL ;
+		/* connnection_in needs to be linked */
+		new_pin->first->pown = new_pin ;
+
+	} else {
+		LEVEL_DEFAULT("Cannot allocate memory for port master structure");
+	}
+	return new_pin;
+}
+
+/* Place a new connection in the chain */
+struct port_in *LinkPort(struct port_in *pin)
+{
+	if (pin != NULL) {
 		// Housekeeping to place in linked list
 		// Locking done at a higher level
-		now->next = Inbound_Control.head;	/* put in linked list at start */
-		Inbound_Control.head = now;
-		now->index = Inbound_Control.next_index++;
-		++Inbound_Control.active ;
+		pin->next = Inbound_Control.head_port;	/* put in linked list at start */
+		Inbound_Control.head_port = pin ;
 
-		_MUTEX_INIT(now->bus_mutex);
-		_MUTEX_INIT(now->dev_mutex);
-		now->dev_db = NULL;
+		_MUTEX_INIT(pin->port_mutex);
 	}
-	return now;
+	return pin;
 }
 
 /* Create a new inbound structure and place it in the chain */
-struct connection_in *NewIn(const struct connection_in *in) {
-	return LinkIn( AllocIn(in) ) ;
+struct port_in *NewPort(const struct port_in *pin) 
+{
+	return LinkPort( AllocPort(pin) ) ;
 }
 
 /* Free all connection_in in reverse order (Added to head on creation, head-first deletion) */
 void FreeInAll( void )
 {
-	while ( Inbound_Control.head ) {
-		RemoveIn(Inbound_Control.head);
+	while ( Inbound_Control.head_port ) {
+		RemovePort(Inbound_Control.head_port);
 	}
 }
 
 void RemoveIn( struct connection_in * conn )
 {
+	struct port_in * pin ;
+	
 	/* NULL safe */
 	if ( conn == NO_CONNECTION ) {
 		return ;
 	}
 
+	pin = conn->pown ;
+
 	/* First unlink from list */
-	if ( Inbound_Control.head == conn ) {
+	if ( pin == NULL ) {
+		// free-floating
+	} else if ( pin->first == conn ) {
 		/* Head of list, easy */
-		Inbound_Control.head = conn->next ;
+		pin->first = conn->next ;
+		-- pin->connections ;
 		--Inbound_Control.active ;
 	} else {
 		struct connection_in * now ;
 		/* find in list and splice out */
-		for ( now = Inbound_Control.head ; now != NO_CONNECTION ; now = now->next ) {
+		for ( now = pin->first ; now != NO_CONNECTION ; now = now->next ) {
 			/* Works even if not linked, since won't match and will just fall through */
 			if ( now->next == conn ) {
 				now->next = conn->next ;
+				-- pin->connections ;
 				--Inbound_Control.active ;
 				break ;
 			}
 		}
 	}
 
-	/* Now free up thread-sync resources */
-	if ( conn->next != NO_CONNECTION ) {
-		/* Only if actually linked in and possibly active */
-		_MUTEX_DESTROY(conn->bus_mutex);
-		_MUTEX_DESTROY(conn->dev_mutex);
-		SAFETDESTROY( conn->dev_db, owfree_func);
+	if ( conn->index == Inbound_Control.next_index-1 ) {
+		Inbound_Control.next_index-- ;
 	}
 
+	/* Now free up thread-sync resources */
+	_MUTEX_DESTROY(conn->bus_mutex);
+	_MUTEX_DESTROY(conn->dev_mutex);
+	SAFETDESTROY( conn->dev_db, owfree_func);
+
 	/* Close master-specific resources */
 	BUS_close(conn) ;
 
 	/* Next free up internal resources */
-	COM_free( conn ) ;
+	SAFEFREE( DEVICENAME(conn) ) ;
 	
 	/* Finally delete the structure */
 	owfree(conn);
 	conn = NO_CONNECTION ;
 }
+
+void RemovePort( struct port_in * pin )
+{
+	/* NULL safe */
+	if ( pin == NULL ) {
+		return ;
+	}
+
+	/* Free port */
+	COM_free( pin->first ) ;
+
+	/* First delete connections */
+	while ( pin->first != NO_CONNECTION ) {
+		RemoveIn( pin->first ) ;
+	}
+
+	/* Next unlink from list */
+	if ( pin == Inbound_Control.head_port ) {
+		/* Head of list, easy */
+		Inbound_Control.head_port = pin->next ;
+	} else {
+		struct port_in * now ;
+		/* find in list and splice out */
+		for ( now = Inbound_Control.head_port ; now != NULL ; now = now->next ) {
+			/* Works even if not linked, since won't match and will just fall through */
+			if ( now->next == pin ) {
+				now->next = pin->next ;
+				break ;
+			}
+		}
+	}
+
+	/* Now free up thread-sync resources */
+	/* Only if actually linked in and possibly active */
+	_MUTEX_DESTROY(pin->port_mutex);
+	SAFEFREE(pin->init_data) ;
+
+	/* Finally delete the structure */
+	owfree(pin);
+	pin = NULL ;
+}
+
+struct connection_in * AddtoPort( struct port_in * pin )
+{
+	struct connection_in * add_in = AllocIn( pin->first ) ;
+	
+	if ( add_in == NO_CONNECTION ) {
+		// cannot allocate
+		return NO_CONNECTION ;
+	}
+
+	// Housekeeping to place in linked list
+	// Locking done at a higher level
+	
+	add_in->next = pin->first;	/* put in linked list at start */
+	pin->first = add_in;
+	add_in->pown = pin ;
+
+	add_in->channel = pin->connections ;
+	++pin->connections ;
+
+	return add_in ;
+}
