@@ -1,9 +1,8 @@
 /*
-$Id: ow_dir.c,v 1.209 2012/04/14 03:52:46 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -27,7 +26,7 @@ static ZERO_OR_ERROR FS_structdevdir(void (*dirfunc) (void *, const struct parse
 static ZERO_OR_ERROR FS_alarmdir(void (*dirfunc) (void *, const struct parsedname * const), void *v, const struct parsedname *pn2);
 static ZERO_OR_ERROR FS_typedir(void (*dirfunc) (void *, const struct parsedname * const), void *v, const struct parsedname *pn_type_directory);
 static ZERO_OR_ERROR FS_realdir(void (*dirfunc) (void *, const struct parsedname * const), void *v, const struct parsedname *pn2, uint32_t * flags);
-static ZERO_OR_ERROR FS_cache2real(void (*dirfunc) (void *, const struct parsedname * const), void *v, const struct parsedname *pn2, uint32_t * flags);
+static ZERO_OR_ERROR FS_cache_or_real(void (*dirfunc) (void *, const struct parsedname * const), void *v, const struct parsedname *pn2, uint32_t * flags);
 static ZERO_OR_ERROR FS_busdir(void (*dirfunc) (void *, const struct parsedname *), void *v, const struct parsedname *pn_directory);
 
 static void FS_stype_dir(void (*dirfunc) (void *, const struct parsedname *), void *v, const struct parsedname *pn_root_directory);
@@ -56,11 +55,14 @@ static ZERO_OR_ERROR FS_dir_plus(void (*dirfunc) (void *, const struct parsednam
     pn_directory->selected_device and pn_directory->sn still set
     pn_directory->selected_filetype loops through
 */
-/* path is the path which "pn_directory" parses */
+
 /* FS_dir produces the "invariant" portion of the directory, passing on to
    FS_dir_all_connections the variable part */
 ZERO_OR_ERROR FS_dir(void (*dirfunc) (void *, const struct parsedname *), void *v, struct parsedname *pn_directory)
 {
+	/* applies 'dirfunc' to each directory element of pn_directory */
+	/* void * v is extra information passed along */
+	
 	uint32_t flags;
 	LEVEL_DEBUG("path=%s", pn_directory->path);
 	pn_directory->control_flags |= ALIAS_REQUEST ; // All local directory queries want alias translation
@@ -151,7 +153,7 @@ static ZERO_OR_ERROR FS_dir_both(void (*dirfunc) (void *, const struct parsednam
 				FS_interface_dir(dirfunc, v, pn_raw_directory);
 			}
 			/* Now get the actual devices */
-			ret = FS_cache2real(dirfunc, v, pn_raw_directory, flags);
+			ret = FS_cache_or_real(dirfunc, v, pn_raw_directory, flags);
 			/* simultaneous directory */
 			if (flags[0] & (DEV_temp | DEV_volt)) {
 				FS_simultaneous_entry(dirfunc, v, pn_raw_directory);
@@ -239,127 +241,111 @@ static void FS_simultaneous_entry(void (*dirfunc) (void *, const struct parsedna
 	FS_dir_plus(dirfunc, v, &ignoreflag, pn_root_directory, "simultaneous");
 }
 
-#if OW_MT
-
 /* path is the path which "pn_directory" parses */
 /* FS_dir_all_connections produces the data that can vary: device lists, etc. */
 
-static ZERO_OR_ERROR FS_dir_all_connections_loop(void (*dirfunc)
-									    (void *, const struct parsedname * const),
-									   void *v, struct connection_in * in, const struct parsedname *pn_directory, uint32_t * flags);
-
 struct dir_all_connections_struct {
-	struct connection_in * current ;
-	const struct parsedname *pn_directory;
+	struct port_in * pin ;
+	struct connection_in * cin ;
+	struct parsedname pn_directory;
 	void (*dirfunc) (void *, const struct parsedname *);
 	void *v;
-	uint32_t *flags;
+	uint32_t flags;
 	ZERO_OR_ERROR ret;
 };
 
 /* Embedded function */
-static void *FS_dir_all_connections_callback(void *v)
+/* Directory on a particular port's channel */
+static void FS_dir_all_connections_callback_conn( struct dir_all_connections_struct * dacs )
 {
-	struct dir_all_connections_struct *dacs = v;
-	dacs->ret = FS_dir_all_connections_loop(dacs->dirfunc, dacs->v, dacs->current, dacs->pn_directory, dacs->flags);
-	pthread_exit(NULL);
-	return VOID_RETURN;
+	if ( dacs->cin == NO_CONNECTION ) {
+		return ;
+	}
+
+	SetKnownBus(dacs->cin->index, &(dacs->pn_directory) );
+
+	if ( BAD(TestConnection( &(dacs->pn_directory) )) ) {	// reconnect ok?
+		dacs->ret = -ECONNABORTED;
+	} else if (BusIsServer(dacs->pn_directory.selected_connection)) {	/* is this a remote bus? */
+		//printf("FS_dir_all_connections: Call ServerDir %s\n", dacs->pn_directory->path);
+		dacs->ret = ServerDir(dacs->dirfunc, dacs->v, &(dacs->pn_directory), &(dacs->flags));
+	} else if (IsAlarmDir( &(dacs->pn_directory) ) ) {	/* root or branch directory -- alarm state */
+		//printf("FS_dir_all_connections: Call FS_alarmdir %s\n", dacs->pn_directory->path);
+		dacs->ret = FS_alarmdir(dacs->dirfunc, dacs->v, &(dacs->pn_directory) );
+	} else {
+		dacs->ret = FS_cache_or_real(dacs->dirfunc, dacs->v, &(dacs->pn_directory), &(dacs->flags));
+	}
+	
+	// next channel
+	dacs->cin = dacs->cin->next ;
+	FS_dir_all_connections_callback_conn( dacs ) ;
 }
 
-static ZERO_OR_ERROR FS_dir_all_connections_loop(void (*dirfunc)
-	(void *, const struct parsedname *), void *v,
-	struct connection_in * current, const struct parsedname *pn_directory, uint32_t * flags)
+/* Callback (thread) once per port */
+/* Will need  to probe each connection (channel) on this port */
+static void *FS_dir_all_connections_callback_port(void *v)
 {
-	ZERO_OR_ERROR ret = 0;
-	struct dir_all_connections_struct dacs = { current->next, pn_directory, dirfunc, v, flags, 0 };
-	struct parsedname s_pn_bus_directory;
-	struct parsedname *pn_bus_directory = &s_pn_bus_directory;
+	struct dir_all_connections_struct *dacs = v;
+	struct dir_all_connections_struct dacs_next ;
 	pthread_t thread;
-	int threadbad = 1;
-
-	threadbad = (dacs.current==NULL) || pthread_create(&thread, DEFAULT_THREAD_ATTR, FS_dir_all_connections_callback, (void *) (&dacs));
-
-	memcpy(pn_bus_directory, pn_directory, sizeof(struct parsedname));	// shallow copy
-
-	SetKnownBus(current->index, pn_bus_directory);
-
-	if ( BAD(TestConnection(pn_bus_directory)) ) {	// reconnect ok?
-		ret = -ECONNABORTED;
-	} else if (BusIsServer(pn_bus_directory->selected_connection)) {	/* is this a remote bus? */
-		//printf("FS_dir_all_connections: Call ServerDir %s\n", pn_directory->path);
-		ret = ServerDir(dirfunc, v, pn_bus_directory, flags);
-	} else if (IsAlarmDir(pn_bus_directory)) {	/* root or branch directory -- alarm state */
-		//printf("FS_dir_all_connections: Call FS_alarmdir %s\n", pn_directory->path);
-		ret = FS_alarmdir(dirfunc, v, pn_bus_directory);
-	} else {
-		ret = FS_cache2real(dirfunc, v, pn_bus_directory, flags);
+	int threadbad = 0;
+	
+	if ( dacs->pin == NULL ) {
+		return VOID_RETURN;
 	}
-	//printf("FS_dir_all_connections4 pid=%ld adapter=%d ret=%d\n",pthread_self(), pn_directory->selected_connection->index,ret);
+
+	// set up structure
+	dacs_next.pin = dacs->pin->next ;
+
+	if ( dacs_next.pin == NULL ) {
+		threadbad = 1 ;
+	} else {
+		dacs_next.dirfunc = dacs->dirfunc ;
+		memcpy( &(dacs_next.pn_directory), &(dacs->pn_directory), sizeof(struct parsedname));	// shallow copy
+		dacs_next.v = dacs->v ;
+		dacs_next.flags = dacs->flags ;
+		dacs_next.ret = dacs->ret ;
+		threadbad = pthread_create(&thread, DEFAULT_THREAD_ATTR, FS_dir_all_connections_callback_port, (void *) (&dacs_next));
+	}
+
+	// First channel
+	dacs->cin = dacs->pin->first ;
+	FS_dir_all_connections_callback_conn( v ) ;
+		
+	//printf("FS_dir_all_connections4 pid=%ld adapter=%d ret=%d\n",pthread_self(), dacs->pn_directory->selected_connection->index,ret);
 	/* See if next bus was also queried */
 	if (threadbad == 0) {		/* was a thread created? */
 		if (pthread_join(thread, NULL)!= 0) {
-			return ret;			/* cannot join, so return only this result */
+			return VOID_RETURN ;			/* cannot join, so return only this result */
 		}
-		if (dacs.ret >= 0) {
-			return dacs.ret;	/* is it an error return? Then return this one */
-		}
-	}
-	return ret;
-}
-
-static ZERO_OR_ERROR
-FS_dir_all_connections(void (*dirfunc) (void *, const struct parsedname *), void *v, const struct parsedname *pn_directory, uint32_t * flags)
-{
-	struct connection_in * in = Inbound_Control.head ;
-	// Make sure head isn't NULL
-	if ( in == NO_CONNECTION ) {
-		return 0 ;
-	}
-	// Start iterating through buses
-	return FS_dir_all_connections_loop(dirfunc, v, in, pn_directory, flags);
-}
-
-#else							/* OW_MT */
-
-/* path is the path which "pn_directory" parses */
-/* FS_dir_all_connections produces the data that can vary: device lists, etc. */
-static ZERO_OR_ERROR
-FS_dir_all_connections(void (*dirfunc) (void *, const struct parsedname *), void *v, const struct parsedname *pn_directory, uint32_t * flags)
-{
-	ZERO_OR_ERROR ret = 0;
-	struct connection_in * in;
-	struct parsedname s_pn_selected_connection;
-	struct parsedname *pn_selected_connection = &s_pn_selected_connection;
-
-	memcpy(pn_selected_connection, pn_directory, sizeof(struct parsedname));	//shallow copy
-
-	for (in = Inbound_Control.head ; in != NO_CONNECTION ; in = in->next ) {
-		SetKnownBus(in->index, pn_selected_connection);
-
-		if ( BAD(TestConnection(pn_selected_connection)) ) {
-			continue ;
-		}
-		if (BusIsServer(pn_selected_connection->selected_connection)) {	/* is this a remote bus? */
-			ZERO_OR_ERROR server = ServerDir(dirfunc, v, pn_selected_connection, flags);
-			if ( server ) {
-				ret = server ;
-			}
-			continue ;
-		}
-		/* local bus */
-		if (IsAlarmDir(pn_selected_connection)) {	/* root or branch directory -- alarm state */
-			ZERO_OR_ERROR alarm_return = FS_alarmdir(dirfunc, v, pn_selected_connection);
-			if ( alarm_return ) {
-				ret = alarm_return ;
-			}
+		if (dacs_next.ret >= 0) {
+			dacs->ret = dacs_next.ret;	/* is it an error return? Then return this one */
 		} else {
-			ret = FS_cache2real(dirfunc, v, pn_selected_connection, flags);
-		}
+			dacs->flags |= dacs_next.flags ;
+		}		
 	}
+	return VOID_RETURN;
+}
 
-	return ret;
+static ZERO_OR_ERROR
+FS_dir_all_connections(void (*dirfunc) (void *, const struct parsedname *), void *v, const struct parsedname *pn_directory, uint32_t * flags)
+{
+	struct dir_all_connections_struct dacs ;
+		
+	// set up structure
+	dacs.pin = Inbound_Control.head_port ; 
+	dacs.dirfunc = dirfunc ;
+	memcpy( &(dacs.pn_directory), pn_directory, sizeof(struct parsedname));	// shallow copy
+	dacs.v = v ;
+	dacs.flags = 0 ;
+	dacs.ret = 0 ;
+	
+	// Start iterating through buses
+	FS_dir_all_connections_callback_port(  (void *) (& dacs) ) ;
+	
+	*flags = dacs.flags ;	
+	return dacs.ret ;
 }
-#endif							/* OW_MT */
 
 /* Device directory (i.e. show the properties) -- all from memory */
 /* Respect the Visibility status and also show only the correct subdir level */
@@ -591,6 +577,7 @@ static ZERO_OR_ERROR FS_alarmdir(void (*dirfunc) (void *, const struct parsednam
 
 	switch ( ret ) {
 		case search_good:
+			// include fo completeness -- can't actually be a value.
 		case search_done:
 			return 0 ;
 		case search_error:
@@ -695,9 +682,7 @@ static ZERO_OR_ERROR FS_realdir(void (*dirfunc) (void *, const struct parsedname
    */
 void FS_LoadDirectoryOnly(struct parsedname *pn_directory, const struct parsedname *pn_original)
 {
-	if (pn_directory != pn_original) {
-		memcpy(pn_directory, pn_original, sizeof(struct parsedname));	//shallow copy
-	}
+	memmove( pn_directory, pn_original, sizeof(struct parsedname)) ; // shallow copy
 	if (RootNotBranch(pn_directory)) {
 		memset(pn_directory->sn, 0, SERIAL_NUMBER_SIZE);
 	} else {
@@ -714,11 +699,11 @@ void FS_LoadDirectoryOnly(struct parsedname *pn_directory, const struct parsedna
 /* not within a device, nor alarm state */
 /* Also, adapters and stats handled elsewhere */
 /* Cache2Real try the cache first, else get directory from bus (and add to cache) */
-static ZERO_OR_ERROR FS_cache2real(void (*dirfunc) (void *, const struct parsedname *), void *v, const struct parsedname *pn_real_directory, uint32_t * flags)
+static ZERO_OR_ERROR FS_cache_or_real(void (*dirfunc) (void *, const struct parsedname *), void *v, const struct parsedname *pn_real_directory, uint32_t * flags)
 {
 	size_t dindex;
 	struct dirblob db;
-	BYTE sn[8];
+	BYTE sn[SERIAL_NUMBER_SIZE];
 
 	/* Special handling for External directory -- just walk tree */
 	if ( get_busmode(pn_real_directory->selected_connection) == bus_external ) {
@@ -726,13 +711,16 @@ static ZERO_OR_ERROR FS_cache2real(void (*dirfunc) (void *, const struct parsedn
 	}
 	
 	/* Test to see whether we should get the directory "directly" */
-	if (SpecifiedBus(pn_real_directory) || IsUncachedDir(pn_real_directory)
-		|| BAD( Cache_Get_Dir(&db, pn_real_directory)) ) {
-		//printf("FS_cache2real: didn't find anything at bus %d\n", pn_real_directory->selected_connection->index);
+	if (
+		SpecifiedBus(pn_real_directory) 
+		|| IsUncachedDir(pn_real_directory) // asking for uncached
+		|| BAD( Cache_Get_Dir(&db, pn_real_directory)) // cache'd version isn't available (or old)
+		) {
+		// directly
 		return FS_realdir(dirfunc, v, pn_real_directory, flags);
 	}
-	//printf("Post test cache for dir, snlist=%p, devices=%lu\n",snlist,devices) ;
-	/* We have a cached list in snlist. Note that we have to free this memory */
+
+	// Use cached version of directory
 
 	/* STATISTICS */
 	STAT_ADD1(dir_main.calls);
@@ -851,18 +839,21 @@ static ZERO_OR_ERROR FS_externaldir(void (*dirfunc) (void *, const struct parsed
 static ZERO_OR_ERROR FS_busdir(void (*dirfunc) (void *, const struct parsedname *), void *v, const struct parsedname *pn_directory)
 {
 	char bus[OW_FULLNAME_MAX];
-	struct connection_in * in ;
+	struct port_in * pin ;
 	uint32_t ignoreflag ;
 
 	if (!RootNotBranch(pn_directory)) {
 		return 0;
 	}
 
-	for ( in = Inbound_Control.head ; in != NO_CONNECTION ; in = in->next ) {
-		UCLIBCLOCK;
-		snprintf(bus, OW_FULLNAME_MAX, "bus.%d", in->index);
-		UCLIBCUNLOCK;
-		FS_dir_plus(dirfunc, v, &ignoreflag, pn_directory, bus);
+	for ( pin = Inbound_Control.head_port ; pin != NULL ; pin = pin->next ) {
+		struct connection_in * cin ;
+		for ( cin = pin->first ; cin != NO_CONNECTION ; cin = cin->next ) {
+			UCLIBCLOCK;
+			snprintf(bus, OW_FULLNAME_MAX, "bus.%d", cin->index);
+			UCLIBCUNLOCK;
+			FS_dir_plus(dirfunc, v, &ignoreflag, pn_directory, bus);
+		}
 	}
 
 	return 0;
