@@ -1,9 +1,9 @@
 /*
-$Id: ow_ds2482.c,v 1.104 2012/04/08 12:57:35 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -52,15 +52,16 @@ enum ds2482_address {
 	ds2482_too_far
 } ;
 
-static GOOD_OR_BAD DS2482_detect_bus(enum ds2482_address chip_num, char * i2c_device, struct connection_in *in) ;
-static GOOD_OR_BAD DS2482_detect_sys( int any, enum ds2482_address chip_num, struct connection_in *in) ;
-static GOOD_OR_BAD DS2482_detect_dir( int any, enum ds2482_address chip_num, struct connection_in *in) ;
-static GOOD_OR_BAD DS2482_detect_single(int lowindex, int highindex, char * i2c_device, struct connection_in *in) ;
+static GOOD_OR_BAD DS2482_detect_bus(enum ds2482_address chip_num, char * i2c_device, struct port_in *pin) ;
+static GOOD_OR_BAD DS2482_detect_sys( int any, enum ds2482_address chip_num, struct port_in *pin) ;
+static GOOD_OR_BAD DS2482_detect_dir( int any, enum ds2482_address chip_num, struct port_in *pin) ;
+static GOOD_OR_BAD DS2482_detect_single(int lowindex, int highindex, char * i2c_device, struct port_in *pin) ;
 static enum search_status DS2482_next_both(struct device_search *ds, const struct parsedname *pn);
 static GOOD_OR_BAD DS2482_triple(BYTE * bits, int direction, FILE_DESCRIPTOR_OR_ERROR file_descriptor);
 static GOOD_OR_BAD DS2482_send_and_get(FILE_DESCRIPTOR_OR_ERROR file_descriptor, const BYTE wr, BYTE * rd);
 static RESET_TYPE DS2482_reset(const struct parsedname *pn);
 static GOOD_OR_BAD DS2482_sendback_data(const BYTE * data, BYTE * resp, const size_t len, const struct parsedname *pn);
+static GOOD_OR_BAD DS2483_test(FILE_DESCRIPTOR_OR_ERROR file_descriptor);
 static void DS2482_setroutines(struct connection_in *in);
 static GOOD_OR_BAD HeadChannel(struct connection_in *head);
 static GOOD_OR_BAD CreateChannels(struct connection_in *head);
@@ -89,10 +90,11 @@ static GOOD_OR_BAD DS2482_PowerByte(const BYTE byte, BYTE * resp, const UINT del
 #define DS2482_CMD_1WIRE_TRIPLET       0x78	/* Param: Dir byte (bit7) */
 
 /* Values for DS2482_CMD_SET_READ_PTR */
-#define DS2482_PTR_CODE_STATUS         0xF0
-#define DS2482_PTR_CODE_DATA           0xE1
-#define DS2482_PTR_CODE_CHANNEL        0xD2	/* DS2482-800 only */
-#define DS2482_PTR_CODE_CONFIG         0xC3
+#define DS2482_STATUS_REGISTER         		 0xF0
+#define DS2482_READ_DATA_REGISTER            0xE1
+#define DS2482_DEVICE_CONFIGURATION_REGISTER 0xC3
+#define DS2482_CHANNEL_SELECTION_REGISTER    0xD2	/* DS2482-800 only */
+#define DS2482_PORT_CONFIGURATION_REGISTER   0xB4   /* DS2483 only */
 
 /**
  * Configure Register bit definitions
@@ -101,7 +103,8 @@ static GOOD_OR_BAD DS2482_PowerByte(const BYTE byte, BYTE * resp, const UINT del
  */
 #define DS2482_REG_CFG_1WS     0x08
 #define DS2482_REG_CFG_SPU     0x04
-#define DS2482_REG_CFG_PPM     0x02
+#define DS2482_REG_CFG_PDN     0x02 /* DS2483 only, power down */
+#define DS2482_REG_CFG_PPM     0x02 /* non-DS2483, presence pulse masking */
 #define DS2482_REG_CFG_APU     0x01
 
 /**
@@ -125,7 +128,7 @@ static GOOD_OR_BAD DS2482_PowerByte(const BYTE byte, BYTE * resp, const UINT del
 
 /* Defines for making messages more explicit */
 #define I2Cformat "I2C bus %s, channel %d/%d"
-#define I2Cvar(in)  SOC(in)->devicename, (in)->master.i2c.index, (in)->master.i2c.channels
+#define I2Cvar(in)  DEVICENAME(in), (in)->master.i2c.index, (in)->master.i2c.channels
 
 /* Device-specific functions */
 static void DS2482_setroutines(struct connection_in *in)
@@ -139,6 +142,8 @@ static void DS2482_setroutines(struct connection_in *in)
 	in->iroutines.sendback_bits = NO_SENDBACKBITS_ROUTINE;
 	in->iroutines.select = NO_SELECT_ROUTINE;
 	in->iroutines.select_and_sendback = NO_SELECTANDSENDBACK_ROUTINE;
+	in->iroutines.set_config = NO_SET_CONFIG_ROUTINE;
+	in->iroutines.get_config = NO_GET_CONFIG_ROUTINE;
 	in->iroutines.reconnect = DS2482_redetect;
 	in->iroutines.close = DS2482_close;
 	in->iroutines.flags = ADAP_FLAG_overdrive;
@@ -148,24 +153,25 @@ static void DS2482_setroutines(struct connection_in *in)
 /* All the rest of the program sees is the DS2482_detect and the entry in iroutines */
 /* Open a DS2482 */
 /* Top level detect routine */
-GOOD_OR_BAD DS2482_detect(struct connection_in *in)
+GOOD_OR_BAD DS2482_detect(struct port_in *pin)
 {
 	struct address_pair ap ;
 	GOOD_OR_BAD gbResult ;
 	enum ds2482_address chip_num ;
 	
-	Parse_Address( SOC(in)->devicename, &ap ) ;
+	Parse_Address( pin->init_data, &ap ) ;
 	
 	switch ( ap.second.type ) {
 		case address_numeric:
 			if ( ap.second.number < ds2482_18 || ap.second.number >= ds2482_too_far ) {
-				LEVEL_CALL("DS2482 bus address <%s> invalid. Will scan.", ap.second.alpha) ;
+				LEVEL_CALL("DS2482 bus address <%s> invalid. Will search.", ap.second.alpha) ;
 				chip_num = ds2482_any ;
 			} else {
 				chip_num = ap.second.number ;
 			}
 			break ;
 		case address_all:
+		case address_asterix:
 			chip_num = ds2482_all ;
 			break ;
 		case address_none:
@@ -177,20 +183,19 @@ GOOD_OR_BAD DS2482_detect(struct connection_in *in)
 			break ;
 	}
 
-	SAFEFREE( SOC(in)->devicename ) ;
-
 	switch ( ap.first.type ) {
 		case address_all:
+		case address_asterix:
 			// All adapters
-			gbResult = DS2482_detect_sys( 0, chip_num, in ) ;
+			gbResult = DS2482_detect_sys( 0, chip_num, pin ) ;
 			break ;
 		case address_none:
 			// Any adapter -- first one found
-			gbResult = DS2482_detect_sys( 1, chip_num, in ) ;
+			gbResult = DS2482_detect_sys( 1, chip_num, pin ) ;
 			break ;
 		default:
 			// traditional, actual bus specified
-			gbResult = DS2482_detect_bus( chip_num, ap.first.alpha, in ) ;
+			gbResult = DS2482_detect_bus( chip_num, ap.first.alpha, pin ) ;
 			break ;
 	}
 	
@@ -203,12 +208,12 @@ GOOD_OR_BAD DS2482_detect(struct connection_in *in)
 /* Use sysfs to find i2c adapters */
 /* cycle through SYSFS_I2C_Path */
 /* return GOOD if any found */
-static GOOD_OR_BAD DS2482_detect_sys( int any, enum ds2482_address chip_num, struct connection_in *in_original)
+static GOOD_OR_BAD DS2482_detect_sys( int any, enum ds2482_address chip_num, struct port_in *pin_original)
 {
 	DIR * i2c_list_dir ;
 	struct dirent * i2c_bus ;
 	int found = 0 ;
-	struct connection_in * in_current = in_original ;
+	struct port_in * pin_current = pin_original ;
 
 	// We'll look in this directory for available i2c adapters.
 	// This may be linux 2.6 specific
@@ -216,7 +221,7 @@ static GOOD_OR_BAD DS2482_detect_sys( int any, enum ds2482_address chip_num, str
 	if ( i2c_list_dir == NULL ) {
 		ERROR_CONNECT( "Cannot open %d to find available i2c devices",SYSFS_I2C_Path ) ;
 		// Use the cruder approach of trying all possible numbers
-		return DS2482_detect_dir( any, chip_num, in_original ) ;
+		return DS2482_detect_dir( any, chip_num, pin_original ) ;
 	}
 
 	/* cycle through entries in /sys/class/i2c-adapter */
@@ -232,34 +237,43 @@ static GOOD_OR_BAD DS2482_detect_sys( int any, enum ds2482_address chip_num, str
 		}
 
 		// Now look for the ds2482's
-		if ( BAD( DS2482_detect_bus( chip_num, dev_name, in_current ) ) ) {
+		if ( BAD( DS2482_detect_bus( chip_num, dev_name, pin_current ) ) ) {
 			continue ; // none found on this i2c bus
 		}
 
 		// at least one found on this i2c bus
 		++found ;
 		if ( any ) { // found one -- that's enough
-			break ;
+			closedir( i2c_list_dir ) ;
+			return gbGOOD ;
 		}
 
 		// ALL? then set up a new connection_in slot for the next one
-		in_current = NewIn(in_current) ;
-		if ( in_current == NO_CONNECTION ) {
+		pin_current = NewPort(pin_current) ;
+		if ( pin_current == NULL ) {
 			break ;
 		}
 	}
 
 	closedir( i2c_list_dir ) ;
-	return found>0 ? gbGOOD : gbBAD ;
+	
+	if ( found==0 ) {
+		return gbBAD ;
+	}
+
+	if ( pin_current != pin_original ) {
+		RemovePort( pin_current ) ;
+	}
+	return gbGOOD ;
 }
 
 /* non sysfs method -- try by bus name */
 /* cycle through /dev/i2c-n */
 /* returns GOOD if any adpters found */
-static GOOD_OR_BAD DS2482_detect_dir( int any, enum ds2482_address chip_num, struct connection_in *in_original)
+static GOOD_OR_BAD DS2482_detect_dir( int any, enum ds2482_address chip_num, struct port_in *pin_original)
 {
 	int found = 0 ;
-	struct connection_in * in_current = in_original ;
+	struct port_in * pin_current = pin_original ;
 	int bus = 0 ;
 	int sn_ret ;
 
@@ -278,54 +292,61 @@ static GOOD_OR_BAD DS2482_detect_dir( int any, enum ds2482_address chip_num, str
 		}
 
 		// Now look for the ds2482's
-		if ( BAD( DS2482_detect_bus( chip_num, dev_name, in_current ) ) ) {
+		if ( BAD( DS2482_detect_bus( chip_num, dev_name, pin_current ) ) ) {
 			continue ;
 		}
 
 		// at least one found
 		++found ;
 		if ( any ) {
-			break ;
+			return gbGOOD ;
 		}
 
 		// ALL? then set up a new connection_in slot
-		in_current = NewIn(in_current) ;
-		if ( in_current == NO_CONNECTION ) {
+		pin_current = NewPort(pin_current) ;
+		if ( pin_current == NULL ) {
 			break ;
 		}
 	}
 
-	return found>0 ? gbGOOD : gbBAD ;
+	if ( found == 0 ) {
+		return gbBAD ;
+	}
+	
+	if ( pin_original != pin_current ) {
+		RemovePort(pin_current) ;
+	}
+	return gbGOOD ;
 }
 
 /* Try to see if there is a DS2482 device on the specified i2c bus */
 /* Includes  a fix from Pascal Baerten */
-static GOOD_OR_BAD DS2482_detect_bus(enum ds2482_address chip_num, char * i2c_device, struct connection_in * in_original)
+static GOOD_OR_BAD DS2482_detect_bus(enum ds2482_address chip_num, char * i2c_device, struct port_in * pin_original)
 {
 	switch (chip_num) {
 		case ds2482_any:
 			// usual case, find the first adapter
-			return DS2482_detect_single( 0, 7, i2c_device, in_original ) ;
+			return DS2482_detect_single( 0, 7, i2c_device, pin_original ) ;
 		case ds2482_all:
 			// Look through all the possible i2c addresses
 			{
 				int start_chip = 0 ;
-				struct connection_in * all_in = in_original ;
+				struct port_in * all_pin = pin_original ;
 				do {
-					if ( BAD( DS2482_detect_single( start_chip, 7, i2c_device, all_in ) ) ) {
-						if ( in_original == all_in ) { //first time
+					if ( BAD( DS2482_detect_single( start_chip, 7, i2c_device, all_pin ) ) ) {
+						if ( pin_original == all_pin ) { //first time
 							return gbBAD ;
 						}
 						LEVEL_DEBUG("Cleaning excess allocated i2c structure");
-						RemoveIn(all_in); //Pascal Baerten :this removes the false DS2482-100 provisioned
+						RemovePort(all_pin); //Pascal Baerten :this removes the false DS2482-100 provisioned
 						return gbGOOD ;
 					}
-					start_chip = all_in->master.i2c.i2c_index + 1 ;
+					start_chip = all_pin->first->master.i2c.i2c_index + 1 ;
 					if ( start_chip > 7 ) {
 						return gbGOOD ;
 					}
-					all_in = NewIn(all_in) ;
-					if ( all_in == NO_CONNECTION ) {
+					all_pin = NewPort(all_pin) ;
+					if ( all_pin == NULL ) {
 						return gbBAD ;
 					}
 				} while (1) ;
@@ -333,35 +354,33 @@ static GOOD_OR_BAD DS2482_detect_bus(enum ds2482_address chip_num, char * i2c_de
 			break ;
 		default:
 			// specific i2c address
-			return DS2482_detect_single( chip_num, chip_num, i2c_device, in_original ) ;
+			return DS2482_detect_single( chip_num, chip_num, i2c_device, pin_original ) ;
 	}
 }
 
 /* Try to see if there is a DS2482 device on the specified i2c bus */
-static GOOD_OR_BAD DS2482_detect_single(int lowindex, int highindex, char * i2c_device, struct connection_in *in)
+static GOOD_OR_BAD DS2482_detect_single(int lowindex, int highindex, char * i2c_device, struct port_in *pin)
 {
 	int test_address[8] = { 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, };	// the last 4 are -800 only
 	int i2c_index;
 	FILE_DESCRIPTOR_OR_ERROR file_descriptor;
+	struct connection_in * in = pin->first ;
 
 	/* Sanity check */
 	if ( lowindex < 0 ) {
 		LEVEL_DEBUG("Bad lower bound");
 		return gbBAD ;
 	}
-	if ( highindex >= sizeof(test_address)/sizeof(int) ) {
+	if ( highindex >= (int) (sizeof(test_address)/sizeof(int)) ) {
 		LEVEL_DEBUG("Bad upper bound");
 		return gbBAD ;
 	}
 	
 	/* open the i2c port */
-	file_descriptor = open(i2c_device, O_RDWR | O_CLOEXEC);
+	file_descriptor = open(i2c_device, O_RDWR);
 	if ( FILE_DESCRIPTOR_NOT_VALID(file_descriptor) ) {
 		ERROR_CONNECT("Could not open i2c device %s", i2c_device);
 		return gbBAD;
-	} else {
-		// in case O_CLOEXEC not supported
-		fcntl (file_descriptor, F_SETFD, FD_CLOEXEC) ;
 	}
 
 	/* Set up low-level routines */
@@ -376,16 +395,15 @@ static GOOD_OR_BAD DS2482_detect_single(int lowindex, int highindex, char * i2c_
 			BYTE c;
 			LEVEL_CONNECT("Found an i2c device at %s address %.2X", i2c_device, trial_address);
 			/* Provisional setup as a DS2482-100 ( 1 channel ) */
-			SOC(in)->file_descriptor = file_descriptor;
-			SOC(in)->state = cs_deflowered;
-			SOC(in)->type = ct_i2c ;
+			in->pown->file_descriptor = file_descriptor;
+			pin->state = cs_deflowered;
+			pin->type = ct_i2c ;
 			in->master.i2c.i2c_address = trial_address;
 			in->master.i2c.i2c_index = i2c_index;
 			in->master.i2c.index = 0;
 			in->master.i2c.channels = 1;
 			in->master.i2c.current = 0;
 			in->master.i2c.head = in;
-			in->master.i2c.next = NO_CONNECTION;
 			in->adapter_name = "DS2482-100";
 			in->master.i2c.configreg = 0x00 ;	// default configuration setting desired
 			if ( Globals.i2c_APU ) {
@@ -394,8 +412,6 @@ static GOOD_OR_BAD DS2482_detect_single(int lowindex, int highindex, char * i2c_
 			if ( Globals.i2c_PPM ) {
 				in->master.i2c.configreg |= DS2482_REG_CFG_PPM ;
 			}
-			_MUTEX_INIT(in->master.i2c.all_channel_lock);
-			in->busmode = bus_i2c;
 			in->Adapter = adapter_DS2482_100;
 
 			/* write the RESET code */
@@ -411,10 +427,11 @@ static GOOD_OR_BAD DS2482_detect_single(int lowindex, int highindex, char * i2c_
 			// Note, only the lower nibble of the device config stored
 			
 			// Create name
-			SOC(in)->devicename = owmalloc( strlen(i2c_device) + 10 ) ;
-			if ( SOC(in)->devicename ) {
+			SAFEFREE( DEVICENAME(in) ) ;
+			DEVICENAME(in) = owmalloc( strlen(i2c_device) + 10 ) ;
+			if ( DEVICENAME(in) ) {
 				UCLIBCLOCK;
-				snprintf(SOC(in)->devicename, strlen(i2c_device) + 10, "%s:%.2X", i2c_device, trial_address);
+				snprintf(DEVICENAME(in), strlen(i2c_device) + 10, "%s:%.2X", i2c_device, trial_address);
 				UCLIBCUNLOCK;
 			}
 
@@ -424,7 +441,6 @@ static GOOD_OR_BAD DS2482_detect_single(int lowindex, int highindex, char * i2c_
 	}
 	/* fell though, no device found */
 	COM_close( in ) ;
-	in->busmode = bus_bad;
 	return gbBAD;
 }
 
@@ -437,19 +453,16 @@ static GOOD_OR_BAD DS2482_redetect(const struct parsedname *pn)
 	struct address_pair ap ; // to get device name from device:address
 
 	/* open the i2c port */
-	Parse_Address( SOC(head)->devicename, &ap ) ;
-	file_descriptor = open(ap.first.alpha, O_RDWR | O_CLOEXEC );
+	Parse_Address( DEVICENAME(head), &ap ) ;
+	file_descriptor = open(ap.first.alpha, O_RDWR );
 	Free_Address( &ap ) ;
 	if ( FILE_DESCRIPTOR_NOT_VALID(file_descriptor) ) {
-		ERROR_CONNECT("Could not open i2c device %s", SOC(head)->devicename);
+		ERROR_CONNECT("Could not open i2c device %s", DEVICENAME(head));
 		return gbBAD;
-	} else {
-		// in case O_CLOEXEC not supported
-		fcntl (file_descriptor, F_SETFD, FD_CLOEXEC) ;
 	}
 	
 	/* address is known */
-	if (ioctl(file_descriptor, I2C_SLAVE, head->master.i2c.i2c_address) < 0) {
+	if (ioctl(file_descriptor, I2C_SLAVE, address) < 0) {
 		ERROR_CONNECT("Cound not set i2c address to %.2X", address);
 	} else {
 		BYTE c;
@@ -458,16 +471,16 @@ static GOOD_OR_BAD DS2482_redetect(const struct parsedname *pn)
 			|| BAD(DS2482_readstatus(&c, file_descriptor, DS2482_Chip_reset_usec))	// pause .5 usec then read status
 			|| (c != (DS2482_REG_STS_LL | DS2482_REG_STS_RST))	// make sure status is properly set
 			) {
-			LEVEL_CONNECT("i2c device at %s address %d cannot be reset. Not a DS2482.", SOC(head)->devicename, address);
+			LEVEL_CONNECT("i2c device at %s address %d cannot be reset. Not a DS2482.", DEVICENAME(head), address);
 		} else {
 			struct connection_in * next ;
 			head->master.i2c.current = 0;
-			SOC(head)->file_descriptor = file_descriptor;
-			SOC(head)->state = cs_deflowered ;
-			SOC(head)->type = ct_i2c ;
+			head->pown->file_descriptor = file_descriptor;
+			head->pown->state = cs_deflowered ;
+			head->pown->type = ct_i2c ;
 			head->master.i2c.configchip = 0x00;	// default configuration register after RESET	
-			LEVEL_CONNECT("i2c device at %s address %d reset successfully", SOC(head)->devicename, address);
-			for ( next = head->master.i2c.next; next; next = next->master.i2c.next ) {
+			LEVEL_CONNECT("i2c device at %s address %d reset successfully", DEVICENAME(head), address);
+			for ( next = head->pown->first; next; next = next->next ) {
 				/* loop through devices, matching those that have the same "head" */
 				/* BUSLOCK also locks the sister channels for this */
 				next->reconnect_state = reconnect_ok;
@@ -515,7 +528,7 @@ static enum search_status DS2482_next_both(struct device_search *ds, const struc
 	int search_direction = 0;	/* initialization just to forestall incorrect compiler warning */
 	int bit_number;
 	int last_zero = -1;
-	FILE_DESCRIPTOR_OR_ERROR file_descriptor = SOC(pn->selected_connection->master.i2c.head)->file_descriptor;
+	FILE_DESCRIPTOR_OR_ERROR file_descriptor = pn->selected_connection->pown->file_descriptor;
 	BYTE bits[3];
 
 	// initialize for search
@@ -581,7 +594,7 @@ static RESET_TYPE DS2482_reset(const struct parsedname *pn)
 {
 	BYTE status_byte;
 	struct connection_in * in = pn->selected_connection ;
-	FILE_DESCRIPTOR_OR_ERROR file_descriptor = SOC(in->master.i2c.head)->file_descriptor;
+	FILE_DESCRIPTOR_OR_ERROR file_descriptor = in->pown->file_descriptor;
 
 	/* Make sure we're using the correct channel */
 	if ( BAD(DS2482_channel_select(in)) ) {
@@ -609,7 +622,7 @@ static RESET_TYPE DS2482_reset(const struct parsedname *pn)
 static GOOD_OR_BAD DS2482_sendback_data(const BYTE * data, BYTE * resp, const size_t len, const struct parsedname *pn)
 {
 	struct connection_in * in = pn->selected_connection ;
-	FILE_DESCRIPTOR_OR_ERROR file_descriptor = SOC(in->master.i2c.head)->file_descriptor;
+	FILE_DESCRIPTOR_OR_ERROR file_descriptor = in->pown->file_descriptor;
 	size_t i;
 
 	/* Make sure we're using the correct channel */
@@ -638,7 +651,7 @@ static GOOD_OR_BAD DS2482_send_and_get(FILE_DESCRIPTOR_OR_ERROR file_descriptor,
 	RETURN_BAD_IF_BAD( DS2482_readstatus(&c, file_descriptor, DS2482_1wire_write_usec) ) ;
 
 	/* Select the data register */
-	if (i2c_smbus_write_byte_data(file_descriptor, DS2482_CMD_SET_READ_PTR, DS2482_PTR_CODE_DATA) < 0) {
+	if (i2c_smbus_write_byte_data(file_descriptor, DS2482_CMD_SET_READ_PTR, DS2482_READ_DATA_REGISTER) < 0) {
 		return gbBAD;
 	}
 
@@ -653,6 +666,18 @@ static GOOD_OR_BAD DS2482_send_and_get(FILE_DESCRIPTOR_OR_ERROR file_descriptor,
 	return gbGOOD;
 }
 
+/* Is this a DS2483? Try to set to new register */
+static GOOD_OR_BAD DS2483_test(FILE_DESCRIPTOR_OR_ERROR file_descriptor)
+{
+	/* Select the data register */
+	if (i2c_smbus_write_byte_data(file_descriptor, DS2482_CMD_SET_READ_PTR, DS2482_PORT_CONFIGURATION_REGISTER) < 0) {
+		LEVEL_DEBUG("Cannot set to port configuration -- not a DS2483");
+		return gbBAD;
+	}
+	LEVEL_DEBUG("Can set to port configuration! a DS2483!");
+	return gbGOOD;
+}
+
 /* It's a DS2482 -- whether 1 channel or 8 channel not yet determined */
 /* All general stored data will be assigned to this "head" channel */
 static GOOD_OR_BAD HeadChannel(struct connection_in *head)
@@ -663,6 +688,12 @@ static GOOD_OR_BAD HeadChannel(struct connection_in *head)
 	if ( BAD(DS2482_channel_select(head)) ) {	/* Couldn't switch */
 		head->master.i2c.index = 0;	/* restore correct value */
 		LEVEL_CONNECT("DS2482-100 (Single channel)");
+		if ( GOOD(DS2483_test(head->pown->file_descriptor)) ) {
+			head->master.i2c.type = ds2483 ;
+		} else {
+				head->master.i2c.type = ds2482_100 ;
+		}
+
 		return gbGOOD;				/* happy as DS2482-100 */
 	}
 
@@ -670,6 +701,7 @@ static GOOD_OR_BAD HeadChannel(struct connection_in *head)
 	LEVEL_CONNECT("DS2482-800 (Eight channels)");
 	/* Must be a DS2482-800 */
 	head->master.i2c.channels = 8;
+	head->master.i2c.type = ds2482_800 ;
 	head->Adapter = adapter_DS2482_800;
 
 	return CreateChannels(head);
@@ -683,7 +715,6 @@ static GOOD_OR_BAD HeadChannel(struct connection_in *head)
  */
 static GOOD_OR_BAD CreateChannels(struct connection_in *head)
 {
-	struct connection_in * prior = head ;
 	int i;
 	char *name[] = { "DS2482-800(0)", "DS2482-800(1)", "DS2482-800(2)",
 		"DS2482-800(3)", "DS2482-800(4)", "DS2482-800(5)", "DS2482-800(6)",
@@ -692,15 +723,12 @@ static GOOD_OR_BAD CreateChannels(struct connection_in *head)
 	head->master.i2c.index = 0;
 	head->adapter_name = name[0];
 	for (i = 1; i < 8; ++i) {
-		struct connection_in * added = NewIn(head);
-		prior->master.i2c.next = added ;
+		struct connection_in * added = AddtoPort(head->pown);
 		if (added == NO_CONNECTION) {
 			return gbBAD;
 		}
 		added->master.i2c.index = i;
 		added->adapter_name = name[i];
-		added->master.i2c.next = NO_CONNECTION ; // for now
-		prior = added ;
 	}
 	return gbGOOD;
 }
@@ -730,7 +758,7 @@ static GOOD_OR_BAD DS2482_channel_select(struct connection_in * in)
 {
 	struct connection_in *head = in->master.i2c.head;
 	int chan = in->master.i2c.index;
-	FILE_DESCRIPTOR_OR_ERROR file_descriptor = SOC(head)->file_descriptor;
+	FILE_DESCRIPTOR_OR_ERROR file_descriptor = in->pown->file_descriptor;
 
 	/*
 		Write and verify codes for the CHANNEL_SELECT command (DS2482-800 only).
@@ -752,15 +780,18 @@ static GOOD_OR_BAD DS2482_channel_select(struct connection_in * in)
 
 		/* Select command */
 		if (i2c_smbus_write_byte_data(file_descriptor, DS2482_CMD_CHANNEL_SELECT, W_chan[chan]) < 0) {
+			LEVEL_DEBUG("Channel select set error");
 			return gbBAD;
 		}
 
 		/* Read back and confirm */
 		read_back = i2c_smbus_read_byte(file_descriptor);
 		if (read_back < 0) {
+			LEVEL_DEBUG("Channel select get error");
 			return gbBAD; // flag for DS2482-100 vs -800 detection
 		}
 		if (((BYTE) read_back) != R_chan[chan]) {
+			LEVEL_DEBUG("Channel selected doesn't match");
 			return gbBAD; // flag for DS2482-100 vs -800 detection
 		}
 
@@ -782,7 +813,7 @@ static GOOD_OR_BAD DS2482_channel_select(struct connection_in * in)
 static GOOD_OR_BAD SetConfiguration(BYTE c, struct connection_in *in)
 {
 	struct connection_in *head = in->master.i2c.head;
-	FILE_DESCRIPTOR_OR_ERROR file_descriptor = SOC(head)->file_descriptor;
+	FILE_DESCRIPTOR_OR_ERROR file_descriptor = in->pown->file_descriptor;
 	int read_back;
 
 	/* Write, readback, and compare configuration register */
@@ -811,7 +842,7 @@ static GOOD_OR_BAD DS2482_PowerByte(const BYTE byte, BYTE * resp, const UINT del
 	RETURN_BAD_IF_BAD(SetConfiguration(  in->master.i2c.configreg | DS2482_REG_CFG_SPU, in)) ;
 
 	/* send and get byte (and trigger strong pull-up */
-	RETURN_BAD_IF_BAD(DS2482_send_and_get( SOC(in->master.i2c.head)->file_descriptor, byte, resp)) ;
+	RETURN_BAD_IF_BAD(DS2482_send_and_get( in->pown->file_descriptor, byte, resp)) ;
 	TrafficOut("power response", resp, 1, in ) ;
 
 	UT_delay(delay);
@@ -824,8 +855,5 @@ static void DS2482_close(struct connection_in *in)
 	if (in == NO_CONNECTION) {
 		return;
 	}
-	if (in->master.i2c.index == 0) {
-		_MUTEX_DESTROY(in->master.i2c.all_channel_lock);
-	}
 }
 #endif							/* OW_I2C */
