@@ -1,9 +1,9 @@
 /*
-$Id: ow_ds9097.c,v 1.116 2011/07/07 13:27:21 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -45,6 +45,8 @@ static void DS9097_setroutines(struct connection_in *in)
 	in->iroutines.sendback_bits = DS9097_sendback_bits;
 	in->iroutines.select = NO_SELECT_ROUTINE;
 	in->iroutines.select_and_sendback = NO_SELECTANDSENDBACK_ROUTINE;
+	in->iroutines.set_config = NO_SET_CONFIG_ROUTINE;
+	in->iroutines.get_config = NO_GET_CONFIG_ROUTINE;
 	in->iroutines.reconnect = NO_RECONNECT_ROUTINE;
 	in->iroutines.close = COM_close;
 	in->iroutines.flags = ADAP_FLAG_default;
@@ -53,23 +55,28 @@ static void DS9097_setroutines(struct connection_in *in)
 
 /* _detect is a bit of a misnomer, no detection is actually done */
 // no bus locking here (higher up)
-GOOD_OR_BAD DS9097_detect(struct connection_in *in)
+GOOD_OR_BAD DS9097_detect(struct port_in *pin)
 {
+	struct connection_in * in = pin->first ;
 	/* Set up low-level routines */
 	DS9097_setroutines(in);
 
 	in->Adapter = adapter_DS9097;
 	// in->adapter_name already set, to support HA3 and HA4B
-	in->busmode = bus_passive;	// in case initially tried DS9097U
+	pin->busmode = bus_passive;	// in case initially tried DS9097U
 
 	/* open the COM port in 9600 Baud  */
 	COM_set_standard( in ) ; // standard COM port settings
-	SOC(in)->vmin = 1; // minimum chars
-	SOC(in)->vtime = 0; // decisec wait
+	pin->vmin = 1; // minimum chars
+	pin->vtime = 0; // decisec wait
 	
+	if (pin->init_data == NULL) {
+		LEVEL_DEFAULT("DS9097 (passive) busmaster requires port name");
+		return gbBAD;
+	}
 	RETURN_BAD_IF_BAD(COM_open(in)) ;
 
-	SOC(in)->flow = flow_first; // flow control
+	pin->flow = flow_first; // flow control
 	switch( DS9097_reset_in(in) ) {
 		case BUS_RESET_OK:
 		case BUS_RESET_SHORT:
@@ -77,10 +84,19 @@ GOOD_OR_BAD DS9097_detect(struct connection_in *in)
 		default:
 			break ;
 	}
+	if ( GOOD(serial_powercycle(in)) ) {
+		switch( DS9097_reset_in(in) ) {
+			case BUS_RESET_OK:
+			case BUS_RESET_SHORT:
+				return gbGOOD ;
+			default:
+				break ;
+		}
+	}
 
 	/* open the COM port in 9600 Baud  */
 	/* Second pass */
-	SOC(in)->flow = flow_second ;
+	pin->flow = flow_second ;
 	RETURN_BAD_IF_BAD(COM_change(in)) ;
 
 	switch( DS9097_reset_in(in) ) {
@@ -93,7 +109,7 @@ GOOD_OR_BAD DS9097_detect(struct connection_in *in)
 
 	/* open the COM port in 9600 Baud  */
 	/* Third pass, hardware flow control */
-	SOC(in)->flow = flow_first ;
+	pin->flow = flow_first ;
 	RETURN_BAD_IF_BAD(COM_change(in)) ;
 
 	switch( DS9097_reset_in(in) ) {
@@ -148,14 +164,16 @@ static RESET_TYPE DS9097_reset_in( struct connection_in * in )
 /* Puts in 9600 baud */
 static GOOD_OR_BAD DS9097_pre_reset(struct connection_in *in )
 {
+	struct port_in * pin = in->pown ;
+
 	RETURN_BAD_IF_BAD( COM_test(in) ) ;
 
 	/* 8 data bits */
-	SOC(in)->bits = 8 ;
-	SOC(in)->baud = B9600 ;
+	pin->bits = 8 ;
+	pin->baud = B9600 ;
 
 	if ( BAD( COM_change(in)) ) {
-		ERROR_CONNECT("Cannot set attributes: %s", SAFESTRING(SOC(in)->devicename));
+		ERROR_CONNECT("Cannot set attributes: %s", SAFESTRING(DEVICENAME(in)));
 		DS9097_post_reset( in ) ;
 		return gbBAD;
 	}
@@ -165,18 +183,20 @@ static GOOD_OR_BAD DS9097_pre_reset(struct connection_in *in )
 /* Restore terminal settings (serial port settings) */
 static void DS9097_post_reset(struct connection_in *in )
 {
+	struct port_in * pin = in->pown ;
+
 	if (Globals.eightbit_serial) {
 		/* coninue with 8 data bits */
-		SOC(in)->bits = 8;
+		pin->bits = 8;
 	} else {
 		/* 6 data bits, Receiver enabled, Hangup, Dont change "owner" */
-		SOC(in)->bits = 6;
+		pin->bits = 6;
 	}
 #ifndef B115200
 	/* MacOSX support max 38400 in termios.h ? */
-	SOC(in)->baud = B38400 ;
+	pin->baud = B38400 ;
 #else
-	SOC(in)->baud = B115200 ;
+	pin->baud = B115200 ;
 #endif
 
 	/* Flush the input and output buffers */
@@ -229,7 +249,7 @@ static GOOD_OR_BAD DS9097_sendback_bits(const BYTE * outbits, BYTE * inbits, con
 /* Routine to send a string of bits and get another string back */
 static GOOD_OR_BAD DS9097_send_and_get(const BYTE * bussend, BYTE * busget, const size_t length, struct connection_in * in)
 {
-	switch( SOC(in)->type ) {
+	switch( in->pown->type ) {
 		case ct_telnet:
 			RETURN_BAD_IF_BAD( telnet_write_binary( bussend, length, in) ) ;
 			break ;
