@@ -1,9 +1,9 @@
 /*
-$Id: ow_ds9097U.c,v 1.178 2012/04/07 18:41:41 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -73,6 +73,8 @@ static void DS2480_setroutines(struct connection_in *in)
     in->iroutines.sendback_bits = DS2480_sendback_bits;
 	in->iroutines.select = NO_SELECT_ROUTINE;
 	in->iroutines.select_and_sendback = NO_SELECTANDSENDBACK_ROUTINE;
+	in->iroutines.set_config = NO_SET_CONFIG_ROUTINE;
+	in->iroutines.get_config = NO_GET_CONFIG_ROUTINE;
 	in->iroutines.reconnect = DS2480_reconnect ;
 	in->iroutines.close = DS2480_close;
 	in->iroutines.flags = ADAP_FLAG_default;
@@ -233,13 +235,21 @@ static void DS2480_setroutines(struct connection_in *in)
 /* Reset and detect a DS2480B */
 /* returns 0=good */
 // bus locking at a higher level
-GOOD_OR_BAD DS2480_detect(struct connection_in *in)
+GOOD_OR_BAD DS2480_detect(struct port_in *pin)
 {
+	struct connection_in * in = pin->first ;
+
+	if (pin->init_data == NULL) {
+		LEVEL_DEFAULT("DS2480B-based bus master needs a port name");
+		return gbBAD;
+	}
+
 	/* Set up low-level routines */
 	DS2480_setroutines(in);
 
 	in->overdrive = 0 ;
 	in->flex = Globals.serial_flextime ;
+	pin->busmode = bus_serial;
 
 	// Now set desired baud and polarity
 	// BUS_reset will do the actual changes
@@ -249,7 +259,7 @@ GOOD_OR_BAD DS2480_detect(struct connection_in *in)
 	// first pass with hardware flow control
 	RETURN_GOOD_IF_GOOD( DS2480_detect_serial(in) ) ;
 
-	SOC(in)->flow = flow_second; // flow control
+	pin->flow = flow_second; // flow control
 	RETURN_BAD_IF_BAD(COM_change(in)) ;
 	return DS2480_detect_serial(in) ;
 }
@@ -257,15 +267,13 @@ GOOD_OR_BAD DS2480_detect(struct connection_in *in)
 // setting for serial port already made
 static GOOD_OR_BAD DS2480_detect_serial(struct connection_in *in)
 {
-	SOC(in)->state = cs_virgin ;
+	in->pown->state = cs_virgin ;
 	if ( BAD(DS2480_initialize_repeatedly(in)) ) {
 		LEVEL_DEBUG("Could not initilize the DS9097U even after several tries") ;
 		COM_close(in) ;
 		return gbBAD ;
 	}
 
-	in->busmode = bus_serial;
-
 	DS2480_adapter(in) ;
 	return gbGOOD ;
 }
@@ -308,31 +316,36 @@ static void DS2480_adapter(struct connection_in *in)
 
 static GOOD_OR_BAD DS2480_reconnect(const struct parsedname * pn)
 {
-	LEVEL_DEBUG("Attempting reconnect on %s",SAFESTRING(SOC(pn->selected_connection)->devicename));
+	LEVEL_DEBUG("Attempting reconnect on %s",SAFESTRING(DEVICENAME(pn->selected_connection)));
 	return DS2480_big_reset(pn->selected_connection) ;
 }
 
 // do the com port and configuration stuff
 static GOOD_OR_BAD DS2480_big_reset(struct connection_in * in)
 {
-	switch (SOC(in)->type) {
+	struct port_in * pin = in->pown ;
+	
+	switch (pin->type) {
 		case ct_telnet:
-			SOC(in)->timeout.tv_sec = Globals.timeout_network ;
-			SOC(in)->timeout.tv_usec = 0 ;
+			pin->timeout.tv_sec = Globals.timeout_network ;
+			pin->timeout.tv_usec = 0 ;
 			return DS2480_big_reset_serial(in) ;
 
 		case ct_serial:
 		default:
-			SOC(in)->timeout.tv_sec = Globals.timeout_serial ;
-			SOC(in)->timeout.tv_usec = 0 ;
+			pin->timeout.tv_sec = Globals.timeout_serial ;
+			pin->timeout.tv_usec = 0 ;
 
-			SOC(in)->flow = flow_none ;
+			pin->flow = flow_none ;
 			RETURN_GOOD_IF_GOOD( DS2480_big_reset_serial(in)) ;
 
-			SOC(in)->flow = flow_none ;
+			serial_powercycle(in) ;
 			RETURN_GOOD_IF_GOOD( DS2480_big_reset_serial(in)) ;
 
-			SOC(in)->flow = flow_hard ;
+			pin->flow = flow_none ;
+			RETURN_GOOD_IF_GOOD( DS2480_big_reset_serial(in)) ;
+
+			pin->flow = flow_hard ;
 			RETURN_GOOD_IF_GOOD( DS2480_big_reset_serial(in)) ;
 
 			return gbBAD ;
@@ -452,22 +465,24 @@ static GOOD_OR_BAD DS2480_configuration_read(BYTE parameter_code, BYTE value_cod
 // Set Baud rate -- can't use DS2480_configuration_code because return byte is at a different speed
 static void DS2480_set_baud_control(struct connection_in * in)
 {
+	struct port_in * pin = in->pown ;
+	
 	// restrict allowable baud rates based on device capabilities
-	COM_BaudRestrict( &(SOC(in)->baud), B9600, B19200, B57600, B115200, 0 ) ;
+	COM_BaudRestrict( &(pin->baud), B9600, B19200, B57600, B115200, 0 ) ;
 
 	if ( GOOD( DS2480_set_baud(in) ) ) {
 		return ;
 	}
-	LEVEL_DEBUG("Failed first attempt at resetting baud rate of bus master %s",SAFESTRING(SOC(in)->devicename)) ;
+	LEVEL_DEBUG("Failed first attempt at resetting baud rate of bus master %s",SAFESTRING(DEVICENAME(in))) ;
 
 	if ( GOOD( DS2480_set_baud(in) ) ) {
 		return ;
 	}
-	LEVEL_DEBUG("Failed second attempt at resetting baud rate of bus master %s",SAFESTRING(SOC(in)->devicename)) ;
+	LEVEL_DEBUG("Failed second attempt at resetting baud rate of bus master %s",SAFESTRING(DEVICENAME(in))) ;
 
 	// uh oh -- undefined state -- not sure what the bus speed is.
 	in->reconnect_state = reconnect_error ;
-	SOC(in)->baud = B9600 ;
+	pin->baud = B9600 ;
 	++in->changed_bus_settings ;
 	return;
 }
@@ -478,7 +493,7 @@ static GOOD_OR_BAD DS2480_set_baud(struct connection_in * in)
 	BYTE send_code ;
 
 	// Find rate parameter
-	switch ( SOC(in)->baud ) {
+	switch ( in->pown->baud ) {
 		case B9600:
 			value_code = PARMSET_9600 ;
 			break ;
@@ -498,7 +513,7 @@ static GOOD_OR_BAD DS2480_set_baud(struct connection_in * in)
 			break ;
 #endif
 		default:
-			SOC(in)->baud = B9600 ;
+			in->pown->baud = B9600 ;
 			value_code = PARMSET_9600 ;
 			break ;
 	}
@@ -887,7 +902,7 @@ static GOOD_OR_BAD DS2480_ProgramPulse(const struct parsedname *pn)
 // Write to the output -- works for tcp and COM
 static GOOD_OR_BAD DS2480_write(const BYTE * buf, const size_t size, struct connection_in * in)
 {
-	switch( SOC(in)->type ) {
+	switch( in->pown->type ) {
 		case ct_telnet:
 			return telnet_write_binary( buf, size, in) ;
 		case ct_serial:
