@@ -1,9 +1,9 @@
 /*
-$Id: ow_ds9490.c,v 1.231 2012/05/16 02:44:31 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -52,7 +52,7 @@ static RESET_TYPE DS9490_reset(const struct parsedname *pn);
 static void BUS_ERROR_fix(const struct parsedname *pn);
 
 static GOOD_OR_BAD DS9490_detect_single_adapter(int usb_nr, struct connection_in *in);
-static GOOD_OR_BAD DS9490_detect_all_adapters(struct connection_in * in_first);
+static GOOD_OR_BAD DS9490_detect_all_adapters(struct port_in * pin_first);
 static GOOD_OR_BAD DS9490_detect_specific_adapter(int bus_nr, int dev_nr, struct connection_in * in) ;
 
 static GOOD_OR_BAD DS9490_reconnect(const struct parsedname *pn);
@@ -88,6 +88,8 @@ static void DS9490_setroutines(struct connection_in *in)
 	in->iroutines.sendback_bits = NO_SENDBACKBITS_ROUTINE;
 	in->iroutines.select = NO_SELECT_ROUTINE;
 	in->iroutines.select_and_sendback = NO_SELECTANDSENDBACK_ROUTINE;
+	in->iroutines.set_config = NO_SET_CONFIG_ROUTINE;
+	in->iroutines.get_config = NO_GET_CONFIG_ROUTINE;
 	in->iroutines.reconnect = DS9490_reconnect;
 	in->iroutines.close = DS9490_close;
 	in->iroutines.flags = ADAP_FLAG_default;
@@ -185,24 +187,24 @@ static void DS9490_setroutines(struct connection_in *in)
 #define PARMSET_Slew0p70Vus 0x6
 #define PARMSET_Slew0p55Vus 0x7
 
-#define PARMSET_W1L_08us 0x0
-#define PARMSET_W1L_09us 0x1
-#define PARMSET_W1L_10us 0x2
-#define PARMSET_W1L_11us 0x3
-#define PARMSET_W1L_12us 0x4
-#define PARMSET_W1L_13us 0x5
-#define PARMSET_W1L_14us 0x6
-#define PARMSET_W1L_15us 0x7
-
-#define PARMSET_DS0_W0R_3us 0x0
-#define PARMSET_DS0_W0R_4us 0x1
-#define PARMSET_DS0_W0R_5us 0x2
-#define PARMSET_DS0_W0R_6us 0x3
-#define PARMSET_DS0_W0R_7us 0x4
-#define PARMSET_DS0_W0R_8us 0x5
-#define PARMSET_DS0_W0R_9us 0x6
-#define PARMSET_DS0_W0R_10us 0x7
+#define PARMSET_W1L_04us 0x0
+#define PARMSET_W1L_05us 0x1
+#define PARMSET_W1L_06us 0x2
+#define PARMSET_W1L_07us 0x3
+#define PARMSET_W1L_08us 0x4
+#define PARMSET_W1L_09us 0x5
+#define PARMSET_W1L_10us 0x6
+#define PARMSET_W1L_11us 0x7
 
+#define PARMSET_DS0_W0R_10us 0x0
+#define PARMSET_DS0_W0R_12us 0x1
+#define PARMSET_DS0_W0R_14us 0x2
+#define PARMSET_DS0_W0R_16us 0x3
+#define PARMSET_DS0_W0R_18us 0x4
+#define PARMSET_DS0_W0R_20us 0x5
+#define PARMSET_DS0_W0R_22us 0x6
+#define PARMSET_DS0_W0R_24us 0x7
+ 
 /* From datasheet http://datasheets.maxim-ic.com/en/ds/DS2490.pdf page 15 */
 /* 480 usec = 8usec * 60 and 60(decimal) = 0x3C */
 /* 480 usec is the recommended program pulse duration in the DS2406 DS2502 DS2505 datasheets */
@@ -212,15 +214,17 @@ static void DS9490_setroutines(struct connection_in *in)
 /* --- USB detection routines ----------------------------------*/
 
 /* Main routine for detecting (and setting up) the DS2490 1-wire USB chip */
-GOOD_OR_BAD DS9490_detect(struct connection_in *in)
+GOOD_OR_BAD DS9490_detect(struct port_in *pin)
 {
+	struct connection_in * in = pin->first ;
 	struct address_pair ap ;
 	GOOD_OR_BAD gbResult = gbBAD;
 	
 	/* uses "name" before it's cleared by connection_init */
-	Parse_Address( SOC(in)->devicename, &ap ) ;
+	Parse_Address( pin->init_data, &ap ) ;
 
 	DS9490_connection_init( in ) ;
+	pin->busmode = bus_usb;
 
 	switch ( ap.entries ) {
 		case 0:
@@ -230,21 +234,18 @@ GOOD_OR_BAD DS9490_detect(struct connection_in *in)
 		case 1:
 			switch( ap.first.type ) {
 				case address_all:
+				case address_asterix:
 					LEVEL_DEBUG("Look for all USB adapters");
-					gbResult = DS9490_detect_all_adapters(in) ;
+					gbResult = DS9490_detect_all_adapters(pin) ;
 					break ;
 				case address_numeric:
 					LEVEL_DEBUG("Look for USB adapter number %d",ap.first.number);
 					gbResult = DS9490_detect_single_adapter( ap.first.number, in) ;
 					break ;
-				case address_alpha:
-					if ( strncasecmp(ap.first.alpha,"scan",4) == 0 ) {
-						SAFEFREE(SOC(in)->devicename) ;
-						LEVEL_DEBUG("Add USB scanning capability");
-						gbResult = USB_monitor_detect(in) ;
-						break ;
-					}
-					// fall through
+				case address_scan:
+					// completely change personality!
+					gbResult = USB_monitor_detect(pin) ;
+					break ;
 				default:
 					LEVEL_DEFAULT("Unclear what <%s> means in USB specification, will use first adapter.",ap.first.alpha) ;
 					gbResult = DS9490_detect_single_adapter( 1, in) ;
@@ -254,6 +255,7 @@ GOOD_OR_BAD DS9490_detect(struct connection_in *in)
 		case 2:
 			if ( ap.first.type != address_numeric || ap.second.type != address_numeric ) {
 				LEVEL_DEFAULT("USB address <%s:%s> not in number:number format",ap.first.alpha,ap.second.alpha) ;
+				gbResult = gbBAD ;
 			} else {
 				gbResult = DS9490_detect_specific_adapter( ap.first.number, ap.second.number, in ) ;
 			}
@@ -267,7 +269,6 @@ GOOD_OR_BAD DS9490_detect(struct connection_in *in)
 static GOOD_OR_BAD DS9490_detect_single_adapter(int usb_nr, struct connection_in * in)
 {
 	struct usb_list ul;
-
 	USB_first(&ul);
 
 	if ( usb_nr == 1 ) {
@@ -309,13 +310,14 @@ static GOOD_OR_BAD DS9490_detect_specific_adapter(int bus_nr, int dev_nr, struct
 }
 
 /* Open a DS9490  -- low level code (to allow for repeats)  */
-static GOOD_OR_BAD DS9490_detect_all_adapters(struct connection_in * in_first)
+static GOOD_OR_BAD DS9490_detect_all_adapters(struct port_in * pin_first)
 {
 	struct usb_list ul;
-	struct connection_in * in = in_first ;
+	struct port_in * pin = pin_first ;
 
 	USB_first(&ul);
 	while ( GOOD(USB_next(&ul)) ) {
+		struct connection_in * in = pin->first ;
 		if ( BAD(DS9490_open_and_name(&ul, in)) ) {
 			LEVEL_DEBUG("Cannot open USB device %.d:%.d", ul.usb_bus_number, ul.usb_dev_number );
 			continue ;
@@ -324,20 +326,20 @@ static GOOD_OR_BAD DS9490_detect_all_adapters(struct connection_in * in_first)
 			LEVEL_DEBUG("Cannot access USB device %.d:%.d", ul.usb_bus_number, ul.usb_dev_number );
 			continue;
 		} else{
-			in = NewIn(in_first) ;
-			if ( in == NO_CONNECTION ) {
+			pin = NewPort(pin) ;
+			if ( pin == NULL ) {
 				return gbGOOD ;
 			}
 			// set up the new connection for the next adapter
-			DS9490_connection_init(in) ;
+			DS9490_connection_init(pin->first) ;
 		}
 	}
-	if ( in == in_first ) {
+	if ( pin == pin_first ) {
 		LEVEL_CONNECT("No USB DS9490 bus masters used");
 		return gbBAD;
 	}
 	// Remove the extra connection
-	RemoveIn(in);
+	RemovePort(pin);
 	return gbGOOD ;
 }
 
@@ -346,12 +348,10 @@ static void DS9490_connection_init( struct connection_in * in )
 	if ( in == NO_CONNECTION ) {
 		return ;
 	}
-	SAFEFREE( SOC(in)->devicename ) ;
-	SOC(in)->devicename = owstrdup(badUSBname);		// initialized
+	DEVICENAME(in) = owstrdup(badUSBname);		// initialized
 
 	DS9490_setroutines(in);		// set up close, reconnect, reset, ...
 
-	in->busmode = bus_usb;
 	in->flex = 1 ; // Michael Markstaller suggests this
 	in->master.usb.usb = NULL ; // no handle yet
 	in->master.usb.usb_bus_number = in->master.usb.usb_dev_number = -1 ;
@@ -387,7 +387,7 @@ static GOOD_OR_BAD DS9490_reconnect(const struct parsedname *pn)
 	LIBUSBUNLOCK;
 
 	if ( GOOD(ret) ) {
-		LEVEL_DEFAULT("Found USB DS9490 bus master after USB rescan as [%s]", SAFESTRING(SOC(in)->devicename));
+		LEVEL_DEFAULT("Found USB DS9490 bus master after USB rescan as [%s]", SAFESTRING(DEVICENAME(in)));
 	}
 	return ret;
 }
@@ -418,13 +418,11 @@ static GOOD_OR_BAD DS9490_redetect_specific_adapter( struct connection_in * in)
 	struct address_pair ap ;
 	GOOD_OR_BAD gbResult ;
 	
-	Parse_Address( SOC(in)->devicename, &ap ) ;
+	Parse_Address( in->pown->init_data, &ap ) ;
 	if ( ap.first.type != address_numeric || ap.second.type != address_numeric ) {
-		LEVEL_DEBUG("Cannot understand the specific usb address pair to reconnect <%s>",SOC(in)->devicename) ;
+		LEVEL_DEBUG("Cannot understand the specific usb address pair to reconnect <%s>",in->pown->init_data) ;
 		gbResult = gbBAD ;
 	} else {
-		char * name_copy = owstrdup( SOC(in)->devicename ) ; // since DS9490_close clears this value and we may need to restore on error
-
 		/* Have to protect usb_find_busses() and usb_find_devices() with
 		 * a lock since libusb could crash if 2 threads call it at the same time.
 		 * It's not called until DS9490_redetect_low(), but I lock here just
@@ -432,14 +430,7 @@ static GOOD_OR_BAD DS9490_redetect_specific_adapter( struct connection_in * in)
 		LIBUSBLOCK;
 		DS9490_close( in ) ;
 		gbResult = DS9490_detect_specific_adapter( ap.first.number, ap.second.number, in) ;
-		LIBUSBUNLOCK;
-		
-		if ( BAD(gbResult) ) {
-			SAFEFREE( SOC(in)->devicename ) ;
-			SOC(in)->devicename = name_copy ; // so no need to free here
-		} else {
-			owfree( name_copy ) ;
-		}
+		LIBUSBUNLOCK;	
 	}
 
 	Free_Address( &ap ) ;
@@ -453,7 +444,7 @@ static GOOD_OR_BAD DS9490_redetect_match( struct connection_in * in)
 	BYTE sn[SERIAL_NUMBER_SIZE] ;
 	int device_number ;
 
-	LEVEL_DEBUG("Attempting reconnect on %s",SAFESTRING(SOC(in)->devicename));
+	LEVEL_DEBUG("Attempting reconnect on %s",SAFESTRING(DEVICENAME(in)));
 
 	// Special case -- originally untagged adapter
 	if ( in->master.usb.ds1420_address[0] == '\0' ) {
@@ -467,7 +458,7 @@ static GOOD_OR_BAD DS9490_redetect_match( struct connection_in * in)
 	// This adapter has no tags, so not the one we want
 	if ( DirblobElements( &db) == 0 ) {
 		DirblobClear( &db ) ;
-		LEVEL_DATA("Empty directory on [%s] (Doesn't match initial scan).", SAFESTRING(SOC(in)->devicename));
+		LEVEL_DATA("Empty directory on [%s] (Doesn't match initial scan).", SAFESTRING(DEVICENAME(in)));
 		return gbBAD ;
 	}
 
@@ -475,14 +466,14 @@ static GOOD_OR_BAD DS9490_redetect_match( struct connection_in * in)
 	device_number = 0 ;
 	while ( DirblobGet( device_number, sn, &db ) == 0 ) {
 		if (memcmp(sn, in->master.usb.ds1420_address, SERIAL_NUMBER_SIZE) == 0) {	// same tag device?
-			LEVEL_DATA("Matching device [%s].", SAFESTRING(SOC(in)->devicename));
+			LEVEL_DATA("Matching device [%s].", SAFESTRING(DEVICENAME(in)));
 			DirblobClear( &db ) ;
 			return gbGOOD ;
 		}
 		++device_number ;
 	}
 	// Couldn't find correct ds1420 chip on this adapter
-	LEVEL_CONNECT("Couldn't find correct ds1420 chip on this bus master [%s] (want: " SNformat ")", SAFESTRING(SOC(in)->devicename), SNvar(in->master.usb.ds1420_address));
+	LEVEL_CONNECT("Couldn't find correct ds1420 chip on this bus master [%s] (want: " SNformat ")", SAFESTRING(DEVICENAME(in)), SNvar(in->master.usb.ds1420_address));
 	DirblobClear( &db ) ;
 	return gbBAD;
 }
@@ -769,14 +760,14 @@ static int FindDiscrepancy(BYTE * last_sn, BYTE * discrepancy_sn)
 static GOOD_OR_BAD DS9490_open_and_name(struct usb_list *ul, struct connection_in *in)
 {
 	if (in->master.usb.usb) {
-		LEVEL_DEFAULT("DS9490 %s was NOT closed?", SOC(in)->devicename);
+		LEVEL_DEFAULT("DS9490 %s was NOT closed?", DEVICENAME(in));
 		return gbBAD ;
 	}
 
-	SAFEFREE(SOC(in)->devicename) ;
-	SOC(in)->devicename = DS9490_device_name(ul);
+	SAFEFREE(DEVICENAME(in)) ;
+	DEVICENAME(in) = DS9490_device_name(ul);
 
-	if (SOC(in)->devicename == NULL) {
+	if (DEVICENAME(in) == NULL) {
 		return gbBAD;
 	}
 	
@@ -784,7 +775,7 @@ static GOOD_OR_BAD DS9490_open_and_name(struct usb_list *ul, struct connection_i
 		STAT_ADD1_BUS(e_bus_open_errors, in);
 		return gbBAD ;
 	} else if ( BAD( DS9490_setup_adapter(in)) ) {
-		LEVEL_DEFAULT("Error setting up USB DS9490 bus master at %s.", SOC(in)->devicename);
+		LEVEL_DEFAULT("Error setting up USB DS9490 bus master at %s.", DEVICENAME(in));
 		DS9490_close( in ) ;
 		return gbBAD ;
 	}
@@ -1064,15 +1055,15 @@ static void DS9490_SetFlexParameters(struct connection_in *in)
 	// default values for bus-timing when using --altUSB
 	if (Globals.altUSB) {
 		in->master.usb.pulldownslewrate = PARMSET_Slew1p37Vus;
-		in->master.usb.writeonelowtime = PARMSET_W1L_10us;
-		in->master.usb.datasampleoffset = PARMSET_DS0_W0R_8us;
+		in->master.usb.writeonelowtime = PARMSET_W1L_06us;
+		in->master.usb.datasampleoffset = PARMSET_DS0_W0R_20us;
 	} else {
 		/* This seem to be the default value when reseting the ds2490 chip */
 		// Change by Michael Markstaller from datasheet
 		in->master.usb.pulldownslewrate = PARMSET_Slew1p10Vus;
 //		in->master.usb.pulldownslewrate = PARMSET_Slew0p83Vus;
-		in->master.usb.writeonelowtime = PARMSET_W1L_12us;
-		in->master.usb.datasampleoffset = PARMSET_DS0_W0R_7us;
+		in->master.usb.writeonelowtime = PARMSET_W1L_08us;
+		in->master.usb.datasampleoffset = PARMSET_DS0_W0R_18us;
 	}
 	in->changed_bus_settings |= CHANGED_USB_SLEW | CHANGED_USB_LOW | CHANGED_USB_OFFSET ;
 }
