@@ -1,5 +1,5 @@
 /*
-$Id: ow_etherweather.c,v 1.45 2011/09/08 03:34:04 alfille Exp $
+$Id: ow_etherweather.c,v 1.53 2012/10/31 01:44:48 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -58,6 +58,7 @@ static void EtherWeather_setroutines(struct connection_in *in);
 
 static int EtherWeather_command(struct connection_in *in, char command, int datalen, const BYTE * idata, BYTE * odata)
 {
+	struct port_in * pin = in->pown ;
 	BYTE *packet;
 
 	// The packet's length field includes the command byte.
@@ -66,11 +67,11 @@ static int EtherWeather_command(struct connection_in *in, char command, int data
 	packet[1] = command;
 	memcpy( &packet[2], idata, datalen);
 
-	SOC(in)->timeout.tv_sec = 0 ;
-	SOC(in)->timeout.tv_usec = 200000 ;
+	pin->timeout.tv_sec = 0 ;
+	pin->timeout.tv_usec = 200000 ;
 
 	if ( BAD(COM_write( packet, datalen+2, in) ) ) {
-		ERROR_CONNECT("Trouble writing data to EtherWeather: %s", SAFESTRING(SOC(in)->devicename));
+		ERROR_CONNECT("Trouble writing data to EtherWeather: %s", SAFESTRING(DEVICENAME(in)));
 		STAT_ADD1_BUS(e_bus_write_errors, in);
 		owfree(packet) ;
 		return -EIO ;
@@ -78,7 +79,7 @@ static int EtherWeather_command(struct connection_in *in, char command, int data
 
 	// Allow extra time for powered bytes
 	if (command == 'P') {
-		SOC(in)->timeout.tv_sec += 2 ;
+		pin->timeout.tv_sec += 2 ;
 	}
 	// Read the response header
 	if ( BAD(COM_read( packet, 2, in )) ) {
@@ -220,14 +221,16 @@ static void EtherWeather_setroutines(struct connection_in *in)
 	in->iroutines.sendback_bits = EtherWeather_sendback_bits;
 	in->iroutines.select = NO_SELECT_ROUTINE;
 	in->iroutines.select_and_sendback = NO_SELECTANDSENDBACK_ROUTINE;
+	in->iroutines.set_config = NO_SET_CONFIG_ROUTINE;
+	in->iroutines.get_config = NO_GET_CONFIG_ROUTINE;
 	in->iroutines.reconnect = NO_RECONNECT_ROUTINE;
 	in->iroutines.close = EtherWeather_close;
 	in->iroutines.flags = ADAP_FLAG_overdrive | ADAP_FLAG_dirgulp | ADAP_FLAG_no2409path | ADAP_FLAG_no2404delay ;
 }
 
-GOOD_OR_BAD EtherWeather_detect(struct connection_in *in)
+GOOD_OR_BAD EtherWeather_detect(struct port_in *pin)
 {
-
+	struct connection_in * in = pin->first ;
 	struct parsedname pn;
 
 	FS_ParsedName_Placeholder(&pn);	// minimal parsename -- no destroy needed
@@ -238,21 +241,22 @@ GOOD_OR_BAD EtherWeather_detect(struct connection_in *in)
 	/* Set up low-level routines */
 	EtherWeather_setroutines(in);
 
-	if (SOC(in)->devicename == NULL) {
+	if (pin->init_data == NULL) {
+		LEVEL_DEFAULT("Etherweather bus master requires a port name");
 		return gbBAD;
 	}
 
-	SOC(in)->type = ct_tcp ;
+	pin->type = ct_tcp ;
 	RETURN_BAD_IF_BAD( COM_open(in) ) ;
 
 	/* TODO: probe version, and confirm that it's actually an EtherWeather */
 
-	LEVEL_CONNECT("Connected to EtherWeather at %s", SOC(in)->devicename);
+	LEVEL_CONNECT("Connected to EtherWeather at %s", DEVICENAME(in));
 
 	in->Adapter = adapter_EtherWeather;
 
 	in->adapter_name = "EtherWeather";
-	in->busmode = bus_etherweather;
+	pin->busmode = bus_etherweather;
 
 	return gbGOOD;
 }
