@@ -1,9 +1,9 @@
 /*
-$Id: ow_external.c,v 1.3 2012/04/14 19:51:08 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-	email: palfille@earthlink.net
+	email: paul.alfille@gmail.com
 	Released under the GPL
 	See the header file: ow.h for full attribution
 	1wire/iButton system from Dallas Semiconductor
@@ -30,14 +30,17 @@ static void External_setroutines(struct connection_in *in)
 	in->iroutines.sendback_bits = NO_SENDBACKBITS_ROUTINE;
 	in->iroutines.select = NO_SELECT_ROUTINE ;
 	in->iroutines.select_and_sendback = NO_SELECTANDSENDBACK_ROUTINE;
+	in->iroutines.set_config = NO_SET_CONFIG_ROUTINE;
+	in->iroutines.get_config = NO_GET_CONFIG_ROUTINE;
 	in->iroutines.reconnect = NO_RECONNECT_ROUTINE;
 	in->iroutines.close = NO_CLOSE_ROUTINE;
 	in->iroutines.flags = 0 ;
 	in->bundling_length = 1;
 }
 
-GOOD_OR_BAD External_detect(struct connection_in *in)
+GOOD_OR_BAD External_detect(struct port_in *pin)
 {
+	struct connection_in * in = pin->first ;
 	External_setroutines(in);
 	in->Adapter = adapter_external;
 	in->adapter_name = "External";
