@@ -1,5 +1,5 @@
 /*
-$Id: ow_fake.c,v 1.67 2011/07/20 02:13:16 alfille Exp $
+$Id: ow_fake.c,v 1.75 2012/10/31 01:44:48 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -28,8 +28,8 @@ static GOOD_OR_BAD Fake_sendback_data(const BYTE * data, BYTE * resp, const size
 static void GetNextByte( const ASCII ** strpointer, BYTE default_byte, BYTE * sn ) ;
 static void GetDeviceName(const ASCII ** strpointer, struct connection_in * in) ;
 static void GetDefaultDeviceName(BYTE * dn, const BYTE * sn, const struct connection_in * in) ;
-static void GetAllDeviceNames( ASCII * remaining_device_list, struct connection_in * in ) ;
-static void SetConninData( int indx, const char * name, struct connection_in *in ) ;
+static void GetAllDeviceNames( struct port_in * pin ) ;
+static void SetConninData( int indx, const char * type, struct port_in *pin ) ;
 
 static void Fake_setroutines(struct connection_in *in)
 {
@@ -42,6 +42,8 @@ static void Fake_setroutines(struct connection_in *in)
 	in->iroutines.sendback_bits = Fake_sendback_bits;
 	in->iroutines.select = NO_SELECT_ROUTINE;
 	in->iroutines.select_and_sendback = NO_SELECTANDSENDBACK_ROUTINE;
+	in->iroutines.set_config = NO_SET_CONFIG_ROUTINE;
+	in->iroutines.get_config = NO_GET_CONFIG_ROUTINE;
 	in->iroutines.reconnect = NO_RECONNECT_ROUTINE;
 	in->iroutines.close = Fake_close;
 	in->iroutines.flags = ADAP_FLAG_no2409path | ADAP_FLAG_presence_from_dirblob | ADAP_FLAG_no2404delay ;
@@ -75,7 +77,7 @@ static void GetNextByte( const ASCII ** strpointer, BYTE default_byte, BYTE * sn
 // set up default device ID
 static void GetDefaultDeviceName(BYTE * dn, const BYTE * sn, const struct connection_in * in)
 {
-	switch (in->busmode) {
+	switch (get_busmode(in)) {
 		case bus_tester:
 			// "bus number"
 			dn[1] = BYTE_MASK(in->master.tester.index >> 0) ;
@@ -141,8 +143,11 @@ static void GetDeviceName(const ASCII ** strpointer, struct connection_in * in)
 	DirblobAdd(sn, &(in->master.fake.main));	// Ignore bad return
 }
 
-static void GetAllDeviceNames( ASCII * remaining_device_list, struct connection_in * in )
+static void GetAllDeviceNames( struct port_in * pin )
 {
+	struct connection_in * in = pin->first ;
+	ASCII * remaining_device_list = owstrdup( pin->init_data ) ;
+
 	while (remaining_device_list != NULL) {
 		const ASCII *current_device_start;
 		for (current_device_start = strsep(&remaining_device_list, " ,"); current_device_start[0] != '\0'; ++current_device_start) {
@@ -153,44 +158,46 @@ static void GetAllDeviceNames( ASCII * remaining_device_list, struct connection_
 		}
 		GetDeviceName( &current_device_start, in ) ;
 	}
+	SAFEFREE( remaining_device_list ) ;
 	in->AnyDevices = (DirblobElements(&(in->master.fake.main)) > 0) ? anydevices_yes : anydevices_no ;
 }
 
-static void SetConninData( int indx, const char * name, struct connection_in *in )
+static void SetConninData( int indx, const char * type, struct port_in *pin )
 {
-	ASCII *oldname = SOC(in)->devicename; // destructively parsed and deleted at the end.
-	char newname[20] ;
+	struct connection_in * in = pin->first ;
+	char name[20] ;
 
-	SOC(in)->file_descriptor = indx;
-	SOC(in)->type = ct_none ;
+	pin->file_descriptor = indx;
+	pin->type = ct_none ;
 	in->master.fake.index = indx;
 	in->master.fake.templow = Globals.templow;
 	in->master.fake.temphigh = Globals.temphigh;
-	LEVEL_CONNECT("Setting up %s Bus Master (%d)", in->adapter_name, indx);
+	LEVEL_CONNECT("Setting up %s Bus Master (%d)", type, indx);
 
 	UCLIBCLOCK ;
-	snprintf(newname, 18, "%s.%d", name, indx);
+	snprintf(name, 18, "%s.%d", type, indx);
 	UCLIBCUNLOCK ;
 
-	GetAllDeviceNames( oldname, in ) ;
-	SAFEFREE(oldname) ;
+	GetAllDeviceNames( pin ) ;
 
-	SOC(in)->devicename = owstrdup(newname);
+	// Device name and init_data diverge now
+	DEVICENAME(in) = owstrdup(name);
 }
 
 /* Device-specific functions */
 /* Since this is simulated bus master, it's creation cannot fail */
 /* in->name starts with a list of devices which is destructovely parsed and freed */
 /* in->name end with a name-index format */
-GOOD_OR_BAD Fake_detect(struct connection_in *in)
+GOOD_OR_BAD Fake_detect(struct port_in *pin)
 {
+	struct connection_in * in = pin->first ;
 	Fake_setroutines(in);		// set up close, reconnect, reset, ...
 	in->iroutines.detect = Fake_detect;
 
 	in->adapter_name = "Simulated-Random";
 	in->Adapter = adapter_fake;
 
-	SetConninData( Inbound_Control.next_fake++, "fake", in  );
+	SetConninData( Inbound_Control.next_fake++, "fake", pin  );
 
 	return gbGOOD;
 }
@@ -199,14 +206,15 @@ GOOD_OR_BAD Fake_detect(struct connection_in *in)
 /* Since this is simulated bus master, it's creation cannot fail */
 /* in->name starts with a list of devices which is destructovely parsed and freed */
 /* in->name end with a name-index format */
-GOOD_OR_BAD Mock_detect(struct connection_in *in)
+GOOD_OR_BAD Mock_detect(struct port_in *pin)
 {
+	struct connection_in * in = pin->first ;
 	Fake_setroutines(in);		// set up close, reconnect, reset, ...
 	in->iroutines.detect = Mock_detect;
 
 	in->adapter_name = "Simulated-Mock";
 	in->Adapter = adapter_mock;
-	SetConninData( Inbound_Control.next_mock++, "mock", in  );
+	SetConninData( Inbound_Control.next_mock++, "mock", pin  );
 
 	return gbGOOD;
 }
@@ -215,14 +223,15 @@ GOOD_OR_BAD Mock_detect(struct connection_in *in)
 /* Since this is simulated bus master, it's creation cannot fail */
 /* in->name starts with a list of devices which is destructovely parsed and freed */
 /* in->name end with a name-index format */
-GOOD_OR_BAD Tester_detect(struct connection_in *in)
+GOOD_OR_BAD Tester_detect(struct port_in *pin)
 {
+	struct connection_in * in = pin->first ;
 	Fake_setroutines(in);		// set up close, reconnect, reset, ...
 	in->iroutines.detect = Tester_detect;
 
 	in->adapter_name = "Simulated-Computed";
 	in->Adapter = adapter_tester;
-	SetConninData( Inbound_Control.next_tester++, "tester", in  );
+	SetConninData( Inbound_Control.next_tester++, "tester", pin  );
 
 	return gbGOOD;
 }
