@@ -1,9 +1,9 @@
 /*
-$Id: ow_ha5.c,v 1.50 2011/07/20 02:13:16 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-	email: palfille@earthlink.net
+	email: paul.alfille@gmail.com
 	Released under the GPL
 	See the header file: ow.h for full attribution
 	1wire/iButton system from Dallas Semiconductor
@@ -44,11 +44,6 @@ static GOOD_OR_BAD HA5_write( char command, char * raw_string, int length, struc
 static GOOD_OR_BAD HA5_sendback_bits(const BYTE * outbits, BYTE * inbits, const size_t length, const struct parsedname *pn) ;
 static GOOD_OR_BAD HA5_bit( BYTE send, BYTE * receive, struct connection_in * in ) ;
 
-#define HA5MUTEX_INIT(in)		_MUTEX_INIT(in->master.ha5.all_channel_lock)
-#define HA5MUTEX_LOCK(in)		_MUTEX_LOCK(in->master.ha5.all_channel_lock ) ;
-#define HA5MUTEX_UNLOCK(in)		_MUTEX_UNLOCK(in->master.ha5.all_channel_lock);
-#define HA5MUTEX_DESTROY(in)	_MUTEX_DESTROY(in->master.ha5.all_channel_lock);
-
 #define CR_char		(0x0D)
 
 static void HA5_setroutines(struct connection_in *in)
@@ -62,14 +57,17 @@ static void HA5_setroutines(struct connection_in *in)
 	in->iroutines.sendback_bits = HA5_sendback_bits;
 	in->iroutines.select = HA5_select ;
 	in->iroutines.select_and_sendback = HA5_select_and_sendback;
+	in->iroutines.set_config = NO_SET_CONFIG_ROUTINE;
+	in->iroutines.get_config = NO_GET_CONFIG_ROUTINE;
 	in->iroutines.reconnect = HA5_reconnect;
 	in->iroutines.close = HA5_close;
 	in->iroutines.flags = ADAP_FLAG_dirgulp | ADAP_FLAG_bundle | ADAP_FLAG_dir_auto_reset | ADAP_FLAG_no2404delay | ADAP_FLAG_presence_from_dirblob ;
 	in->bundling_length = HA5_FIFO_SIZE;
 }
 
-GOOD_OR_BAD HA5_detect(struct connection_in *in)
+GOOD_OR_BAD HA5_detect(struct port_in *pin)
 {
+	struct connection_in * in = pin->first ;
 	struct address_pair ap ;
 	GOOD_OR_BAD gbResult ; // default
 
@@ -78,24 +76,28 @@ GOOD_OR_BAD HA5_detect(struct connection_in *in)
 
 	/* By definition, this is the head adapter on this port */
 	in->master.ha5.head = in ;
-	HA5MUTEX_INIT(in); // closed in HA5_close
 
 	// Poison current "Address" for adapter
-	memset( in->master.ha5.sn, 0x00, SERIAL_NUMBER_SIZE ) ;
+	memset( in->remembered_sn, 0x00, SERIAL_NUMBER_SIZE ) ;
+
+	if (pin->init_data == NULL) {
+		LEVEL_DEFAULT("HA5 bus master requires port name");
+		return gbBAD;
+	}
 
 	COM_set_standard( in ) ; // standard COM port settings
 
 	// 9600 isn't valid for the HA5, so we can tell that this value was actually selected
 	if ( Globals.baud != B9600 ) {
-		SOC(in)->baud = Globals.baud ;
+		pin->baud = Globals.baud ;
 	} else {
-		SOC(in)->baud = B115200 ; // works at this fast rate
+		pin->baud = B115200 ; // works at this fast rate
 	}
 
 	// allowable speeds
-	COM_BaudRestrict( &(SOC(in)->baud), B1200, B19200, B38400, B115200, 0 ) ;
+	COM_BaudRestrict( &(pin->baud), B1200, B19200, B38400, B115200, 0 ) ;
 
-	Parse_Address( SOC(in)->devicename, &ap ) ;
+	Parse_Address( DEVICENAME(in), &ap ) ;
 
 	gbResult = HA5_detect_parsed( &ap, in) ;
 
@@ -112,24 +114,23 @@ GOOD_OR_BAD HA5_detect(struct connection_in *in)
 // Detect now that the name has been parsed
 GOOD_OR_BAD HA5_detect_parsed(struct address_pair *ap, struct connection_in *in)
 {
-	GOOD_OR_BAD gbResult ;
 	char * channel_list = NULL ;
 
 	// Create name (ip:port or /dev/tty) and optional channel list
-	switch (SOC(in)->type) {
+	switch (in->pown->type) {
 		case ct_telnet:
 			// address:port:channel_list
 			if ( ap->first.type != address_none ) {
 				// IP address exists
-				strcpy(SOC(in)->devicename, ap->first.alpha) ; // subset so will fit
+				strcpy(DEVICENAME(in), ap->first.alpha) ; // subset so will fit
 				if ( ap->second.type != address_none ) {
 					// Add :port
-					strcat(SOC(in)->devicename, ":") ; // subset so will fit
-					strcat(SOC(in)->devicename, ap->second.alpha) ; // subset so will fit
+					strcat(DEVICENAME(in), ":") ; // subset so will fit
+					strcat(DEVICENAME(in), ap->second.alpha) ; // subset so will fit
 				}
 			} else if ( ap->second.type != address_none ) {
 				// just :port
-				strcpy(SOC(in)->devicename, ap->second.alpha) ; // subset so will fit
+				strcpy(DEVICENAME(in), ap->second.alpha) ; // subset so will fit
 			} else {
 				// No tcp address or port specified
 				return gbBAD ;
@@ -153,7 +154,7 @@ GOOD_OR_BAD HA5_detect_parsed(struct address_pair *ap, struct connection_in *in)
 			// device:channel_list
 			if ( ap->first.type != address_none ) {
 				// device address exists
-				strcpy(SOC(in)->devicename, ap->first.alpha) ; // subset so will fit
+				strcpy(DEVICENAME(in), ap->first.alpha) ; // subset so will fit
 			} else {
 				// No serial port specified
 				return gbBAD ;
@@ -184,12 +185,17 @@ GOOD_OR_BAD HA5_detect_parsed(struct address_pair *ap, struct connection_in *in)
 
 	/* Find the channels */
 	if ( channel_list == NULL ) { // scan for channel
-		gbResult = HA5_find_channel(in) ;
+		if ( BAD( HA5_find_channel(in) ) ) {
+			RETURN_BAD_IF_BAD( serial_powercycle(in) ) ;
+			return HA5_find_channel(in) ;
+		}
 	} else { // A list of channels
-		gbResult = HA5_channel_list( channel_list, in ) ;
+		if ( BAD( HA5_channel_list( channel_list, in ) ) ) {
+			RETURN_BAD_IF_BAD( serial_powercycle(in) ) ;
+			return HA5_channel_list( channel_list, in ) ;
+		}
 	}
-
-	return gbResult;
+	return gbGOOD ;
 }
 
 static GOOD_OR_BAD HA5_channel_list( char * alpha_string, struct connection_in * initial_in )
@@ -206,7 +212,7 @@ static GOOD_OR_BAD HA5_channel_list( char * alpha_string, struct connection_in *
 		}
 
 		current_in->master.ha5.channel = tolower(c) ;
-		LEVEL_DEBUG("Looking for HA5 adapter on %s:%c", SOC(current_in)->devicename, current_in->master.ha5.channel ) ;
+		LEVEL_DEBUG("Looking for HA5 adapter on %s:%c", DEVICENAME(current_in), current_in->master.ha5.channel ) ;
 
 		if (
 		GOOD( HA5_test_channel(current_in))
@@ -216,18 +222,16 @@ static GOOD_OR_BAD HA5_channel_list( char * alpha_string, struct connection_in *
 			// work around for new USB serial port that seems to need a repeat test
 			first_time = 0 ;
 
-			LEVEL_CONNECT("HA5 bus master FOUND on port %s at channel %c", SOC(current_in)->devicename, current_in->master.ha5.channel ) ;
-			current_in = NewIn( initial_in ) ; // point to newer space for next candidate
+			LEVEL_CONNECT("HA5 bus master FOUND on port %s at channel %c", DEVICENAME(current_in), current_in->master.ha5.channel ) ;
+			current_in = AddtoPort( initial_in->pown ) ; // point to newer space for next candidate
 
 			if ( current_in == NO_CONNECTION ) {
 				break ;
 			}
-			// Only head has a valid file descriptor
-			SOC(current_in)->file_descriptor = FILE_DESCRIPTOR_BAD ;
 			continue ;
 		}
 
-		LEVEL_CONNECT("HA5 bus master NOT FOUND on port %s at channel %c", SOC(current_in)->devicename, c ) ;
+		LEVEL_CONNECT("HA5 bus master NOT FOUND on port %s at channel %c", DEVICENAME(current_in), c ) ;
 	}
 	
 	// Now see if we've added any adapters
@@ -255,13 +259,13 @@ static GOOD_OR_BAD HA5_find_channel( struct connection_in * in )
 		// test char set
 		in->master.ha5.channel = c[0] ;
 
-		LEVEL_DEBUG("Looking for HA5 adapter on %s:%c", SOC(in)->devicename, in->master.ha5.channel ) ;
+		LEVEL_DEBUG("Looking for HA5 adapter on %s:%c", DEVICENAME(in), in->master.ha5.channel ) ;
 		if ( GOOD( HA5_test_channel(in)) ) {
-			LEVEL_CONNECT("HA5 bus master FOUND on port %s at channel %c", SOC(in)->devicename, in->master.ha5.channel ) ;
+			LEVEL_CONNECT("HA5 bus master FOUND on port %s at channel %c", DEVICENAME(in), in->master.ha5.channel ) ;
 			return gbGOOD ;
 		}
 	}
-	LEVEL_DEBUG("HA5 bus master NOT FOUND on port %s", SOC(in)->devicename ) ;
+	LEVEL_DEBUG("HA5 bus master NOT FOUND on port %s", DEVICENAME(in) ) ;
 	in->master.ha5.channel = '\0' ; 
 	return gbBAD;
 }
@@ -323,8 +327,7 @@ static GOOD_OR_BAD TestChecksum( unsigned char * check_string, int length )
 static GOOD_OR_BAD HA5_reconnect( const struct parsedname  *pn )
 {
 	struct connection_in * in = pn->selected_connection ;
-	COM_close(in->master.ha5.head ) ;
-	RETURN_BAD_IF_BAD( COM_open(in->master.ha5.head ) ) ;
+	RETURN_BAD_IF_BAD(serial_powercycle(in->master.ha5.head)) ;
 	return HA5_test_channel( in ) ;
 }
 
@@ -372,7 +375,7 @@ static GOOD_OR_BAD HA5_checksum_support( struct connection_in * in )
 
 	if ( resp[2] == 0x0D ) {
 		in->master.ha5.checksum = 0 ;
-		LEVEL_DETAIL("HA5 %s in NON-CHECKSUM mode",SAFESTRING(SOC(in)->devicename));
+		LEVEL_DETAIL("HA5 %s in NON-CHECKSUM mode",SAFESTRING(DEVICENAME(in)));
 		return gbGOOD ;
 	} else if ( resp[2] == 'X' ) {
 		// nothing actually read
@@ -389,7 +392,7 @@ static GOOD_OR_BAD HA5_checksum_support( struct connection_in * in )
 		return gbBAD ;
 	}
 	in->master.ha5.checksum = 1 ;
-	LEVEL_DETAIL("HA5 %s in CHECKSUM mode",SAFESTRING(SOC(in)->devicename));
+	LEVEL_DETAIL("HA5 %s in CHECKSUM mode",SAFESTRING(DEVICENAME(in)));
 	return gbGOOD ;
 }
 
@@ -400,13 +403,7 @@ static RESET_TYPE HA5_reset(const struct parsedname *pn)
 
 static RESET_TYPE HA5_reset_in( struct connection_in * in )
 {
-	RESET_TYPE ret ;
-
-	HA5MUTEX_LOCK( in->master.ha5.head ) ;
-	ret = HA5_reset_wrapped(in) ;
-	HA5MUTEX_UNLOCK( in->master.ha5.head ) ;
-
-	return ret ;
+	return HA5_reset_wrapped(in) ;
 }
 
 static RESET_TYPE HA5_reset_wrapped( struct connection_in * in )
@@ -448,12 +445,7 @@ static enum search_status HA5_next_both(struct device_search *ds, const struct p
 	HA5_flush(in);
 
 	if (ds->index == -1) {
-		enum search_status ret ;
-		HA5MUTEX_LOCK( in->master.ha5.head ) ;
-		ret = HA5_directory(ds, pn) ;
-		HA5MUTEX_UNLOCK( in->master.ha5.head ) ;
-
-		if ( ret != search_good ) {
+		if ( HA5_directory(ds, pn) != search_good ) {
 			return search_error;
 		}
 	}
@@ -552,7 +544,7 @@ static enum search_status HA5_directory(struct device_search *ds, const struct p
 		sn[0] = string2num((char *)&resp[14]);
 
 		// Set as current "Address" for adapter
-		memcpy( in->master.ha5.sn, sn, 8) ;
+		memcpy( in->remembered_sn, sn, SERIAL_NUMBER_SIZE ) ;
 
 		LEVEL_DEBUG("SN found: " SNformat, SNvar(sn));
 		// CRC check
@@ -572,28 +564,22 @@ static enum search_status HA5_directory(struct device_search *ds, const struct p
 static GOOD_OR_BAD HA5_resync( struct connection_in * in )
 {
 	HA5_flush(in->master.ha5.head );
-	HA5_reset_in(in);
+	HA5_reset_wrapped(in);
 	HA5_flush(in->master.ha5.head );
 
 	// Poison current "Address" for adapter
-	in->master.ha5.sn[0] = 0 ; // so won't match
+	memset( in->remembered_sn, 0, SERIAL_NUMBER_SIZE ) ; // so won't match
 
 	return gbBAD ;
 }
 
 static GOOD_OR_BAD HA5_select( const struct parsedname * pn )
 {
-	GOOD_OR_BAD ret ;
-
 	if ( (pn->selected_device==NO_DEVICE) || (pn->selected_device==DeviceThermostat) ) {
 		RETURN_BAD_IF_BAD( gbRESET( HA5_reset_in(pn->selected_connection) ) ) ;
 	}
 
-	HA5MUTEX_LOCK( pn->selected_connection->master.ha5.head ) ;
-	ret = HA5_select_wrapped(pn) ;
-	HA5MUTEX_UNLOCK( pn->selected_connection->master.ha5.head ) ;
-
-	return ret ;
+	return HA5_select_wrapped(pn) ;
 }
 
 static GOOD_OR_BAD HA5_select_wrapped( const struct parsedname * pn )
@@ -637,7 +623,7 @@ static GOOD_OR_BAD HA5_select_wrapped( const struct parsedname * pn )
 	}
 
 	// Set as current "Address" for adapter
-	memcpy( in->master.ha5.sn, pn->sn, SERIAL_NUMBER_SIZE) ;
+	memcpy( in->remembered_sn, pn->sn, SERIAL_NUMBER_SIZE) ;
 
 	return gbGOOD ;
 }
@@ -685,15 +671,10 @@ static GOOD_OR_BAD HA5_sendback_data(const BYTE * data, BYTE * resp, const size_
 	struct connection_in * in = pn->selected_connection ;
 
 	for ( left=size ; left>0 ; left -= 32 ) {
-		GOOD_OR_BAD ret ;
 		size_t pass_start = size - left ;
 		size_t pass_size = (left>32)?32:left ;
 
-		HA5MUTEX_LOCK( in->master.ha5.head ) ;
-		ret = HA5_sendback_part( 'W', &data[pass_start], &resp[pass_start], pass_size, in ) ;
-		HA5MUTEX_UNLOCK( in->master.ha5.head ) ;
-
-		RETURN_BAD_IF_BAD( ret ) ;
+		RETURN_BAD_IF_BAD( HA5_sendback_part( 'W', &data[pass_start], &resp[pass_start], pass_size, in ) ) ;
 	}
 	return gbGOOD;
 }
@@ -704,7 +685,7 @@ static GOOD_OR_BAD HA5_select_and_sendback(const BYTE * data, BYTE * resp, const
 	int left;
 	char block_cmd ;
 
-	if ( memcmp( pn->sn, in->master.ha5.sn, SERIAL_NUMBER_SIZE ) ) {
+	if ( memcmp( pn->sn, in->remembered_sn, SERIAL_NUMBER_SIZE ) ) {
 		// Need a formal change of device
 		RETURN_BAD_IF_BAD( HA5_select(pn) ) ;
 		block_cmd = 'W' ;
@@ -718,9 +699,7 @@ static GOOD_OR_BAD HA5_select_and_sendback(const BYTE * data, BYTE * resp, const
 		size_t pass_start = size - left ;
 		size_t pass_size = (left>32)?32:left ;
 
-		HA5MUTEX_LOCK( in->master.ha5.head ) ;
 		ret = HA5_sendback_part( block_cmd, &data[pass_start], &resp[pass_start], pass_size, in ) ;
-		HA5MUTEX_UNLOCK( in->master.ha5.head ) ;
 		block_cmd = 'W' ; // for next pass
 		RETURN_BAD_IF_BAD( ret ) ;
 	}
@@ -736,14 +715,9 @@ static GOOD_OR_BAD HA5_sendback_bits(const BYTE * outbits, BYTE * inbits, const
 	size_t counter ;
 	struct connection_in * in = pn->selected_connection ;
 
-	HA5MUTEX_LOCK( in->master.ha5.head ) ;
 	for ( counter = 0 ; counter < length ; ++counter ) {
-		if ( BAD( HA5_bit( outbits[counter], &inbits[counter], in ) ) ) {
-			HA5MUTEX_UNLOCK( in->master.ha5.head ) ;
-			return gbBAD ;
-		}		
+		RETURN_BAD_IF_BAD( HA5_bit( outbits[counter], &inbits[counter], in ) ) ;
 	} 
-	HA5MUTEX_UNLOCK( in->master.ha5.head ) ;
 
 	return gbGOOD;
 }
@@ -784,9 +758,8 @@ static GOOD_OR_BAD HA5_bit( BYTE outbit, BYTE * inbit, struct connection_in * in
 static void HA5_close(struct connection_in *in)
 {
 	if ( in->master.ha5.head == in ) {
-		HA5MUTEX_DESTROY(in);
 	} else {
-		SOC(in)->state = cs_virgin ;
+		in->pown->state = cs_virgin ;
 	}
 }
 
