@@ -1,9 +1,8 @@
 /*
-$Id: ow_ha7.c,v 1.73 2011/07/20 02:13:16 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-	email: palfille@earthlink.net
+	email: paul.alfille@gmail.com
 	Released under the GPL
 	See the header file: ow.h for full attribution
 	1wire/iButton system from Dallas Semiconductor
@@ -16,7 +15,6 @@ $Id: ow_ha7.c,v 1.73 2011/07/20 02:13:16 alfille Exp $
 #include "ow_connection.h"
 #include "ow_codes.h"
 
-#if OW_HA7
 struct toHA7 {
 	ASCII *command;
 	ASCII lock[10];
@@ -54,14 +52,18 @@ static void HA7_setroutines(struct connection_in *in)
 	in->iroutines.sendback_bits = NO_SENDBACKBITS_ROUTINE;
 	in->iroutines.select = HA7_select;
 	in->iroutines.select_and_sendback = NO_SELECTANDSENDBACK_ROUTINE;
+	in->iroutines.set_config = NO_SET_CONFIG_ROUTINE;
+	in->iroutines.get_config = NO_GET_CONFIG_ROUTINE;
 	in->iroutines.reconnect = NO_RECONNECT_ROUTINE;
 	in->iroutines.close = HA7_close;
+	in->iroutines.verify = NO_VERIFY_ROUTINE ;
 	in->iroutines.flags = ADAP_FLAG_dirgulp | ADAP_FLAG_bundle | ADAP_FLAG_dir_auto_reset | ADAP_FLAG_no2404delay ;
 	in->bundling_length = HA7_FIFO_SIZE;	// arbitrary number
 }
 
-GOOD_OR_BAD HA7_detect(struct connection_in *in)
+GOOD_OR_BAD HA7_detect(struct port_in *pin)
 {
+	struct connection_in * in = pin->first ;
 	struct parsedname pn;
 	struct toHA7 ha7;
 
@@ -73,13 +75,13 @@ GOOD_OR_BAD HA7_detect(struct connection_in *in)
 
 	in->master.ha7.locked = 0;
 
-	if (SOC(in)->devicename == NULL) {
+	if (pin->init_data == NULL) {
 		return gbBAD;
 	}
 
-	SOC(in)->type = ct_tcp ;
-	SOC(in)->timeout.tv_sec = Globals.timeout_ha7 ;
-	SOC(in)->timeout.tv_usec = 0 ;
+	pin->type = ct_tcp ;
+	pin->timeout.tv_sec = Globals.timeout_ha7 ;
+	pin->timeout.tv_usec = 0 ;
 	RETURN_BAD_IF_BAD( COM_open(in) ) ;
 
 	in->Adapter = adapter_HA7NET;
@@ -90,7 +92,18 @@ GOOD_OR_BAD HA7_detect(struct connection_in *in)
 		struct memblob mb;
 		if ( GOOD( HA7_read( &mb, in )) ) {
 			in->adapter_name = "HA7Net";
-			in->busmode = bus_ha7net;
+			pin->busmode = bus_ha7net;
+			in->AnyDevices = anydevices_yes;
+			MemblobClear(&mb);
+			return gbGOOD;
+		}
+	}
+	serial_powercycle(in) ;
+	if (GOOD( HA7_toHA7( &ha7, in)) ) {
+		struct memblob mb;
+		if ( GOOD( HA7_read( &mb, in )) ) {
+			in->adapter_name = "HA7Net";
+			pin->busmode = bus_ha7net;
 			in->AnyDevices = anydevices_yes;
 			MemblobClear(&mb);
 			return gbGOOD;
@@ -102,8 +115,6 @@ GOOD_OR_BAD HA7_detect(struct connection_in *in)
 
 static RESET_TYPE HA7_reset(const struct parsedname *pn)
 {
-	struct memblob mb;
-	RESET_TYPE ret = BUS_RESET_OK;
 	struct toHA7 ha7;
 	struct connection_in * in = pn->selected_connection ;
 
@@ -111,13 +122,17 @@ static RESET_TYPE HA7_reset(const struct parsedname *pn)
 	ha7.command = "Reset";
 	if ( BAD(HA7_toHA7( &ha7, in)) ) {
 		LEVEL_DEBUG("Trouble sending reset command");
-		ret = BUS_RESET_ERROR;
-	} else if ( BAD(HA7_read( &mb, in )) ) {
-		LEVEL_DEBUG("Trouble with reset command response");
-		ret = BUS_RESET_ERROR;
+		return BUS_RESET_ERROR;
+	} else {
+		RESET_TYPE ret = BUS_RESET_OK;
+		struct memblob mb;
+		if ( BAD(HA7_read( &mb, in )) ) {
+			LEVEL_DEBUG("Trouble with reset command response");
+			ret = BUS_RESET_ERROR;
+		}
+		MemblobClear(&mb);
+		return ret;
 	}
-	MemblobClear(&mb);
-	return ret;
 }
 
 static GOOD_OR_BAD HA7_directory( struct device_search *ds, const struct parsedname *pn)
@@ -196,16 +211,18 @@ static enum search_status HA7_next_both(struct device_search *ds, const struct p
 
 static GOOD_OR_BAD HA7_read( struct memblob *mb, struct connection_in * in )
 {
+	struct port_in * pin = in->pown ;
+	
 	ASCII readin_area[HA7_READ_BUFFER_LENGTH + 1];
 	ASCII *start;
-	size_t read_size;
+	ssize_t read_size;
 
 	MemblobInit(mb, HA7_READ_BUFFER_LENGTH);
-	SOC(in)->timeout.tv_sec = 2 ;
-	SOC(in)->timeout.tv_usec = 0 ;
+	pin->timeout.tv_sec = 2 ;
+	pin->timeout.tv_usec = 0 ;
 
 	// Read first block of data from HA7
-	read_size = COM_read_size( (BYTE*) readin_area, HA7_READ_BUFFER_LENGTH, in) ;
+	read_size = COM_read_with_timeout( (BYTE*) readin_area, HA7_READ_BUFFER_LENGTH, in) ;
 	if ( read_size <= 0 ) {
 		LEVEL_CONNECT("Read error");
 		return gbBAD;
@@ -235,7 +252,7 @@ static GOOD_OR_BAD HA7_read( struct memblob *mb, struct connection_in * in )
 	}
 	// loop through reading in HA7_READ_BUFFER_LENGTH blocks
 	while (read_size == HA7_READ_BUFFER_LENGTH) {	// full read, so presume more waiting
-		read_size = COM_read_size( (BYTE*) readin_area, HA7_READ_BUFFER_LENGTH, in) ;
+		read_size = COM_read_with_timeout( (BYTE*) readin_area, HA7_READ_BUFFER_LENGTH, in) ;
 		if (read_size <= 0) {
 			LEVEL_DATA("Couldn't get rest of HA7 data (err=%d)", read_size);
 			MemblobClear(mb);
@@ -291,7 +308,7 @@ static GOOD_OR_BAD HA7_toHA7( const struct toHA7 *ha7, struct connection_in *in)
 	strcat(full_command, ".html");
 
 	if (ha7->address[0]) {
-		strcat(full_command, first ? "?" : "&");
+		strcat(full_command, "?" ); // first (if exists)
 		strcat(full_command, "Address=");
 		strcat(full_command, ha7->address);
 		first = 0;
@@ -412,6 +429,7 @@ static GOOD_OR_BAD HA7_sendback_block(const BYTE * data, BYTE * resp, const size
 		} else {
 			STAT_ADD1_BUS(e_bus_read_errors, in);
 		}
+		return ret ;
 	}
 	return gbGOOD;
 }
@@ -461,5 +479,3 @@ static void toHA7init(struct toHA7 *ha7)
 {
 	memset(ha7, 0, sizeof(struct toHA7));
 }
-
-#endif							/* OW_HA7 */
