@@ -1,5 +1,5 @@
 /*
-$Id: ow_help.c,v 1.56 2011/05/06 19:16:21 alfille Exp $
+$Id: ow_help.c,v 1.63 2012/10/01 17:56:10 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -17,15 +17,16 @@ $Id: ow_help.c,v 1.56 2011/05/06 19:16:21 alfille Exp $
 
 void ow_help_general(void)
 {
-	switch (Globals.opt) {
-	case opt_owfs:
+	switch (Globals.program_type) {
+	case program_type_filesystem:
 		printf("Syntax: %s [options] device mountpoint\n", SAFESTRING(Globals.progname));
 		break;
-	case opt_httpd:
-	case opt_server:
+	case program_type_httpd:
+	case program_type_server:
+	case program_type_external:
 		printf("Syntax: %s [options] device clientport\n", SAFESTRING(Globals.progname));
 		break;
-	case opt_ftpd:
+	case program_type_ftpd:
 	default:
 		printf("Syntax: %s [options] device\n", SAFESTRING(Globals.progname));
 		break;
@@ -34,6 +35,7 @@ void ow_help_general(void)
 	"Help resources:\n"
 	" %s --help              This page\n"
 	" %s --help=device       Bus master device options\n"
+	" %s --help=error        List of error return codes\n"
 	" %s --help=program      Program services (mountpoint, port)\n"
 	" %s --help=cache        Cache and communication timing\n"
 	" %s --help=job          Job control and debugging\n"
@@ -42,7 +44,7 @@ void ow_help_general(void)
 	" man %s                 man page for this program\n"
 	"  and man pages for individual 1-wire devices e.g. 'man DS2409'\n",
 	SAFESTRING(Globals.progname), SAFESTRING(Globals.progname),
-	SAFESTRING(Globals.progname), SAFESTRING(Globals.progname),
+	SAFESTRING(Globals.progname), SAFESTRING(Globals.progname), SAFESTRING(Globals.progname),
 	SAFESTRING(Globals.progname), SAFESTRING(Globals.progname), SAFESTRING(Globals.progname)
 	);
 }
@@ -137,6 +139,15 @@ void ow_help_cache(void)
 		   );
 }
 
+void ow_help_error(void)
+{
+	int return_code ;
+	printf("Error return codes Help\n");
+	for ( return_code=0 ; return_code < N_RETURN_CODES ; ++return_code ) {
+		printf("%3d. %s\n",return_code,return_code_strings[return_code]);
+	}
+}
+
 void ow_help_program(void)
 {
 	printf("Program-Specific Help\n"
@@ -215,7 +226,9 @@ void ow_help_device(void)
 	"  --LINK=address  LINK-HUB-E network LINK\n"
 	"  --HA7NET=address HA7NET bus master\n"
 	"  --HA7NET        HA7NET bus master address auto-discovered\n"
-	"  --ENET=address  OWServer-Enet bus master\n"
+	"  --ENET=address  OWServer-Enet or ENET2 bus master\n"
+	"  --ENET          OWServer-Enet or ENET2 automatically discovered\n"
+//	"  --ENET=scan     Keep looking for ENET or ENET2 bus masters\n"
 	"  --etherweather=address EtherWeather\n"
 	"  --autoserver    Find owserver using zeroconf\n"
 	"\n"
@@ -228,7 +241,12 @@ void ow_help_device(void)
 	"\n"
 	" Linux Kernel Device\n"
 	"  --w1            Scan for kernel-managed bus masters\n"
-	"\n" " 1-wire device selection\n" "  --one-device     Only single device on bus, use ROM SKIP command\n");
+	"\n" 
+	"  --external      Allow external scripts to be called\n"
+	"  --no_external   Do not allow external scripts to be called\n"
+	
+	"\n" 
+	" 1-wire device selection\n" "  --one-device     Only single device on bus, use ROM SKIP command\n");
 }
 
 void FS_help(const char *arg)
@@ -240,6 +258,10 @@ void FS_help(const char *arg)
 		case 'D':
 			ow_help_device();
 			break;
+		case 'e':
+		case 'E':
+			ow_help_error();
+			break;
 		case 'p':
 		case 'P':
 			ow_help_program();
