@@ -1,5 +1,5 @@
 /*
-$Id: ow_interface.c,v 1.50 2012/04/07 17:45:51 alfille Exp $
+$Id: ow_interface.c,v 1.55 2012/10/09 00:46:47 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -22,9 +22,7 @@ $Id: ow_interface.c,v 1.50 2012/04/07 17:45:51 alfille Exp $
 READ_FUNCTION(FS_name);
 READ_FUNCTION(FS_port);
 READ_FUNCTION(FS_version);
-READ_FUNCTION(FS_r_overdrive);
 READ_FUNCTION(FS_r_channel);
-WRITE_FUNCTION(FS_w_overdrive);
 READ_FUNCTION(FS_r_yesno);
 WRITE_FUNCTION(FS_w_yesno);
 READ_FUNCTION(FS_r_reconnect);
@@ -59,7 +57,7 @@ int DS9490_getstatus(BYTE * buffer, int readlen, const struct parsedname *pn);
 
 static enum e_visibility VISIBLE_DS2482( const struct parsedname * pn )
 {
-	switch ( pn->selected_connection->busmode ) {
+	switch ( get_busmode(pn->selected_connection) ) {
 		case bus_i2c:
 			return visible_now ;
 		default:
@@ -69,7 +67,7 @@ static enum e_visibility VISIBLE_DS2482( const struct parsedname * pn )
 
 static enum e_visibility VISIBLE_DS9490( const struct parsedname * pn )
 {
-	switch ( pn->selected_connection->busmode ) {
+	switch ( get_busmode(pn->selected_connection) ) {
 		case bus_usb:
 			return visible_now ;
 		default:
@@ -79,7 +77,7 @@ static enum e_visibility VISIBLE_DS9490( const struct parsedname * pn )
 
 static enum e_visibility VISIBLE_DS2480B( const struct parsedname * pn )
 {
-	switch ( pn->selected_connection->busmode ) {
+	switch ( get_busmode(pn->selected_connection) ) {
 		case bus_serial:
 		case bus_xport:
 			return visible_now ;
@@ -90,7 +88,7 @@ static enum e_visibility VISIBLE_DS2480B( const struct parsedname * pn )
 
 static enum e_visibility VISIBLE_HA5( const struct parsedname * pn )
 {
-	switch ( pn->selected_connection->busmode ) {
+	switch ( get_busmode(pn->selected_connection) ) {
 		case bus_ha5:
 			return visible_now ;
 		default:
@@ -100,7 +98,7 @@ static enum e_visibility VISIBLE_HA5( const struct parsedname * pn )
 
 static enum e_visibility VISIBLE_PSEUDO( const struct parsedname * pn )
 {
-	switch ( pn->selected_connection->busmode ) {
+	switch ( get_busmode(pn->selected_connection) ) {
 		case bus_fake:
 		case bus_tester:
 		case bus_mock:
@@ -222,7 +220,7 @@ static ZERO_OR_ERROR FS_w_reconnect(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_r_APU(struct one_wire_query *owq)
 {
 	struct parsedname *pn = PN(owq);
-	switch ( pn->selected_connection->busmode ) {
+	switch ( get_busmode(pn->selected_connection) ) {
 		case bus_i2c:
 			OWQ_Y(owq) = ( (pn->selected_connection->master.i2c.configreg & DS2482_REG_CFG_APU) != 0x00 ) ;
 			return 0;
@@ -234,7 +232,7 @@ static ZERO_OR_ERROR FS_r_APU(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_w_APU(struct one_wire_query *owq)
 {
 	struct parsedname *pn = PN(owq);
-	switch ( pn->selected_connection->busmode ) {
+	switch ( get_busmode(pn->selected_connection) ) {
 		case bus_i2c:
 			if ( OWQ_Y(owq) ) {
 				pn->selected_connection->master.i2c.configreg |= DS2482_REG_CFG_APU ;
@@ -295,7 +293,7 @@ static ZERO_OR_ERROR FS_w_yesno( struct one_wire_query * owq )
 static ZERO_OR_ERROR FS_r_templimit(struct one_wire_query *owq)
 {
 	struct parsedname *pn = PN(owq);
-	switch ( pn->selected_connection->busmode ) {
+	switch ( get_busmode(pn->selected_connection) ) {
 		case bus_fake:
 		case bus_mock:
 		case bus_tester:
@@ -309,7 +307,7 @@ static ZERO_OR_ERROR FS_r_templimit(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_w_templimit(struct one_wire_query *owq)
 {
 	struct parsedname *pn = PN(owq);
-	switch ( pn->selected_connection->busmode ) {
+	switch ( get_busmode(pn->selected_connection) ) {
 		case bus_fake:
 		case bus_mock:
 		case bus_tester:
@@ -329,12 +327,12 @@ static ZERO_OR_ERROR FS_w_templimit(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_r_baud(struct one_wire_query *owq)
 {
 	struct connection_in * in = PN(owq)->selected_connection ;
-	switch ( in->busmode ) {
+	switch ( get_busmode(in) ) {
 		case bus_serial:
 		case bus_link:
 		case bus_ha5:
 		case bus_ha7e:
-			OWQ_U(owq) = COM_BaudRate( SOC(in)->baud ) ;
+			OWQ_U(owq) = COM_BaudRate( in->pown->baud ) ;
 			return 0;
 		default:
 			return -ENOTSUP ;
@@ -344,10 +342,10 @@ static ZERO_OR_ERROR FS_r_baud(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_w_baud(struct one_wire_query *owq)
 {
 	struct connection_in * in = PN(owq)->selected_connection ;
-	switch ( in->busmode ) {
+	switch ( get_busmode(in) ) {
 		case bus_serial:
 		case bus_link:
-			SOC(in)->baud = COM_MakeBaud( (speed_t) OWQ_U(owq) ) ;
+			in->pown->baud = COM_MakeBaud( (speed_t) OWQ_U(owq) ) ;
 			++in->changed_bus_settings ;
 			break ;
 		default:
@@ -361,7 +359,7 @@ static ZERO_OR_ERROR FS_w_baud(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_r_PPM(struct one_wire_query *owq)
 {
 	struct connection_in * in = PN(owq)->selected_connection ;
-	switch ( in->busmode ) {
+	switch ( get_busmode(in) ) {
 		case bus_i2c:
 			OWQ_Y(owq) = ( (in->master.i2c.configreg & DS2482_REG_CFG_PPM) != 0x00 ) ;
 			return 0;
@@ -373,7 +371,7 @@ static ZERO_OR_ERROR FS_r_PPM(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_w_PPM(struct one_wire_query *owq)
 {
 	struct connection_in * in = PN(owq)->selected_connection ;
-	switch ( in->busmode ) {
+	switch ( get_busmode(in) ) {
 		case bus_i2c:
 			if ( OWQ_Y(owq) ) {
 				in->master.i2c.configreg |= DS2482_REG_CFG_PPM ;
@@ -393,22 +391,6 @@ static ZERO_OR_ERROR FS_r_channel(struct one_wire_query *owq)
 	return OWQ_format_output_offset_and_size( (char *) &(PN(owq)->selected_connection->master.ha5.channel), 1, owq);
 }
 
-/* Just some tests to support overdrive */
-static ZERO_OR_ERROR FS_r_overdrive(struct one_wire_query *owq)
-{
-	struct parsedname *pn = PN(owq);
-	OWQ_Y(owq) = pn->selected_connection->overdrive ;
-	return 0;
-}
-
-static ZERO_OR_ERROR FS_w_overdrive(struct one_wire_query *owq)
-{
-	struct parsedname *pn = PN(owq);
-	pn->selected_connection->overdrive = OWQ_Y(owq) ;
-	pn->selected_connection->changed_bus_settings |= CHANGED_USB_SPEED ;
-	return 0;
-}
-
 #ifdef DEBUG_DS2490
 static ZERO_OR_ERROR FS_r_ds2490status(struct one_wire_query *owq)
 {
@@ -417,7 +399,7 @@ static ZERO_OR_ERROR FS_r_ds2490status(struct one_wire_query *owq)
 	char buffer[ DS9490_getstatus_BUFFER_LENGTH ];
 	int ret;
 	res[0] = '\0';
-	if (pn->selected_connection->busmode == bus_usb) {
+	if (get_busmode(pn->selected_connection) == bus_usb) {
 #if OW_USB
 		ret = DS9490_getstatus(buffer, 0, PN(owq));
 		if (ret < 0) {
@@ -467,7 +449,7 @@ static ZERO_OR_ERROR FS_r_ds2490status(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_r_pulldownslewrate(struct one_wire_query *owq)
 {
 	struct parsedname *pn = PN(owq);
-	if (pn->selected_connection->busmode != bus_usb) {
+	if (get_busmode(pn->selected_connection) != bus_usb) {
 		return -ENOTSUP;
 #if OW_USB
 	} else {
@@ -480,7 +462,7 @@ static ZERO_OR_ERROR FS_r_pulldownslewrate(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_w_pulldownslewrate(struct one_wire_query *owq)
 {
 	struct parsedname *pn = PN(owq);
-	if (pn->selected_connection->busmode != bus_usb) {
+	if (get_busmode(pn->selected_connection) != bus_usb) {
 		return -ENOTSUP;
 	}
 #if OW_USB
@@ -504,7 +486,7 @@ static ZERO_OR_ERROR FS_w_pulldownslewrate(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_r_writeonelowtime(struct one_wire_query *owq)
 {
 	struct parsedname *pn = PN(owq);
-	if (pn->selected_connection->busmode != bus_usb) {
+	if (get_busmode(pn->selected_connection) != bus_usb) {
 		OWQ_U(owq) = 10;
 #if OW_USB
 	} else {
@@ -517,7 +499,7 @@ static ZERO_OR_ERROR FS_r_writeonelowtime(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_w_writeonelowtime(struct one_wire_query *owq)
 {
 	struct parsedname *pn = PN(owq);
-	if (pn->selected_connection->busmode != bus_usb) {
+	if (get_busmode(pn->selected_connection) != bus_usb) {
 		return -ENOTSUP;
 	}
 
@@ -541,7 +523,7 @@ static ZERO_OR_ERROR FS_w_writeonelowtime(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_r_datasampleoffset(struct one_wire_query *owq)
 {
 	struct parsedname *pn = PN(owq);
-	if (pn->selected_connection->busmode != bus_usb) {
+	if (get_busmode(pn->selected_connection) != bus_usb) {
 		OWQ_U(owq) = 8;
 #if OW_USB
 	} else {
@@ -554,7 +536,7 @@ static ZERO_OR_ERROR FS_r_datasampleoffset(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_w_datasampleoffset(struct one_wire_query *owq)
 {
 	struct parsedname *pn = PN(owq);
-	if (pn->selected_connection->busmode != bus_usb){
+	if (get_busmode(pn->selected_connection) != bus_usb){
 		return -ENOTSUP;
 	}
 
@@ -586,7 +568,7 @@ static ZERO_OR_ERROR FS_name(struct one_wire_query *owq)
 static ZERO_OR_ERROR FS_port(struct one_wire_query *owq)
 {
 	return OWQ_format_output_offset_and_size_z(
-		SAFESTRING( SOC(PN(owq)->selected_connection)->devicename),
+		SAFESTRING( DEVICENAME(PN(owq)->selected_connection)),
 		owq);
 }
 
