@@ -1,9 +1,9 @@
 /*
-$Id: ow_lib_setup.c,v 1.11 2011/05/11 01:53:30 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -18,22 +18,20 @@ int main_threadid_init = 0 ;
 pthread_t main_threadid;
 
 /* All ow library setup */
-void LibSetup(enum opt_program opt)
+void LibSetup(enum enum_program_type program_type)
 {
+	Return_code_setup() ;
+	
 	/* Setup the multithreading synchronizing locks */
 	LockSetup();
 
-	Globals.opt = opt;
+	Globals.program_type = program_type;
 
-	/* special resort in case static data (devices and filetypes) not properly sorted */
-	DeviceSort();
-
-#if OW_CACHE
 	Cache_Open();
-#endif							/* OW_CACHE */
+	Detail_Init();
 
 	StateInfo.start_time = NOW_TIME;
-	SetLocalControlFlags() ; // reset by every option and othere change.
+	SetLocalControlFlags() ; // reset by every option and other change.
 	errno = 0;					/* set error level none */
 
 }
