@@ -1,9 +1,9 @@
 /*
-$Id: ow_lib_stop.c,v 1.8 2010/01/25 16:08:27 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -24,10 +24,9 @@ void LibStop(void)
 	Cache_Clear();
 	LEVEL_CALL("Closing input devices");
 	FreeInAll();
-	LEVEL_CALL("Closing outout devices");
+	LEVEL_CALL("Closing output devices");
 	FreeOutAll();
 
-
 	/* Have to reset more internal variables, and this should be fixed
 	 * by setting optind = 0 and call getopt()
 	 * (first_nonopt = last_nonopt = 1;)
