@@ -1,9 +1,9 @@
 /*
-$Id: ow_link.c,v 1.156 2011/12/11 02:29:49 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-	email: palfille@earthlink.net
+	email: paul.alfille@gmail.com
 	Released under the GPL
 	See the header file: ow.h for full attribution
 	1wire/iButton system from Dallas Semiconductor
@@ -35,6 +35,13 @@ would be in response to FF FA codes that you would send, so that seems
 unlikely. Handling these would be just the same as the FF FA codes above.
 
 */
+
+/* Multimaster:
+ * In general the link is NOT a multimaster design (only one bus master per serial or telnet port)
+ * The LinkHubE is supposed to have separately addressable lines, but this is not implemented
+ * None the less, all settings are assigned to the head line
+ */
+ 
 struct LINK_id {
 	char verstring[36];
 	char name[30];
@@ -66,10 +73,6 @@ static GOOD_OR_BAD LINK_PowerByte(const BYTE data, BYTE * resp, const UINT delay
 static GOOD_OR_BAD LINK_PowerBit(const BYTE data, BYTE * resp, const UINT delay, const struct parsedname *pn);
 static void LINK_close(struct connection_in *in) ;
 
-static GOOD_OR_BAD LinkHubE_Control( struct connection_in * in_link ) ;
-static GOOD_OR_BAD Control_com( struct connection_in * in_control ) ;
-
-
 static void LINK_setroutines(struct connection_in *in);
 static void LINKE_setroutines(struct connection_in *in);
 
@@ -104,6 +107,8 @@ static void LINK_setroutines(struct connection_in *in)
     in->iroutines.sendback_bits = LINK_sendback_bits;
 	in->iroutines.select = NO_SELECT_ROUTINE;
 	in->iroutines.select_and_sendback = NO_SELECTANDSENDBACK_ROUTINE;
+	in->iroutines.set_config = NO_SET_CONFIG_ROUTINE;
+	in->iroutines.get_config = NO_GET_CONFIG_ROUTINE;
 	in->iroutines.reconnect = NO_RECONNECT_ROUTINE;
 	in->iroutines.close = LINK_close;
 	in->iroutines.flags = ADAP_FLAG_no2409path | ADAP_FLAG_no2404delay ;
@@ -163,53 +168,50 @@ static GOOD_OR_BAD LinkVersion_unknownstring( const char * reported_string, stru
 }
 
 // bus locking done at a higher level
-GOOD_OR_BAD LINK_detect(struct connection_in *in)
+GOOD_OR_BAD LINK_detect(struct port_in *pin)
 {
-	if (SOC(in)->devicename == NULL) {
+	struct connection_in * in = pin->first ;
+
+	if (pin->init_data == NULL) {
+		// requires input string
+		LEVEL_DEFAULT("LINK busmaster requires port name");
 		return gbBAD;
 	}
 
 	COM_set_standard( in ) ; // standard COM port settings
 
-	switch( SOC(in)->type ) {
+	switch( pin->type ) {
 		case ct_telnet:
 			// LinkHub-E
-			SOC(in)->baud = B115200 ;
+			pin->baud = B115200 ;
 			LEVEL_DEBUG("Attempt connection to networked LINK at 115200 baud");
 			RETURN_GOOD_IF_GOOD(  LINK_detect_net( in )  );
 
 			// Xport or ser2net
-			SOC(in)->baud = B9600 ;
+			pin->baud = B9600 ;
 			LEVEL_DEBUG("Attempt connection to networked LINK at 9600 baud");
 			RETURN_GOOD_IF_GOOD(  LINK_detect_net( in )  );
 
-			// Now try control port reset
-			RETURN_BAD_IF_BAD( LinkHubE_Control( in ) ) ;
-			SOC(in)->baud = B115200 ;
-			LEVEL_DEBUG("Reattempt LINK connection") ;
-			UT_delay( 5000 ) ;
-			RETURN_GOOD_IF_GOOD(  LINK_detect_net( in )  );
-			
 			break ;
 
 			
 
 		case ct_serial:
-			SOC(in)->baud = B9600 ;
+			pin->baud = B9600 ;
 
-			SOC(in)->flow = flow_first ;
+			pin->flow = flow_first ;
 			RETURN_GOOD_IF_GOOD( LINK_detect_serial(in) ) ;
 
 			LEVEL_DEBUG("Second attempt at serial LINK setup");
-			SOC(in)->flow = flow_second ;
+			pin->flow = flow_second ;
 			RETURN_GOOD_IF_GOOD( LINK_detect_serial(in) ) ;
 
 			LEVEL_DEBUG("Third attempt at serial LINK setup");
-			SOC(in)->flow = flow_first ;
+			pin->flow = flow_first ;
 			RETURN_GOOD_IF_GOOD( LINK_detect_serial(in) ) ;
 
 			LEVEL_DEBUG("Fourth attempt at serial LINK setup");
-			SOC(in)->flow = flow_second ;
+			pin->flow = flow_second ;
 			RETURN_GOOD_IF_GOOD( LINK_detect_serial(in) ) ;
 			break ;
 
@@ -221,10 +223,11 @@ GOOD_OR_BAD LINK_detect(struct connection_in *in)
 
 static GOOD_OR_BAD LINK_detect_serial(struct connection_in * in)
 {
+	struct port_in * pin = in->pown ;
 	/* Set up low-level routines */
 	LINK_setroutines(in);
-	SOC(in)->timeout.tv_sec = Globals.timeout_serial ;
-	SOC(in)->timeout.tv_usec = 0 ;
+	pin->timeout.tv_sec = Globals.timeout_serial ;
+	pin->timeout.tv_usec = 0 ;
 
 	/* Open the com port */
 	RETURN_BAD_IF_BAD(COM_open(in)) ;
@@ -235,6 +238,15 @@ static GOOD_OR_BAD LINK_detect_serial(struct connection_in * in)
 	UT_delay(100) ; // based on http://morpheus.wcf.net/phpbb2/viewtopic.php?t=89&sid=3ab680415917a0ebb1ef020bdc6903ad
 	LINK_slurp( in ) ;
 	
+	RETURN_GOOD_IF_GOOD( LINK_version(in) ) ;
+	LEVEL_DEFAULT("LINK detection error");
+	
+	serial_powercycle(in) ;
+	LEVEL_DEBUG("Slurp in initial bytes");
+	LINK_slurp( in ) ;
+	UT_delay(100) ; // based on http://morpheus.wcf.net/phpbb2/viewtopic.php?t=89&sid=3ab680415917a0ebb1ef020bdc6903ad
+	LINK_slurp( in ) ;
+	
 	RETURN_GOOD_IF_GOOD( LINK_version(in) ) ;
 	LEVEL_DEFAULT("LINK detection error");
 	COM_close(in) ;
@@ -243,10 +255,11 @@ static GOOD_OR_BAD LINK_detect_serial(struct connection_in * in)
 
 static GOOD_OR_BAD LINK_detect_net(struct connection_in * in)
 {
+	struct port_in * pin = in->pown ;
 	/* Set up low-level routines */
 	LINKE_setroutines(in);
-	SOC(in)->timeout.tv_sec = 0 ;
-	SOC(in)->timeout.tv_usec = 300000 ;
+	pin->timeout.tv_sec = 0 ;
+	pin->timeout.tv_usec = 300000 ;
 
 	/* Open the tcp port */
 	RETURN_BAD_IF_BAD( COM_open(in) ) ;
@@ -257,7 +270,7 @@ static GOOD_OR_BAD LINK_detect_net(struct connection_in * in)
 	LINK_slurp( in ) ;
 //	LINK_flush(in);
 
-	SOC(in)->dev.telnet.telnet_negotiated = needs_negotiation ;
+	pin->dev.telnet.telnet_negotiated = needs_negotiation ;
 	RETURN_GOOD_IF_GOOD( LINK_version(in) ) ;
 
 	// second try -- send a break and line settings
@@ -336,7 +349,7 @@ static GOOD_OR_BAD LINK_version(struct connection_in * in)
 						// add to string
 						break ; 
 					case lvs_0d:
-						++in->master.serial.tcp.CRLF_size;
+						++in->CRLF_size;
 						break ;
 				}
 				break ;
@@ -348,18 +361,19 @@ static GOOD_OR_BAD LINK_version(struct connection_in * in)
 
 static void LINK_set_baud(struct connection_in * in)
 {
+	struct port_in * pin = in->pown ;
 	char * speed_code ;
 
-	if ( SOC(in)->type == ct_telnet ) {
+	if ( pin->type == ct_telnet ) {
 		// telnet pinned at 115200
 		return ;
 	}
 
-	COM_BaudRestrict( &(SOC(in)->baud), B9600, B19200, B38400, B57600, 0 ) ;
+	COM_BaudRestrict( &(pin->baud), B9600, B19200, B38400, B57600, 0 ) ;
 
-	LEVEL_DEBUG("to %d",COM_BaudRate(SOC(in)->baud));
+	LEVEL_DEBUG("to %d",COM_BaudRate(pin->baud));
 	// Find rate parameter
-	switch ( SOC(in)->baud ) {
+	switch ( pin->baud ) {
 		case B9600:
 			COM_break(in) ;
 			LINK_flush(in);
@@ -385,7 +399,7 @@ static void LINK_set_baud(struct connection_in * in)
 	LINK_flush(in);
 	if ( BAD( LINK_write(LINK_string(speed_code), 1, in) ) ) {
 		LEVEL_DEBUG("LINK change baud error -- will return to 9600");
-		SOC(in)->baud = B9600 ;
+		pin->baud = B9600 ;
 		++in->changed_bus_settings ;
 		return ;
 	}
@@ -415,7 +429,7 @@ static RESET_TYPE LINK_reset(const struct parsedname *pn)
 
 static RESET_TYPE LINK_reset_in(struct connection_in * in)
 {
-	BYTE resp[1+in->master.serial.tcp.CRLF_size];
+	BYTE resp[1+in->CRLF_size];
 
 	if (in->changed_bus_settings > 0) {
 		--in->changed_bus_settings ;
@@ -489,7 +503,7 @@ static void LINK_slurp(struct connection_in *in)
 
 static GOOD_OR_BAD LINK_read(BYTE * buf, size_t size, struct connection_in *in)
 {
-	return LINK_read_true_length( buf, size+in->master.serial.tcp.CRLF_size, in ) ;
+	return LINK_read_true_length( buf, size+in->CRLF_size, in ) ;
 }
 
 static GOOD_OR_BAD LINK_read_true_length(BYTE * buf, size_t size, struct connection_in *in)
@@ -508,7 +522,7 @@ static GOOD_OR_BAD LINK_write(const BYTE * buf, size_t size, struct connection_i
 
 static GOOD_OR_BAD LINK_search_type(struct device_search *ds, struct connection_in * in)
 {
-	char resp[3+in->master.serial.tcp.CRLF_size];
+	char resp[3+in->CRLF_size];
 	int response_length ;
 
 	LEVEL_DEBUG("Test to see if LINK supports the tF0 command");
@@ -581,7 +595,7 @@ static GOOD_OR_BAD LINK_search_type(struct device_search *ds, struct connection_
 
 static GOOD_OR_BAD LINK_directory(struct device_search *ds, struct connection_in * in)
 {
-	char resp[DEVICE_LENGTH+COMMA_LENGTH+PLUS_LENGTH+in->master.serial.tcp.CRLF_size];
+	char resp[DEVICE_LENGTH+COMMA_LENGTH+PLUS_LENGTH+in->CRLF_size];
 
 	DirblobClear( &(ds->gulp) );
 
@@ -616,7 +630,7 @@ static GOOD_OR_BAD LINK_directory(struct device_search *ds, struct connection_in
 			break ;
 	}
 	
-	if ( BAD(LINK_read(LINK_string(&resp[1+in->master.serial.tcp.CRLF_size]), DEVICE_LENGTH+COMMA_LENGTH+PLUS_LENGTH-1-in->master.serial.tcp.CRLF_size, in)) ) {
+	if ( BAD(LINK_read(LINK_string(&resp[1+in->CRLF_size]), DEVICE_LENGTH+COMMA_LENGTH+PLUS_LENGTH-1-in->CRLF_size, in)) ) {
 		return gbBAD;
 	}
 
@@ -685,7 +699,7 @@ static GOOD_OR_BAD LINK_PowerByte(const BYTE data, BYTE * resp, const UINT delay
 {
 	struct connection_in * in = pn->selected_connection ;
 	ASCII buf[3] = "pxx";
-	BYTE respond[2+in->master.serial.tcp.CRLF_size] ;
+	BYTE respond[2+in->CRLF_size] ;
 	
 	num2string(&buf[1], data);
 	
@@ -709,7 +723,7 @@ static GOOD_OR_BAD LINK_PowerByte(const BYTE data, BYTE * resp, const UINT delay
 static GOOD_OR_BAD LINK_PowerBit(const BYTE data, BYTE * resp, const UINT delay, const struct parsedname *pn)
 {
 	struct connection_in * in = pn->selected_connection ;
-	BYTE buf[2+1+1+in->master.serial.tcp.CRLF_size] ;
+	BYTE buf[2+1+1+in->CRLF_size] ;
 	
 	buf[0] = '~';			//put in power bit mode
 	buf[1] = data ? '1' : '0' ;
@@ -741,7 +755,7 @@ static GOOD_OR_BAD LINK_sendback_data(const BYTE * data, BYTE * resp, const size
 	struct connection_in * in = pn->selected_connection ;
 	size_t left = size;
 	size_t location = 0 ;
-	BYTE buf[1+LINK_SEND_SIZE*2+1+1+in->master.serial.tcp.CRLF_size] ;
+	BYTE buf[1+LINK_SEND_SIZE*2+1+1+in->CRLF_size] ;
 	
 	if (size == 0) {
 		return gbGOOD;
@@ -780,7 +794,7 @@ static GOOD_OR_BAD LINK_sendback_bits(const BYTE * databits, BYTE * respbits, co
 	struct connection_in * in = pn->selected_connection ;
 	size_t left = size;
 	size_t location = 0 ;
-	BYTE buf[1+LINK_SEND_SIZE+1+1+in->master.serial.tcp.CRLF_size] ;
+	BYTE buf[1+LINK_SEND_SIZE+1+1+in->CRLF_size] ;
 	
 	if (size == 0) {
 		return gbGOOD;
@@ -845,41 +859,3 @@ static GOOD_OR_BAD LINK_readback_data( BYTE * buf, const size_t size, struct con
 		
 	return gbGOOD;
 }
-
-// use the Xport control port to reset
-static GOOD_OR_BAD LinkHubE_Control( struct connection_in * in_link )
-{
-	struct connection_in * in_control = AllocIn( NO_CONNECTION ) ;
-
-	if ( SOC(in_link)->dev.tcp.host == NULL ) {
-		LEVEL_DEBUG("No Xport control on local machine");
-		return gbBAD ;
-	}
-
-	if ( in_control != NO_CONNECTION ) {
-		GOOD_OR_BAD ret ;
-
-		SOC(in_control)->type = ct_telnet ;
-		SOC(in_control)->devicename = owstrdup( SOC(in_link)->dev.tcp.host ) ;
-		in_control->busmode = bus_xport_control ;
-		SOC(in_control)->timeout.tv_sec = 1 ;
-		SOC(in_control)->timeout.tv_usec = 0 ;
-
-		ret = Control_com( in_control ) ;
-
-		RemoveIn( in_control ) ;
-		return ret ;
-	}
-	return gbBAD ;
-}
-
-static GOOD_OR_BAD Control_com( struct connection_in * in_control )
-{
-	RETURN_BAD_IF_BAD( COM_open( in_control ) ) ;
-	COM_slurp( in_control ) ;
-	RETURN_BAD_IF_BAD( COM_write_simple( (BYTE *) "\r", 1, in_control ) ) ;
-	COM_slurp( in_control ) ;
-	RETURN_BAD_IF_BAD( COM_write_simple( (BYTE *) "9\r", 2, in_control ) ) ;
-	COM_slurp( in_control ) ;
-	return gbGOOD ;
-}
