@@ -1,9 +1,8 @@
 /*
-$Id: ow_locks.c,v 1.94 2012/04/14 03:52:46 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-	email: palfille@earthlink.net
+	email: paul.alfille@gmail.com
 	Released under the GPL
 	See the header file: ow.h for full attribution
 	1wire/iButton system from Dallas Semiconductor
@@ -19,7 +18,6 @@ $Id: ow_locks.c,v 1.94 2012/04/14 03:52:46 alfille Exp $
 
 /* ------- Globals ----------- */
 
-#if OW_MT
   #ifdef __UCLIBC__
     #if ((__UCLIBC_MAJOR__ << 16)+(__UCLIBC_MINOR__ << 8)+(__UCLIBC_SUBLEVEL__) < 0x00091D)
 	/* If uClibc < 0.9.29, then re-initialize internal pthread-structs */
@@ -27,7 +25,6 @@ $Id: ow_locks.c,v 1.94 2012/04/14 03:52:46 alfille Exp $
 	void __pthread_initialize(void);
     #endif							/* UCLIBC_VERSION */
  #endif							/* __UCLIBC__ */
-#endif							/* OW_MT */
 
 // Global structure holding mutexes
 struct mutexes Mutex ;
@@ -35,11 +32,11 @@ struct mutexes Mutex ;
 /* Essentially sets up mutexes to protect global data/devices */
 void LockSetup(void)
 {
-#if OW_MT
 	/* global mutex attribute */
 	_MUTEX_ATTR_INIT(Mutex.mattr);
-  #ifdef __UCLIBC__
-    #if ((__UCLIBC_MAJOR__ << 16)+(__UCLIBC_MINOR__ << 8)+(__UCLIBC_SUBLEVEL__) < 0x00091D)
+
+#ifdef __UCLIBC__
+#if ((__UCLIBC_MAJOR__ << 16)+(__UCLIBC_MINOR__ << 8)+(__UCLIBC_SUBLEVEL__) < 0x00091D)
 	/* If uClibc < 0.9.29, then re-initialize internal pthread-structs
 	 * pthread and mutexes doesn't work after daemon() is called and
 	 *   the main-process is gone.
@@ -50,32 +47,42 @@ void LockSetup(void)
 	__pthread_initial_thread_bos = NULL;
 	__pthread_initialize();
 
+	/* global mutex attribute */
 	_MUTEX_ATTR_SET(Mutex.mattr, PTHREAD_MUTEX_ADAPTIVE_NP);
-    #else /* UCLIBC_VERSION */
+#else /* UCLIBC_VERSION */
 	_MUTEX_ATTR_SET(Mutex.mattr, PTHREAD_MUTEX_DEFAULT);
-    #endif							/* UCLIBC_VERSION */
+#endif							/* UCLIBC_VERSION */
+#define MUTEX_ATTR_INIT_DONE
 	_MUTEX_INIT(Mutex.uclibc_mutex);
-  #else /* __UCLIBC__ */
-	_MUTEX_ATTR_SET(Mutex.mattr, PTHREAD_MUTEX_DEFAULT);   
-  #endif							/* __UCLIBC__ */
+#endif							/* __UCLIBC__ */
 
+#ifndef MUTEX_ATTR_INIT_DONE
+#define MUTEX_ATTR_INIT_DONE
+#ifdef EXTENDED_MUTEX_DEBUG
+	_MUTEX_ATTR_SET(Mutex.mattr, PTHREAD_MUTEX_ERRORCHECK);
+#else
+	_MUTEX_ATTR_SET(Mutex.mattr, PTHREAD_MUTEX_DEFAULT);
+#endif
+#endif
+	
 	_MUTEX_INIT(Mutex.stat_mutex);
 	_MUTEX_INIT(Mutex.controlflags_mutex);
 	_MUTEX_INIT(Mutex.fstat_mutex);
 	_MUTEX_INIT(Mutex.dir_mutex);
+#if OW_USB
+	_MUTEX_INIT(Mutex.libusb_mutex);
+#endif							/* OW_USB */
 	_MUTEX_INIT(Mutex.typedir_mutex);
 	_MUTEX_INIT(Mutex.externaldir_mutex);
 	_MUTEX_INIT(Mutex.namefind_mutex);
-	_MUTEX_INIT(Mutex.aliasfind_mutex);
+	_MUTEX_INIT(Mutex.aliaslist_mutex);
 	_MUTEX_INIT(Mutex.externalcount_mutex);
+	_MUTEX_INIT(Mutex.timegm_mutex);
+	_MUTEX_INIT(Mutex.detail_mutex);
 
 	RWLOCK_INIT(Mutex.lib);
 	RWLOCK_INIT(Mutex.cache);
 	RWLOCK_INIT(Mutex.persistent_cache);
-	RWLOCK_INIT(Inbound_Control.lock);
-	RWLOCK_INIT(Inbound_Control.monitor_lock);
-  #if OW_USB
-	_MUTEX_INIT(Mutex.libusb_mutex);
-  #endif							/* OW_USB */
-#endif							/* OW_MT */
+	RWLOCK_INIT(Mutex.connin);
+	RWLOCK_INIT(Mutex.monitor);
 }
