@@ -1,15 +1,15 @@
 /*
-$Id: ow_multicast.c,v 1.27 2012/04/01 18:18:05 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
 */
 
-/* ow_net holds the network utility routines. Many stolen unashamedly from Steven's Book */
+/* ow_net holds the network utility routines. Mainly stolen unashamedly from Steven's Book */
 /* Much modification by Christian Magnusson especially for Valgrind and embedded */
 /* non-threaded fixes by Jerry Scharf */
 
@@ -22,7 +22,6 @@ $Id: ow_multicast.c,v 1.27 2012/04/01 18:18:05 alfille Exp $
 #include <arpa/inet.h>
 #endif
 
-#if OW_HA7
 /* Multicast to discover HA7 servers */
 /* Wait 10 seconds for responses */
 /* returns number found (>=0) or <0 on error */
@@ -90,7 +89,7 @@ static int Get_HA7_response( struct addrinfo *now, char * name )
 		ERROR_DEBUG("Cannot get socket file descriptor for broadcast.");
 		return 1;
 	}
-	fcntl (file_descriptor, F_SETFD, FD_CLOEXEC); // for safe forking
+//	fcntl (file_descriptor, F_SETFD, FD_CLOEXEC); // for safe forking
 	if (setsockopt(file_descriptor, SOL_SOCKET, SO_BROADCAST, &on, sizeof(on)) == -1) {
 		ERROR_DEBUG("Cannot set socket option for broadcast.");
 		return 1;
@@ -126,34 +125,36 @@ GOOD_OR_BAD FS_FindHA7(void)
 	int number_found = 0;
 	int getaddr_error ;
 
-	LEVEL_DEBUG("Attempting udp multicast search for the HA7Net bus master at 224.1.2.3:4567");
+	LEVEL_DEBUG("Attempting udp multicast search for the HA7Net bus master at %s:%s",HA7_DISCOVERY_ADDRESS,HA7_DISCOVERY_PORT);
 	Setup_HA7_hint( &hint ) ;
-	if ((getaddr_error = getaddrinfo("224.1.2.3", "4567", &hint, &ai))) {
+	if ((getaddr_error = getaddrinfo(HA7_DISCOVERY_ADDRESS, HA7_DISCOVERY_PORT, &hint, &ai))) {
 		LEVEL_CONNECT("Couldn't set up HA7 broadcast message %s", gai_strerror(getaddr_error));
 		return gbBAD;
 	}
 
 	for (now = ai; now; now = now->ai_next) {
 		ASCII name[INET_ADDRSTRLEN+20]; // tcp quad + port
+		struct port_in * pin ;
 		struct connection_in *in;
 
 		if ( Get_HA7_response( now, name ) ) {
 			continue ;
 		}
 
-		in = NewIn(NO_CONNECTION) ;
-		if (in == NO_CONNECTION) {
+		pin = NewPort(NO_CONNECTION) ;
+		if (pin == NULL) {
 			continue;
 		}
+		in = pin->first ;
 
-		SOC(in)->type = ct_tcp ;
-		SOC(in)->devicename = owstrdup(name);
-		in->busmode = bus_ha7net;
+		pin->type = ct_tcp ;
+		pin->init_data = owstrdup(name);
+		DEVICENAME(in) = owstrdup(name);
+		pin->busmode = bus_ha7net;
 
-		LEVEL_CONNECT("HA7Net bus master discovered at %s",SOC(in)->devicename);
+		LEVEL_CONNECT("HA7Net bus master discovered at %s",DEVICENAME(in));
 		++number_found ;
 	}
 	freeaddrinfo(ai);
 	return number_found > 0 ? gbGOOD : gbBAD ;
 }
-#endif							/* OW_HA7 */
