@@ -1,5 +1,5 @@
 /*
-$Id: ow_net_server.c,v 1.60 2012/03/27 17:35:23 alfille Exp $
+$Id: ow_net_server.c,v 1.62 2013/01/01 16:03:36 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -109,7 +109,7 @@ static FILE_DESCRIPTOR_OR_ERROR ServerListen(struct connection_out *out)
 		} else if (listen(file_descriptor, 10) != 0) {
 			ERROR_CONNECT("Listen problem [%s]", SAFESTRING(out->name));
 		} else {
-			fcntl (file_descriptor, F_SETFD, FD_CLOEXEC); // for safe forking
+//			fcntl (file_descriptor, F_SETFD, FD_CLOEXEC); // for safe forking
 			out->file_descriptor = file_descriptor;
 			return file_descriptor;
 		}
@@ -223,8 +223,6 @@ static GOOD_OR_BAD ListenCycle( void )
 	return gbBAD ;
 }
 
-#if OW_MT
-
 // Read data from the waiting socket and do the actual work
 static void *ProcessAcceptSocket(void *arg)
 {
@@ -308,8 +306,8 @@ void ServerProcess(void (*HandlerRoutine) (FILE_DESCRIPTOR_OR_ERROR file_descrip
 		ERROR_DEFAULT("Cannot allocate a shutdown pipe. The program shutdown may be messy");
 		Init_Pipe( shutdown_pipe ) ;
 	}
-	fcntl (shutdown_pipe[fd_pipe_read], F_SETFD, FD_CLOEXEC); // for safe forking
-	fcntl (shutdown_pipe[fd_pipe_write], F_SETFD, FD_CLOEXEC); // for safe forking
+//	fcntl (shutdown_pipe[fd_pipe_read], F_SETFD, FD_CLOEXEC); // for safe forking
+//	fcntl (shutdown_pipe[fd_pipe_write], F_SETFD, FD_CLOEXEC); // for safe forking
 		
 	if ( GOOD( SetupListenSockets( HandlerRoutine ) ) ) {
 		while (	GOOD( ListenCycle() ) ) {
@@ -339,61 +337,3 @@ void ServerProcess(void (*HandlerRoutine) (FILE_DESCRIPTOR_OR_ERROR file_descrip
 	/* Cleanup that may never be reached */
 	return;
 }
-
-#else /* OW_MT */
-
-// Read data from the waiting socket and do the actual work
-static void *ProcessAcceptSocket(void *arg)
-{
-	struct Accept_Socket_Data * asd = (struct Accept_Socket_Data *) arg;
-
-	// Do the actual work
-	asd->out->HandlerRoutine( asd->acceptfd );
-
-	// cleanup
-	Test_and_Close( &(asd->acceptfd) );
-	owfree(asd);
-	LEVEL_DEBUG("Normal exit.");
-
-	return VOID_RETURN;
-}
-
-static void ProcessListenSocket( struct connection_out * out )
-{
-	FILE_DESCRIPTOR_OR_ERROR acceptfd;
-	struct Accept_Socket_Data * asd ;
-
-	acceptfd = accept(out->file_descriptor, NULL, NULL);
-
-	if ( FILE_DESCRIPTOR_NOT_VALID( acceptfd ) ) {
-		return ;
-	}
-
-	// allocate space to pass variables to thread -- cleaned up in thread handler
-	asd = owmalloc( sizeof(struct Accept_Socket_Data) ) ;
-	if ( asd == NULL ) {
-		close( acceptfd ) ;
-		return ;
-	}
-	asd->acceptfd = acceptfd ;
-	asd->out = out ;
-
-	ProcessAcceptSocket(asd) ;
-}
-
-/* Setup Servers -- select on each port */
-void ServerProcess(void (*HandlerRoutine) (FILE_DESCRIPTOR_OR_ERROR file_descriptor))
-{
-	if ( GOOD( SetupListenSockets( HandlerRoutine ) ) ) {
-		while (	GOOD( ListenCycle() ) ) {
-		}
-		CloseListenSockets() ;
-	} else {
-		LEVEL_DEFAULT("Isolated from any control -- exit") ;
-	}
-	/* Cleanup that may never be reached */
-	return;
-}
-
-#endif /* OW_MT */
-
