@@ -1,9 +1,8 @@
 /*
-$Id: ow_opt.c,v 1.166 2012/05/06 21:21:55 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -11,9 +10,13 @@ $Id: ow_opt.c,v 1.166 2012/05/06 21:21:55 alfille Exp $
 
 /* ow_opt -- owlib specific command line options processing */
 
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #include <stdio.h> // for getline
 #undef _GNU_SOURCE
+#else
+#include <stdio.h> // for getline
+#endif
 
 #include <config.h>
 #include "owfs_config.h"
@@ -99,10 +102,12 @@ const struct option owopts_long[] = {
 	{"external", no_argument, &Globals.allow_external, 1},
 	{"no_external", no_argument, &Globals.allow_external, 0},
 
-	{"background", no_argument, &Globals.want_background, 1},
-	{"foreground", no_argument, &Globals.want_background, 0},
+	{"background", no_argument, NO_LINKED_VAR, e_want_background},
+	{"foreground", no_argument, NO_LINKED_VAR, e_want_foreground},
 	{"fatal_debug", no_argument, &Globals.fatal_debug, 1},
 	{"fatal-debug", no_argument, &Globals.fatal_debug, 1},
+	{"nofatal_debug", no_argument, &Globals.fatal_debug, 0},
+	{"nofatal-debug", no_argument, &Globals.fatal_debug, 0},
 	{"fatal_debug_file", required_argument, NO_LINKED_VAR, e_fatal_debug_file},
 	{"fatal-debug-file", required_argument, NO_LINKED_VAR, e_fatal_debug_file},
 	{"error_print", required_argument, NO_LINKED_VAR, e_error_print},
@@ -147,6 +152,8 @@ const struct option owopts_long[] = {
 	{"fake", required_argument, NO_LINKED_VAR, e_fake},	/* Fake */
 	{"link", required_argument, NO_LINKED_VAR, e_link},	/* link in ascii mode */
 	{"LINK", required_argument, NO_LINKED_VAR, e_link},	/* link in ascii mode */
+	{"PBM", required_argument, NO_LINKED_VAR, e_pbm}, /* Dirk Opfer's elabnet "power bus master" */
+	{"pbm", required_argument, NO_LINKED_VAR, e_pbm}, /* Dirk Opfer's elabnet "power bus master" */
 	{"HA3", required_argument, NO_LINKED_VAR, e_ha3},
 	{"ha3", required_argument, NO_LINKED_VAR, e_ha3},
 	{"HA4B", required_argument, NO_LINKED_VAR, e_ha4b},
@@ -183,6 +190,11 @@ const struct option owopts_long[] = {
 	{"browse", no_argument, NO_LINKED_VAR, e_browse},
 	{"w1", no_argument, NO_LINKED_VAR, e_w1_monitor},
 	{"W1", no_argument, NO_LINKED_VAR, e_w1_monitor},
+	{"masterhub", required_argument, NO_LINKED_VAR, e_masterhub},
+	{"Masterhub", required_argument, NO_LINKED_VAR, e_masterhub},
+	{"MasterHub", required_argument, NO_LINKED_VAR, e_masterhub},
+	{"MASTERHUB", required_argument, NO_LINKED_VAR, e_masterhub},
+
 	{"announce", required_argument, NO_LINKED_VAR, e_announce},
 	{"allow_other", no_argument, &Globals.allow_other, 1},
 	{"altUSB", no_argument, &Globals.altUSB, 1},	/* Willy Robison's tweaks */
@@ -271,6 +283,10 @@ const struct option owopts_long[] = {
 	{"no_presencepulsemasking", no_argument, &Globals.i2c_PPM, 0},
 	{"no_PPM", no_argument, &Globals.i2c_PPM, 0},
 	{"no_ppm", no_argument, &Globals.i2c_PPM, 0},
+	{"trim", no_argument, &Globals.trim, 1},
+	{"TRIM", no_argument, &Globals.trim, 1},
+	{"detail", required_argument, NO_LINKED_VAR, e_detail},	/* slave detail */
+	{"details", required_argument, NO_LINKED_VAR, e_detail},	/* slave detail */
 	{0, 0, 0, 0},
 };
 
@@ -366,102 +382,102 @@ static void ParseTheLine(struct lineparse *lp)
 		// change states on special chars
 		switch (*current_char) {
 			// end of line characters
-		case '#':
-		case '\n':
-		case '\r':
-			*current_char = '\0';
-			return;
-			// whitespace characters
-		case ' ':
-		case '\t':
-			switch (parse_state) {
-			case ps_in_opt:
-				*current_char = '\0';
-				parse_state = ps_pre_equals;
-				break;
-			case ps_in_value:
+			case '#':
+			case '\n':
+			case '\r':
 				*current_char = '\0';
 				return;
-			default:
+				// whitespace characters
+			case ' ':
+			case '\t':
+				switch (parse_state) {
+				case ps_in_opt:
+					*current_char = '\0';
+					parse_state = ps_pre_equals;
+					break;
+				case ps_in_value:
+					*current_char = '\0';
+					return;
+				default:
+					break;
+				}
 				break;
-			}
-			break;
-			// dashes before options ignored (only needed in real command line)
-		case '-':
-			switch (parse_state) {
-			case ps_pre_value:	// not ignored before values, of course
-				lp->val = current_char;
-				parse_state = ps_in_value;
+				// dashes before options ignored (only needed in real command line)
+			case '-':
+				switch (parse_state) {
+				case ps_pre_value:	// not ignored before values, of course
+					lp->val = current_char;
+					parse_state = ps_in_value;
+					break;
+				default:
+					break;
+				}
 				break;
-			default:
+				// negate program state (even though program not yet assigned)
+			case '!':
+				switch (parse_state) {
+					case ps_pre_opt:
+						lp->reverse_prog = !lp->reverse_prog;
+						break;
+					case ps_pre_value:	// not ignored before values, of course
+						lp->val = current_char;
+						parse_state = ps_in_value;
+						break;
+					default:
+						break;
+				}
 				break;
-			}
-			break;
-			// negate program state (even though program not yet assigned)
-		case '!':
-			switch (parse_state) {
-			case ps_pre_opt:
-				lp->reverse_prog = !lp->reverse_prog;
-				break;
-			case ps_pre_value:	// not ignored before values, of course
-				lp->val = current_char;
-				parse_state = ps_in_value;
-				break;
-			default:
-				break;
-			}
-			break;
-			// colon special case -- assigns to a program
-			// Since we had assumed this was an opt, we have to adjust
-		case ':':
-			switch (parse_state) {
-			case ps_in_opt:
-			case ps_pre_equals:
-				*current_char = '\0';
-				lp->prog = lp->opt;
-				lp->opt = NULL;
-				{	// put in lower case
-					ASCII *prog_char;
-					for (prog_char = lp->prog; *prog_char; ++prog_char) {
-						*prog_char = tolower(*prog_char);
+				// colon special case -- assigns to a program
+				// Since we had assumed this was an opt, we have to adjust
+			case ':':
+				switch (parse_state) {
+					case ps_in_opt:
+					case ps_pre_equals:
+						*current_char = '\0';
+						lp->prog = lp->opt;
+						lp->opt = NULL;
+						{	// put in lower case
+							ASCII *prog_char;
+							for (prog_char = lp->prog; *prog_char; ++prog_char) {
+								*prog_char = tolower(*prog_char);
+							}
+						}
+						// special cases for sensor and property lines
+						// they use a different syntax
+						if (strstr(lp->prog, "sensor") != NULL) {
+							// sensor line for external device
+							LEVEL_DEBUG("SENSOR entry found <%s>", current_char+1);
+							lp->prog = NULL ;
+							AddSensor(current_char+1) ;
+							return ;
+						} else if (strstr(lp->prog, "script") != NULL) {
+							// property line for external device
+							LEVEL_DEBUG("SCRIPT entry found <%s>", current_char+1);
+							lp->prog = NULL ;
+							AddProperty(current_char+1,et_script) ;
+							return ;
+						} else if (strstr(lp->prog, "property") != NULL) {
+							// property line for external device
+							LEVEL_DEBUG("PROPERTY (SCRIPT) entry found <%s>", current_char+1);
+							lp->prog = NULL ;
+							AddProperty(current_char+1,et_script) ;
+							return ;
 					}
-				}
-				// special cases for sensor and property lines
-				// they use a different syntax
-				if (strstr(lp->prog, "sensor") != NULL) {
-					// sensor line for external device
-					LEVEL_DEBUG("SENSOR entry found <%s>", current_char+1);
-					lp->prog = NULL ;
-					AddSensor(current_char+1) ;
-					return ;
-				} else if (strstr(lp->prog, "script") != NULL) {
-					// property line for external device
-					LEVEL_DEBUG("SCRIPT entry found <%s>", current_char+1);
-					lp->prog = NULL ;
-					AddProperty(current_char+1,et_script) ;
-					return ;
-				} else if (strstr(lp->prog, "property") != NULL) {
-					// property line for external device
-					LEVEL_DEBUG("PROPERTY (SCRIPT) entry found <%s>", current_char+1);
-					lp->prog = NULL ;
-					AddProperty(current_char+1,et_script) ;
-					return ;
-				}
-				parse_state = ps_pre_opt;
-				break;
-			case ps_pre_value:	// not ignored before values, of course
-				lp->val = current_char;
-				parse_state = ps_in_value;
-				break;
-			default:
-				break;
+					parse_state = ps_pre_opt;
+					break;
+				case ps_pre_value:	// not ignored before values, of course
+					lp->val = current_char;
+					parse_state = ps_in_value;
+					break;
+				default:
+					break;
 			}
 			break;
 			// equals special case: jump state
 		case '=':
 			switch (parse_state) {
 			case ps_in_opt:
-				parse_state = ps_pre_equals;
+				// parse_state = ps_pre_equals;
 				// fall through intentionally
 			case ps_pre_equals:
 				*current_char = '\0';
@@ -482,8 +498,6 @@ static void ParseTheLine(struct lineparse *lp)
 				parse_state = ps_in_opt;
 				break;
 			case ps_pre_equals:
-				parse_state = ps_pre_value;
-				// fall through intentionally
 			case ps_pre_value:
 				lp->val = current_char;
 				parse_state = ps_in_value;
@@ -586,6 +600,7 @@ GOOD_OR_BAD owopt_packed(const char *params)
 
 /* Parses one argument */
 /* return 0 if ok */
+/* Note: owopt is called in single thread mode -- rest of owfs machinery hasn't been launched */
 GOOD_OR_BAD owopt(const int option_char, const char *arg)
 {
 	// depth of nesting for config files
@@ -649,21 +664,25 @@ GOOD_OR_BAD owopt(const int option_char, const char *arg)
 	case 's':
 		return ARG_Net(arg);
 	case 'p':
+		if ( Globals.daemon_status == e_daemon_sd ) {
+			LEVEL_DEFAULT("systemd mode -- ignore 'p' option");
+			return gbGOOD ;
+		}
 		switch (Globals.program_type) {
-		case program_type_httpd:
-		case program_type_server:
-		case program_type_external:
-		case program_type_ftpd:
-			return ARG_Server(arg);
-		default:
-			return gbGOOD;
+			case program_type_httpd:
+			case program_type_server:
+			case program_type_external:
+			case program_type_ftpd:
+				return ARG_Server(arg);
+			default:
+				return gbGOOD;
 		}
 	case 'm':
 		switch (Globals.program_type) {
-		case program_type_filesystem:
-			return ARG_Server(arg);
-		default:
-			return gbGOOD;
+			case program_type_filesystem:
+				return ARG_Server(arg);
+			default:
+				return gbGOOD;
 		}
 	case 'f':
 		if (!strcasecmp(arg, "f.i")) {
@@ -714,8 +733,15 @@ GOOD_OR_BAD owopt(const int option_char, const char *arg)
 		// shortcut for --foreground --error_level=9
 		printf("DEBUG MODE\n");
 		printf("libow version:\n\t" VERSION "\n");
-		Globals.want_background = 0 ; //foreground
+		switch (Globals.daemon_status) {
+			case e_daemon_want_bg:
+				Globals.daemon_status = e_daemon_fg ;
+				break ;
+			default:
+				break ;
+		}		
 		Globals.error_level = 9 ;
+		Globals.error_level_restore = 9 ;
 		break ;
 	case e_concurrent_connections:
 		RETURN_BAD_IF_BAD(OW_parsevalue_I(&arg_to_integer, arg)) ;
@@ -733,6 +759,32 @@ GOOD_OR_BAD owopt(const int option_char, const char *arg)
 		RETURN_BAD_IF_BAD(OW_parsevalue_I(&arg_to_integer, arg)) ;
 		Globals.max_clients = (int) arg_to_integer;
 		break;
+	case e_want_background:
+		switch (Globals.daemon_status) {
+			case e_daemon_sd:
+				LEVEL_DEFAULT("systemd mode -- ignore background request");
+				break ;
+			case e_daemon_fg:
+			case e_daemon_unknown:
+				Globals.daemon_status = e_daemon_want_bg ;
+				break ;
+			default:
+				break ;
+		}		
+		break ;
+	case e_want_foreground:
+		switch (Globals.daemon_status) {
+			case e_daemon_sd:
+				LEVEL_DEFAULT("systemd mode -- ignore foreground request");
+				break ;
+			case e_daemon_want_bg:
+			case e_daemon_unknown:
+				Globals.daemon_status = e_daemon_fg ;
+				break ;
+			default:
+				break ;
+		}		
+		break ;
 	case e_i2c:
 		return ARG_I2C(arg);
 	case e_ha5:
@@ -749,6 +801,8 @@ GOOD_OR_BAD owopt(const int option_char, const char *arg)
 		return ARG_Link(arg);
 	case e_passive:
 		return ARG_Passive("Passive", arg);
+	case e_pbm:
+		return ARG_PBM(arg);
 	case e_ha3:
 		return ARG_Passive("HA3", arg);
 	case e_ha4b:
@@ -761,6 +815,8 @@ GOOD_OR_BAD owopt(const int option_char, const char *arg)
 		return ARG_Mock(arg);
 	case e_etherweather:
 		return ARG_EtherWeather(arg);
+	case e_masterhub:
+		return ARG_MasterHub(arg);
 	case e_w1_monitor:
 		return ARG_W1_monitor();
 	case e_usb_monitor:
@@ -829,6 +885,9 @@ GOOD_OR_BAD owopt(const int option_char, const char *arg)
 		break;
 	case e_safemode:
 		LocalControlFlags |= SAFEMODE ;
+		break ;
+	case e_detail:
+		return Detail_Add(arg) ;
 		break;
 	case 0:
 		break;
