@@ -1,9 +1,9 @@
 /*
-$Id: ow_parse_sn.c,v 1.6 2010/05/05 02:15:26 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -25,7 +25,7 @@ enum parse_serialnumber Parse_SerialNumber(char *sn_char, BYTE * sn)
 		if (isxdigit(*sn_char)) {
 			ID[i] = *sn_char;
 		} else {
-			return sn_alias; // non-hex
+			return sn_not_sn; // non-hex
 		}
 	}
 	sn[0] = string2num(&ID[0]);
@@ -48,6 +48,33 @@ enum parse_serialnumber Parse_SerialNumber(char *sn_char, BYTE * sn)
 		}
 		sn_char += 2 ;
 	}
-	return ( *sn_char == '\0' ) ? sn_valid : sn_alias ;
+	return ( *sn_char == '\0' ) ? sn_valid : sn_not_sn ;
 }
 
+// returns number of valid bytes in serial number
+int SerialNumber_length(char *sn_char, BYTE * sn)
+{
+	int bytes;
+
+	for ( bytes = 0 ; bytes < SERIAL_NUMBER_SIZE ; ++bytes ) {
+		char ID[2] ;
+		if (*sn_char == '.') { // ignore dots
+			++sn_char;
+		}
+		if (isxdigit(*sn_char)) {
+			ID[0] = *sn_char;
+		} else {
+			return bytes; // non-hex
+		}
+		++sn_char ;
+		if (isxdigit(*sn_char)) {
+			ID[1] = *sn_char;
+		} else {
+			return bytes; // non-hex
+		}
+		sn[bytes] = string2num(ID);
+		++sn_char ;
+	}
+	return SERIAL_NUMBER_SIZE ;
+}
+	
