@@ -1,9 +1,8 @@
 /*
-$Id: ow_parsename.c,v 1.203 2011/06/23 01:21:20 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -13,6 +12,7 @@ $Id: ow_parsename.c,v 1.203 2011/06/23 01:21:20 alfille Exp $
 #include "owfs_config.h"
 #include "ow_devices.h"
 #include "ow_counters.h"
+#include "ow_external.h"
 
 static ZERO_OR_ERROR BranchAdd(struct parsedname *pn);
 
@@ -38,6 +38,7 @@ static enum parse_enum Parse_Property(char *filename, struct parsedname *pn);
 
 static enum parse_enum Parse_RealDeviceSN(enum parse_pass remote_status, struct parsedname *pn);
 static enum parse_enum Parse_NonRealDevice(char *filename, struct parsedname *pn);
+static enum parse_enum Parse_External_Device( char *filename, struct parsedname *pn) ;
 static enum parse_enum Parse_Bus(char *pathnow, struct parsedname *pn);
 static enum parse_enum Parse_Alias(char *filename, enum parse_pass remote_status, struct parsedname *pn);
 static enum parse_enum Parse_Alias_Known( char *filename, enum parse_pass remote_status, struct parsedname *pn);
@@ -45,6 +46,7 @@ static void ReplaceAliasInPath( char * filename, struct parsedname * pn);
 
 static ZERO_OR_ERROR FS_ParsedName_anywhere(const char *path, enum parse_pass remote_status, struct parsedname *pn);
 static ZERO_OR_ERROR FS_ParsedName_setup(struct parsedname_pointers *pp, const char *path, struct parsedname *pn);
+static char * find_segment_in_path( char * segment, char * path ) ;
 
 #define BRANCH_INCR (9)
 
@@ -58,15 +60,20 @@ void FS_ParsedName_destroy(struct parsedname *pn)
 	}
 	LEVEL_DEBUG("%s", SAFESTRING(pn->path));
 	CONNIN_RUNLOCK ;
+	Detail_Free( pn ) ;
+	SAFEFREE(pn->sparse_name);
 	SAFEFREE(pn->bp) ;
-	SAFEFREE(pn->path) ;
 }
 
-// Path is either NULL (in which case a minimal structure is created that doesn't need Destroy -- used for Bus Master setups)
-// Or Path is a full "filename" type string of form: 10.1243Ab000 or uncached/bus.0/statistics etc.
-// The Path passed in isn't altered, but 2 copies are made -- one with the full path, the other (to_server) has the first bus.n removed.
-// An initial / is added to the path, and the full length has to be less than MAX_PATH (2048)
-// For efficiency, the two path copies are allocated in the same call, and so can be removed together.
+/* 
+ * Path is either NULL (in which case a minimal structure is created that doesn't need Destroy -- used for Bus Master setups)
+ * or Path is a full "filename" type string of form: 10.1243Ab000 or uncached/bus.0/statistics etc.
+ * 
+ * The Path passed in isn't altered, but 2 copies are made -- one with the full path, the other (to_server) has the first bus.n removed.
+ * An initial / is added to the path, and the full length has to be less than MAX_PATH (2048)
+ * 
+ * For efficiency, the two path copies are allocated in the same memory allocation call, and so can be removed together.
+ * */
 
 /* Parse a path to check it's validity and attach to the propery data structures */
 ZERO_OR_ERROR FS_ParsedName(const char *path, struct parsedname *pn)
@@ -94,12 +101,10 @@ static ZERO_OR_ERROR FS_ParsedName_anywhere(const char *path, enum parse_pass re
 
 	LEVEL_CALL("path=[%s]", SAFESTRING(path));
 
-	parse_error_status = FS_ParsedName_setup(pp, path, pn);
-	if (parse_error_status) {
-		return parse_error_status;
-	}
+	RETURN_CODE_ERROR_RETURN( FS_ParsedName_setup(pp, path, pn) );
+
 	if (path == NO_PATH) {
-		return 0;
+		RETURN_CODE_RETURN( 0 ) ; // success (by default)
 	}
 
 	while (1) {
@@ -116,8 +121,7 @@ static ZERO_OR_ERROR FS_ParsedName_anywhere(const char *path, enum parse_pass re
 
 			if ( pp->pathnext != NULL ) {
 				// extra text -- make this an error
-				LEVEL_DEBUG("Extra text in file %s",pp->pathnext) ;
-				parse_error_status = -ENOENT;
+				RETURN_CODE_SET_SCALAR( parse_error_status, 77 ) ; // extra text in path
 				pe = parse_done;
 				continue;				
 			}
@@ -126,6 +130,7 @@ static ZERO_OR_ERROR FS_ParsedName_anywhere(const char *path, enum parse_pass re
 			// Play with remote levels
 			switch ( pn->type ) {
 				case ePN_interface:
+					// /interface is interesting -- it's actually a property of the calling server
 					if ( SpecifiedVeryRemoteBus(pn) ) {
 						// veryremote -> remote
 						pn->state &= ~ePS_busveryremote ;
@@ -149,11 +154,12 @@ static ZERO_OR_ERROR FS_ParsedName_anywhere(const char *path, enum parse_pass re
 					break ;
 			}
 			//printf("%s: Parse %s after  corrections: %.4X -- state = %d\n\n",(back_from_remote)?"BACK":"FORE",pn->path,pn->state,pn->type) ;
+			// set up detail debugging
+			Detail_Test( pn ) ; // turns on debug mode only during this device's query
 			return 0;
 
 		case parse_error:
-			//LEVEL_DEBUG("PARSENAME parse_error") ;
-			parse_error_status = -ENOENT;
+			RETURN_CODE_SET_SCALAR( parse_error_status, 27 ) ; // bad path syntax
 			pe = parse_done;
 			continue;
 
@@ -161,16 +167,28 @@ static ZERO_OR_ERROR FS_ParsedName_anywhere(const char *path, enum parse_pass re
 			break;
 		}
 
-		// break out next name in path, make sure pp->pathnext isn't NULL. (SIGSEGV in uClibc)
-		pp->pathnow = (pp->pathnext != NULL) ? strsep(&(pp->pathnext), "/") : NULL;
-		//LEVEL_DEBUG("PARSENAME pathnow=[%s] rest=[%s]",pp->pathnow,pp->pathnext) ;
-		if (pp->pathnow == NULL || pp->pathnow[0] == '\0') {
-			pe = parse_done;
-			continue;
+		// break out next name in path
+		if ( pp->pathnext == NULL ) {
+			// make sure pp->pathnext isn't NULL. (SIGSEGV in uClibc)
+			pp->pathnow = NULL ;
+		} else {
+			pp->pathnow = strsep(&(pp->pathnext), "/") ;
 		}
+		//LEVEL_DEBUG("PARSENAME pathnow=[%s] rest=[%s]",pp->pathnow,pp->pathnext) ;
+
+		// test next path segment for existence
+		if (pp->pathnow == NULL || pp->pathnow[0] == '\0') {
+			// done parsing
+			pe = parse_done;
+		}
+
 		// rest of state machine on parsename
 		switch (pe) {
 
+		case parse_done:
+			// nothing left to process -- will be handled in next loop pass
+			break ;
+		
 		case parse_first:
 			//LEVEL_DEBUG("PARSENAME parse_first") ;
 			pe = Parse_Unspecified(pp->pathnow, remote_status, pn);
@@ -219,11 +237,13 @@ static ZERO_OR_ERROR FS_ParsedName_anywhere(const char *path, enum parse_pass re
 static ZERO_OR_ERROR FS_ParsedName_setup(struct parsedname_pointers *pp, const char *path, struct parsedname *pn)
 {
 	if (pn == NO_PARSEDNAME) {
-		return -EINVAL;
+		RETURN_CODE_RETURN( 78 ); // unexpected null pointer
 	}
 
 	memset(pn, 0, sizeof(struct parsedname));
 	pn->known_bus = NULL;		/* all buses */
+	pn->sparse_name = NULL ;
+	RETURN_CODE_INIT(pn);
 
 	/* Set the persistent state info (temp scale, ...) -- will be overwritten by client settings in the server */
 	CONTROLFLAGSLOCK;
@@ -263,22 +283,16 @@ static ZERO_OR_ERROR FS_ParsedName_setup(struct parsedname_pointers *pp, const c
 	/* minimal structure for initial bus "detect" use -- really has connection and LocalControlFlags only */
 	pn->dirlength = -1 ;
 	if (path == NO_PATH) {
-		return 0;
+		return 0; // success
 	}
 
 	if (strlen(path) > PATH_MAX) {
-		return -EBADF;
-	}
-
-	pn->path = (char *) owmalloc(2 * PATH_MAX + 4);
-	if (pn->path == NO_PATH) {
-		return -ENOMEM;
+		RETURN_CODE_RETURN( 26 ) ; // path too long
 	}
 
 	/* Have to save pn->path at once */
 	strcpy(pn->path, "/"); // initial slash
 	strcpy(pn->path+1, path[0]=='/'?path+1:path);
-	pn->path_to_server = pn->path + strlen(pn->path) + 1;
 	strcpy(pn->path_to_server, pn->path);
 
 	/* make a copy for destructive parsing  without initial '/'*/
@@ -286,6 +300,9 @@ static ZERO_OR_ERROR FS_ParsedName_setup(struct parsedname_pointers *pp, const c
 	/* pointer to rest of path after current token peeled off */
 	pp->pathnext = pp->pathcpy;
 	pn->dirlength = strlen(pn->path) ;
+	
+	/* device name */
+	pn->device_name = NULL ;
 
 	/* connection_in list and start */
 	/* ---------------------------- */
@@ -299,7 +316,7 @@ static ZERO_OR_ERROR FS_ParsedName_setup(struct parsedname_pointers *pp, const c
 	CONNIN_RLOCK;
 	pn->selected_connection = NO_CONNECTION ; // Default bus assignment
 
-	return 0;
+	return 0 ; // success
 }
 
 /* Used for virtual directories like settings and statistics
@@ -348,6 +365,10 @@ static enum parse_enum Parse_Unspecified(char *pathnow, enum parse_pass remote_s
 		pn->state |= ePS_text;
 		return parse_first;
 
+	} else if (strcasecmp(pathnow, "json") == 0) {
+		pn->state |= ePS_json;
+		return parse_first;
+
 	} else if (strcasecmp(pathnow, "uncached") == 0) {
 		pn->state |= ePS_uncached;
 		return parse_first;
@@ -375,7 +396,6 @@ static enum parse_enum Parse_Branch(char *pathnow, enum parse_pass remote_status
 static enum parse_enum Parse_Real(char *pathnow, enum parse_pass remote_status, struct parsedname *pn)
 {
 	if (strcasecmp(pathnow, "simultaneous") == 0) {
-		LEVEL_DEBUG("TEST set simultaneous");
 		pn->selected_device = DeviceSimultaneous;
 		return parse_prop;
 
@@ -383,6 +403,10 @@ static enum parse_enum Parse_Real(char *pathnow, enum parse_pass remote_status,
 		pn->state |= ePS_text;
 		return parse_real;
 
+	} else if (strcasecmp(pathnow, "json") == 0) {
+		pn->state |= ePS_json;
+		return parse_real;
+
 	} else if (strcasecmp(pathnow, "thermostat") == 0) {
 		pn->selected_device = DeviceThermostat;
 		return parse_prop;
@@ -406,6 +430,10 @@ static enum parse_enum Parse_NonReal(char *pathnow, struct parsedname *pn)
 		pn->state |= ePS_text;
 		return parse_nonreal;
 
+	} else if (strcasecmp(pathnow, "json") == 0) {
+		pn->state |= ePS_json;
+		return parse_nonreal;
+
 	} else if (strcasecmp(pathnow, "uncached") == 0) {
 		pn->state |= ePS_uncached;
 		return parse_nonreal;
@@ -472,40 +500,51 @@ static enum parse_enum Parse_Bus(char *pathnow, struct parsedname *pn)
 	return parse_first;
 }
 
+// search path for this exact mathing path segment
+static char * find_segment_in_path( char * segment, char * path )
+{
+	int segment_length = strlen(segment) ;
+	char augmented_segment[ segment_length + 2 ] ;
+	
+	char * path_pointer = path ;
+	
+	augmented_segment[0] = '/' ;
+	strcpy( &augmented_segment[1], segment ) ;
+	
+	while ( (path_pointer = strstr( path_pointer , augmented_segment )) != NULL ) {
+		++ path_pointer ; // point after '/'
+		switch( path_pointer[segment_length] ) {
+			case '\0':
+			case '/':
+				return path_pointer ;
+			default:
+				// not a full match -- try again
+				break ;
+		}
+	}
+	
+	return NULL ;
+}
+
 /* replace alias with sn */
 static void ReplaceAliasInPath( char * filename, struct parsedname * pn)
 {
 	int alias_len = strlen(filename) ;
 
-	char alias[alias_len + 2] ;
-
-	char * alias_loc ;
-	char * post_alias_loc ;
-
 	// check total length
-	if ( strlen(pn->path_to_server) + 14 - alias_len > PATH_MAX ) {
-		return ;
-	}
-	
-	strcpy( alias, "/") ;
-	strcat( alias, filename ) ; // we include an initial / to make the match more accurate
+	if ( strlen(pn->path_to_server) + 14 - alias_len <= PATH_MAX ) {
+		// find the alias
+		char * alias_loc = find_segment_in_path( filename, pn->path_to_server ) ;
+		
+		if ( alias_loc != NULL ) {
+			char * post_alias_loc ;
 
-	for ( alias_loc = pn->path_to_server; alias_loc != NULL ; alias_loc = strstr( alias_loc, alias ) ) {
-		++alias_loc ; // point after '/'
+			post_alias_loc = alias_loc + alias_len ;
 
-		post_alias_loc = alias_loc + alias_len ;
-		switch ( post_alias_loc[0] ) {
-			case '\0':
-			case '/':
-				// move rest of path
-				memmove( &alias_loc[14], post_alias_loc, strlen(post_alias_loc)+1 ) ;
-				//write in serial number for alias
-				bytes2string( alias_loc, pn->sn, 7 ) ;
-				return ;
-			default:
-				// alias not really found (only partial match)
-				// loop starting one char later
-				break ;
+			// move rest of path
+			memmove( &alias_loc[14], post_alias_loc, strlen(post_alias_loc)+1 ) ;
+			//write in serial number for alias
+			bytes2string( alias_loc, pn->sn, 7 ) ;
 		}
 	}
 }
@@ -571,9 +610,14 @@ static enum parse_enum Parse_Alias(char *filename, enum parse_pass remote_status
  */
 static enum parse_enum Parse_RealDevice(char *filename, enum parse_pass remote_status, struct parsedname *pn)
 {
+	pn->device_name = find_segment_in_path( filename, pn->path ) ;
 	switch ( Parse_SerialNumber(filename,pn->sn) ) {
-		case sn_alias:
-			return Parse_Alias( filename, remote_status, pn) ;
+		case sn_not_sn:
+			if ( Find_External_Sensor( filename ) ) {
+				return Parse_External_Device( filename, pn ) ;
+			} else {
+				return Parse_Alias( filename, remote_status, pn) ;
+			}
 		case sn_valid:
 			return Parse_RealDeviceSN( remote_status, pn ) ;
 		case sn_invalid:
@@ -594,6 +638,7 @@ static enum parse_enum Parse_RealDeviceSN(enum parse_pass remote_status, struct
 	}
 
 	if (Globals.one_device) {
+		// Single slave device -- use faster routines
 		SetKnownBus(INDEX_DEFAULT, pn);
 	} else {
 		/* Check the presence, and cache the proper bus number for better performance */
@@ -606,6 +651,18 @@ static enum parse_enum Parse_RealDeviceSN(enum parse_pass remote_status, struct
 	return parse_prop;
 }
 
+/* Device is known with serial number */
+static enum parse_enum Parse_External_Device( char *filename, struct parsedname *pn)
+{
+	struct sensor_node * sensor_n = Find_External_Sensor( filename ) ;
+	struct family_node * family_n = Find_External_Family( sensor_n->family ) ;
+	
+	pn->selected_device = &(family_n->dev) ;
+	SetKnownBus(Inbound_Control.external->index, pn);
+	return parse_prop;
+}
+
+
 /* Parse Name (non-device name) part of string */
 /* Return -ENOENT if not a valid name
    return 0 if good
@@ -613,6 +670,7 @@ static enum parse_enum Parse_RealDeviceSN(enum parse_pass remote_status, struct
 static enum parse_enum Parse_NonRealDevice(char *filename, struct parsedname *pn)
 {
 	//printf("Parse_NonRealDevice: [%s] [%s]\n", filename, pn->path);
+	pn->device_name = find_segment_in_path( filename, pn->path ) ;
 	FS_devicefind(filename, pn);
 	return (pn->selected_device == &UnknownDevice) ? parse_error : parse_prop;
 }
@@ -625,104 +683,127 @@ static enum parse_enum Parse_Property(char *filename, struct parsedname *pn)
 
 	// Special case for remote device. Use distant data
 	if ( pn->selected_device == &RemoteDevice ) {
-		// remote device, no known sn, can't handle a prolerty
+		// remote device, no known sn, can't handle a property
 		return parse_error ;
 	}
 
+	// separate filename.dot
 	filename = strsep(&dot, ".");
 	//printf("FP name=%s, dot=%s\n", filename, dot);
+
 	/* Match to known filetypes for this device */
-	if ((pn->selected_filetype =
+	pn->selected_filetype =
 		 bsearch(filename, pn->selected_device->filetype_array,
-				 (size_t) pn->selected_device->count_of_filetypes, sizeof(struct filetype), filetype_cmp))) {
-		//printf("FP known filetype %s\n",pn->selected_filetype->name) ;
-		/* Filetype found, now process extension */
-		if (dot == NULL || dot[0] == '\0') {	/* no extension */
-			if (pn->selected_filetype->ag != NON_AGGREGATE) {
-				return parse_error;	/* aggregate filetypes need an extension */
+				 (size_t) pn->selected_device->count_of_filetypes, sizeof(struct filetype), filetype_cmp) ;
+				 
+	if ( pn->selected_filetype == NO_FILETYPE ) {
+		LEVEL_DEBUG("Unknown property for this device %s",SAFESTRING(filename) ) ;
+		return parse_error;			/* filetype not found */
+	}
+		
+	//printf("FP known filetype %s\n",pn->selected_filetype->name) ;
+	/* Filetype found, now process extension */
+	if (dot == NULL || dot[0] == '\0') {	/* no extension */
+		if (pn->selected_filetype->ag != NON_AGGREGATE) {
+			return parse_error;	/* aggregate filetypes need an extension */
+		}
+		pn->extension = 0;	/* default when no aggregate */
+
+	// Non-aggregate cannot have an extension
+	} else if (pn->selected_filetype->ag == NON_AGGREGATE) {
+		return parse_error;	/* An extension not allowed when non-aggregate */
+
+	// Sparse uses the extension verbatim (text or number)
+	} else if (pn->selected_filetype->ag->combined==ag_sparse)  { /* Sparse */
+		if (pn->selected_filetype->ag->letters == ag_letters) {	/* text string */
+			pn->extension = 0;	/* text extension, not number */
+			pn->sparse_name = owstrdup(dot) ;
+		} else {			/* Numbers */
+			char *p;
+			//printf("FP numbers\n") ;
+			pn->extension = strtol(dot, &p, 0);	/* Number conversion */
+			if ((p == dot) || ((pn->extension == 0) && (errno == -EINVAL))) {
+				return parse_error;	/* Bad number */
 			}
-			pn->extension = 0;	/* default when no aggregate */
-
-		} else if (pn->selected_filetype->ag == NON_AGGREGATE) {
-			return parse_error;	/* An extension not allowed when non-aggregate */
-
-		} else if (strcasecmp(dot, "ALL") == 0) {
-			//printf("FP ALL\n");
-			pn->extension = EXTENSION_ALL;	/* ALL */
-
-		} else if (pn->selected_filetype->format == ft_bitfield && strcasecmp(dot, "BYTE") == 0) {
-			pn->extension = EXTENSION_BYTE;	/* BYTE */
-			//printf("FP BYTE\n") ;
-
-		} else {				/* specific extension */
-			if (pn->selected_filetype->ag->letters == ag_letters) {	/* Letters */
-				//printf("FP letters\n") ;
-				if ((strlen(dot) != 1) || !isupper(dot[0])) {
-					return parse_error;
-				}
-				pn->extension = dot[0] - 'A';	/* Letter extension */
-			} else {			/* Numbers */
-				char *p;
-				//printf("FP numbers\n") ;
-				pn->extension = strtol(dot, &p, 0);	/* Number conversion */
-				if ((p == dot) || ((pn->extension == 0) && (errno == -EINVAL))) {
-					return parse_error;	/* Bad number */
-				}
-			}
-			//printf("FP ext=%d nr_elements=%d\n", pn->extension, pn->selected_filetype->ag->elements) ;
-			/* Now check range */
-			if ((pn->extension < 0)
-				|| (pn->extension >= pn->selected_filetype->ag->elements)) {
-				//printf("FP Extension out of range %d %d %s\n", pn->extension, pn->selected_filetype->ag->elements, pn->path);
-				return parse_error;	/* Extension out of range */
-			}
-			//printf("FP in range\n") ;
 		}
 
-		//printf("FP Good\n") ;
-		switch (pn->selected_filetype->format) {
-		case ft_directory:		// aux or main
-			if ( pn->type == ePN_structure ) {
-				// special case, structure for aux and main
-				return parse_done;
-			}
-			if (BranchAdd(pn) != 0) {
-				//printf("PN BranchAdd failed for %s\n", filename);
+	// Non-sparse "ALL"
+	} else if (strcasecmp(dot, "ALL") == 0) {
+		//printf("FP ALL\n");
+		pn->extension = EXTENSION_ALL;	/* ALL */
+	
+	// Non-sparse "BYTE"
+	} else if (pn->selected_filetype->format == ft_bitfield && strcasecmp(dot, "BYTE") == 0) {
+		pn->extension = EXTENSION_BYTE;	/* BYTE */
+		//printf("FP BYTE\n") ;
+
+	// Non-sparse extension -- interpret and check bounds
+	} else {				/* specific extension */
+		if (pn->selected_filetype->ag->letters == ag_letters) {	/* Letters */
+			//printf("FP letters\n") ;
+			if ((strlen(dot) != 1) || !isupper(dot[0])) {
 				return parse_error;
 			}
-			/* STATISTICS */
-			STATLOCK;
-			if (pn->pathlength > dir_depth) {
-				dir_depth = pn->pathlength;
+			pn->extension = dot[0] - 'A';	/* Letter extension */
+		} else {			/* Numbers */
+			char *p;
+			//printf("FP numbers\n") ;
+			pn->extension = strtol(dot, &p, 0);	/* Number conversion */
+			if ((p == dot) || ((pn->extension == 0) && (errno == -EINVAL))) {
+				return parse_error;	/* Bad number */
 			}
-			STATUNLOCK;
-			return parse_branch;
-		case ft_subdir:
-			//printf("PN %s is a subdirectory\n", filename);
-			pn->subdir = pn->selected_filetype;
-			pn->selected_filetype = NO_FILETYPE;
-			return parse_subprop;
-		default:
+		}
+		//printf("FP ext=%d nr_elements=%d\n", pn->extension, pn->selected_filetype->ag->elements) ;
+		/* Now check range */
+		if ((pn->extension < 0)
+			|| (pn->extension >= pn->selected_filetype->ag->elements)) {
+			//printf("FP Extension out of range %d %d %s\n", pn->extension, pn->selected_filetype->ag->elements, pn->path);
+			return parse_error;	/* Extension out of range */
+		}
+		//printf("FP in range\n") ;
+	}
+
+	//printf("FP Good\n") ;
+	switch (pn->selected_filetype->format) {
+	case ft_directory:		// aux or main
+		if ( pn->type == ePN_structure ) {
+			// special case, structure for aux and main
 			return parse_done;
 		}
+		if (BranchAdd(pn) != 0) {
+			//printf("PN BranchAdd failed for %s\n", filename);
+			return parse_error;
+		}
+		/* STATISTICS */
+		STATLOCK;
+		if (pn->ds2409_depth > dir_depth) {
+			dir_depth = pn->ds2409_depth;
+		}
+		STATUNLOCK;
+		return parse_branch;
+	case ft_subdir:
+		//printf("PN %s is a subdirectory\n", filename);
+		pn->subdir = pn->selected_filetype;
+		pn->selected_filetype = NO_FILETYPE;
+		return parse_subprop;
+	default:
+		return parse_done;
 	}
-	//printf("FP not found\n") ;
-	return parse_error;			/* filetype not found */
 }
 
 static ZERO_OR_ERROR BranchAdd(struct parsedname *pn)
 {
 	//printf("BRANCHADD\n");
-	if ((pn->pathlength % BRANCH_INCR) == 0) {
+	if ((pn->ds2409_depth % BRANCH_INCR) == 0) {
 		void *temp = pn->bp;
-		if ((pn->bp = owrealloc(temp, (BRANCH_INCR + pn->pathlength) * sizeof(struct buspath))) == NULL) {
+		if ((pn->bp = owrealloc(temp, (BRANCH_INCR + pn->ds2409_depth) * sizeof(struct ds2409_hubs))) == NULL) {
 			SAFEFREE(temp) ;
-			return -ENOMEM;
+			RETURN_CODE_RETURN( 79 ) ; // unable to allocate memory
 		}
 	}
-	memcpy(pn->bp[pn->pathlength].sn, pn->sn, SERIAL_NUMBER_SIZE);	/* copy over DS2409 name */
-	pn->bp[pn->pathlength].branch = pn->selected_filetype->data.i;
-	++pn->pathlength;
+	memcpy(pn->bp[pn->ds2409_depth].sn, pn->sn, SERIAL_NUMBER_SIZE);	/* copy over DS2409 name */
+	pn->bp[pn->ds2409_depth].branch = pn->selected_filetype->data.i;
+	++pn->ds2409_depth;
 	pn->selected_filetype = NO_FILETYPE;
 	pn->selected_device = NO_DEVICE;
 	return 0;
@@ -748,7 +829,7 @@ ZERO_OR_ERROR FS_ParsedNamePlus(const char *path, const char *file, struct parse
 
 	fullpath = owmalloc(strlen(file) + strlen(path) + 2);
 	if (fullpath == NO_PATH) {
-		return -ENOMEM;
+		RETURN_CODE_RETURN( 79 ) ; // unable to allocate memory
 	}
 	strcpy(fullpath, path);
 	if (fullpath[strlen(fullpath) - 1] != '/') {
@@ -763,20 +844,33 @@ ZERO_OR_ERROR FS_ParsedNamePlus(const char *path, const char *file, struct parse
 	return ret;
 }
 
+
+
 /* Parse a path/file combination */
 ZERO_OR_ERROR FS_ParsedNamePlusExt(const char *path, const char *file, int extension, enum ag_index alphanumeric, struct parsedname *pn)
+{
+	if (extension == EXTENSION_BYTE ) {
+		return FS_ParsedNamePlusText(path, file, "BYTE", pn);
+	} else if (extension == EXTENSION_ALL ) {
+		return FS_ParsedNamePlusText(path, file, "ALL", pn);
+	} else if (alphanumeric == ag_letters) {
+		char name[2] = { 'A'+extension, 0x00, } ;
+		return FS_ParsedNamePlusText(path, file, name, pn);
+	} else {
+		char name[OW_FULLNAME_MAX];
+		UCLIBCLOCK;
+		snprintf(name, OW_FULLNAME_MAX, "%d", extension);
+		UCLIBCUNLOCK;
+		return FS_ParsedNamePlusText(path, file, name, pn);
+	}
+}
+
+/* Parse a path/file combination */
+ZERO_OR_ERROR FS_ParsedNamePlusText(const char *path, const char *file, const char *extension, struct parsedname *pn)
 {
 	char name[OW_FULLNAME_MAX];
 	UCLIBCLOCK;
-	if (extension == EXTENSION_BYTE ) {
-		snprintf(name, OW_FULLNAME_MAX, "%s.BYTE", file);
-	} else if (extension == EXTENSION_ALL ) {
-		snprintf(name, OW_FULLNAME_MAX, "%s.ALL", file);
-	} else if (alphanumeric == ag_letters) {
-		snprintf(name, OW_FULLNAME_MAX, "%s.%c", file, extension + 'A');
-	} else {
-		snprintf(name, OW_FULLNAME_MAX, "%s.%d", file, extension);
-	}
+		snprintf(name, OW_FULLNAME_MAX, "%s.%s", file, extension );
 	UCLIBCUNLOCK;
 	return FS_ParsedNamePlus(path, name, pn);
 }
