@@ -1,9 +1,9 @@
 /*
-$Id: ow_parseobject.c,v 1.60 2011/04/05 19:03:37 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -229,17 +229,21 @@ static GOOD_OR_BAD OWQ_allocate_array( struct one_wire_query * owq )
 void OWQ_assign_read_buffer(char *buffer, size_t size, off_t offset, struct one_wire_query *owq)
 {
 	OWQ_buffer(owq) = buffer;
-	//OWQ_read_buffer(owq) = NULL ;
-	//OWQ_write_buffer(owq) = NULL ;
 	OWQ_size(owq) = size;
 	OWQ_offset(owq) = offset;
 }
 
 void OWQ_assign_write_buffer(const char *buffer, size_t size, off_t offset, struct one_wire_query *owq)
 {
-	OWQ_buffer(owq) = buffer;
-	//OWQ_read_buffer(owq) = NULL ;
-	//OWQ_write_buffer(owq) = NULL ;
+	// OWQ_buffer used for both read (non-const) and write (const)
+#if ( __GNUC__ > 4 ) || (__GNUC__ == 4 && __GNUC_MINOR__ > 4 )
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+	OWQ_buffer(owq) = (char *) buffer;
+#pragma GCC diagnostic pop
+#else
+	OWQ_buffer(owq) = (char *) buffer;
+#endif
 	OWQ_size(owq) = size;
 	OWQ_offset(owq) = offset;
 }
