@@ -1,9 +1,9 @@
 /*
-$Id: ow_read.c,v 1.198 2011/12/22 03:08:32 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -234,9 +234,8 @@ static SIZE_OR_ERROR FS_r_given_bus(struct one_wire_query *owq)
 
 	if (KnownBus(pn) && BusIsServer(pn->selected_connection)) {
 		/* The bus is not local... use a network connection instead */
-#if OW_MT
 		LEVEL_DEBUG("pid=%ld call ServerRead", pthread_self());
-#endif							/* OW_MT */
+
 		// Read afar -- returns already formatted in buffer
 		read_or_error = ServerRead(owq);
 		LEVEL_DEBUG("back from server");
@@ -443,6 +442,9 @@ static ZERO_OR_ERROR FS_structure(struct one_wire_query *owq)
 	struct filetype * ftype = pn->selected_filetype ;
 	char format_char ;
 	char change_char ;
+	int index_num ;
+	int elements_num ;
+	
 
 	// TEMPORARY change type from structure->real to get real length rhather than structure length
 	pn->type = ePN_real;	/* "real" type to get return length, rather than "structure" length */
@@ -494,6 +496,24 @@ static ZERO_OR_ERROR FS_structure(struct one_wire_query *owq)
 			break ;
 	}
 
+	// Set index
+	index_num = (ftype->ag) ? pn->extension : 0 ;
+	
+	// Set elements
+	if ( ftype-> ag ) { // array
+		if ( ftype -> ag -> combined == ag_sparse) {
+			if ( ftype -> ag -> letters == ag_letters) {
+				elements_num = -1 ;
+			} else { // numbered
+				elements_num = 0 ;
+			}
+		} else { // non sparse
+			elements_num = ftype->ag->elements ;
+		}
+	} else { // scallar
+		elements_num = 1 ;
+	}
+
 	// Set changability
 	switch ( ftype->change ) {
 		case fc_static:
@@ -527,8 +547,8 @@ static ZERO_OR_ERROR FS_structure(struct one_wire_query *owq)
 		 PROPERTY_LENGTH_STRUCTURE+1,
 		 "%c,%.6d,%.6d,%.2s,%.6d,%c,",
 		 format_char,
-		 (ftype->ag) ? pn->extension : 0,
-		 (ftype->ag) ? ftype->ag->elements : 1,
+		 index_num ,
+		 elements_num,
 		 (ftype->read == NO_READ_FUNCTION) ? ((ftype->write == NO_WRITE_FUNCTION) ? "oo" : "wo")
 		                                   : ((ftype->write == NO_WRITE_FUNCTION) ? "ro" : "rw"), 
 		 (int) FullFileLength(PN(owq)),
@@ -561,20 +581,21 @@ static ZERO_OR_ERROR FS_read_all( struct one_wire_query *owq_all)
 	// bitfield, convert to .BYTE format and write ( and delete cache ) as BYTE.
 	if ( pn->selected_filetype->format == ft_bitfield ) {
 		struct one_wire_query * owq_byte = OWQ_create_separate( EXTENSION_BYTE, owq_all ) ;
+		int ret = -EINVAL ;
+
 		if ( owq_byte != NO_ONE_WIRE_QUERY ) {
 			if ( FS_read_owq( owq_byte ) >= 0 ) {
 				size_t elements = pn->selected_filetype->ag->elements;
 				size_t extension;
 			
 				for ( extension=0 ; extension < elements ; ++extension ) {
-					OWQ_array_Y(owq_all,extension) = UT_getbit( (BYTE *) &OWQ_U(owq_byte), extension ) ;
+					OWQ_array_Y(owq_all,extension) = UT_getbit_U( OWQ_U(owq_byte), extension ) ;
 				}
-				return 0 ;
+				ret = 0 ;
 			}
 			OWQ_destroy( owq_byte ) ;
-			return -EINVAL ;
 		}
-		return -ENOENT ;
+		return ret ;
 	}
 	return FS_read_owq( owq_all ) ;
 }
@@ -626,7 +647,7 @@ static ZERO_OR_ERROR FS_read_in_parts( struct one_wire_query *owq_all )
 			return -EINVAL ;
 		}
 		for (extension = 0; extension < elements; ++extension) {
-			OWQ_array_Y(owq_all,extension) = UT_getbit( (BYTE *) &OWQ_U(owq_part), extension ) ;
+			OWQ_array_Y(owq_all,extension) = UT_getbit_U( OWQ_U(owq_part), extension ) ;
 		}
 		OWQ_destroy( owq_part ) ;
 		return 0 ;
@@ -694,7 +715,7 @@ static ZERO_OR_ERROR FS_read_all_bits(struct one_wire_query *owq_byte)
 			OWQ_destroy(owq_bit);
 			return -EINVAL;
 		}
-		UT_setbit( (BYTE *) &OWQ_U(owq_byte), extension, OWQ_Y(owq_bit) ) ;
+		UT_setbit_U( &OWQ_U(owq_byte), extension, OWQ_Y(owq_bit) ) ;
 	}
 
 	OWQ_destroy(owq_bit);
@@ -715,7 +736,7 @@ static ZERO_OR_ERROR FS_read_a_bit( struct one_wire_query *owq_bit )
 
 	/* read the UINT */
 	if ( FS_read_owq(owq_byte) >= 0) {
-		OWQ_Y(owq_bit) = UT_getbit((void *) &OWQ_U(owq_byte), pn->extension) ;
+		OWQ_Y(owq_bit) = UT_getbit_U( OWQ_U(owq_byte), pn->extension) ;
 		z_or_e = 0 ;
 	}
 
