@@ -1,5 +1,5 @@
 /*
-$Id: ow_reconnect.c,v 1.29 2011/07/27 19:31:35 alfille Exp $
+$Id: ow_reconnect.c,v 1.31 2012/09/28 20:43:34 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -54,7 +54,7 @@ GOOD_OR_BAD TestConnection(const struct parsedname *pn)
 			// reconnect method exists
 			ret = (in->iroutines.reconnect) (pn) ;	// call bus-specific reconnect
 		} else {
-			ret = BUS_detect(in) ;	// call initial opener
+			ret = BUS_detect(in->pown) ;	// call initial opener
 		}
 		if ( BAD( ret ) ) {
 			in->reconnect_state = reconnect_ok + 1 ;
