@@ -1,9 +1,9 @@
 /*
-$Id: ow_serial_free.c,v 1.1 2010/12/22 02:27:10 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-	email: palfille@earthlink.net
+	email: paul.alfille@gmail.com
 	Released under the GPL
 	See the header file: ow.h for full attribution
 	1wire/iButton system from Dallas Semiconductor
@@ -23,19 +23,16 @@ $Id: ow_serial_free.c,v 1.1 2010/12/22 02:27:10 alfille Exp $
 /* ---------------------------------------------- */
 
 //free serial port and restore attributes
-
+/* Called on head of multibus group */
 void serial_free(struct connection_in *connection)
 {
 	FILE_DESCRIPTOR_OR_ERROR fd ;
+	struct port_in * pin = connection->pown ;
 
-	if ( SOC(connection)->state == cs_virgin ) {
-		return ;
-	}
-
-	fd = SOC(connection)->file_descriptor ;
+	fd = pin->file_descriptor ;
 	if ( FILE_DESCRIPTOR_NOT_VALID( fd ) ) {
 		// reopen to restore attributes
-		fd = open( SOC(connection)->devicename, O_RDWR | O_NONBLOCK | O_NOCTTY) ;
+		fd = open( pin->init_data, O_RDWR | O_NONBLOCK | O_NOCTTY ) ;
 	}
 
 	// restore tty settings
@@ -43,10 +40,10 @@ void serial_free(struct connection_in *connection)
 		LEVEL_DEBUG("COM_close: flush");
 		tcflush( fd, TCIOFLUSH);
 		LEVEL_DEBUG("COM_close: restore");
-		if ( tcsetattr( fd, TCSANOW, &(SOC(connection)->dev.serial.oldSerialTio) ) < 0) {
-			ERROR_CONNECT("Cannot restore port attributes: %s", SAFESTRING(SOC(connection)->devicename));
+		if ( tcsetattr( fd, TCSANOW, &(pin->dev.serial.oldSerialTio) ) < 0) {
+			ERROR_CONNECT("Cannot restore port attributes: %s", pin->init_data);
 		}
 	}
-	Test_and_Close( &( SOC(connection)->file_descriptor) ) ;
+	Test_and_Close( &( pin->file_descriptor) ) ;
 }
 
