@@ -1,9 +1,9 @@
 /*
-$Id: ow_server_message.c,v 1.30 2011/12/22 03:08:32 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -193,7 +193,7 @@ ZERO_OR_ERROR ServerDir(void (*dirfunc) (void *, const struct parsedname * const
 	struct connection_in * in = pn_whole_directory->selected_connection ;
 
 	// Do we know this server doesn't support DIRALL?
-	if (in->master.tcp.no_dirall) {
+	if (in->master.server.no_dirall) {
 		return ServerDIR(dirfunc, v, pn_whole_directory, flags);
 	}
 	// Did we ask for no DIRALL explicitly?
@@ -206,7 +206,7 @@ ZERO_OR_ERROR ServerDir(void (*dirfunc) (void *, const struct parsedname * const
 	}
 	// try DIRALL and see if supported
 	if ((ret = ServerDIRALL(dirfunc, v, pn_whole_directory, flags)) == -ENOMSG) {
-		in->master.tcp.no_dirall = 1;
+		in->master.server.no_dirall = 1;
 		return ServerDIR(dirfunc, v, pn_whole_directory, flags);
 	}
 	return ret;
@@ -217,7 +217,6 @@ static ZERO_OR_ERROR ServerDIR(void (*dirfunc) (void *, const struct parsedname
 {
 	struct server_msg sm;
 	struct client_msg cm;
-	struct connection_in * in = pn_whole_directory->selected_connection ;
 	struct serverpackage sp = { pn_whole_directory->path_to_server, NULL, 0,
 		pn_whole_directory->tokenstring, pn_whole_directory->tokens,
 	};
@@ -233,7 +232,7 @@ static ZERO_OR_ERROR ServerDIR(void (*dirfunc) (void *, const struct parsedname
 	sm.type = msg_dir;
 
 	LEVEL_CALL("SERVER(%d) path=%s path_to_server=%s",
-			   in->index, SAFESTRING(pn_whole_directory->path), SAFESTRING(pn_whole_directory->path_to_server));
+			   scs.in->index, SAFESTRING(pn_whole_directory->path), SAFESTRING(pn_whole_directory->path_to_server));
 
 	// Send to owserver
 	sm.control_flags = SetupControlFlags( pn_whole_directory);
@@ -479,10 +478,11 @@ static SIZE_OR_ERROR From_Server( struct server_connection_state * scs, struct c
 {
 	size_t rtry;
 	size_t actual_read ;
-	struct timeval tv = { Globals.timeout_network + 1, 0, };
+	struct timeval tv1 = { Globals.timeout_network + 1, 0, };
+	struct timeval tv2 = { Globals.timeout_network + 1, 0, };
 
 	do {						// read regular header, or delay (delay when payload<0)
-		tcp_read(scs->file_descriptor, (BYTE *) cm, sizeof(struct client_msg), &tv, &actual_read);
+		tcp_read(scs->file_descriptor, (BYTE *) cm, sizeof(struct client_msg), &tv1, &actual_read);
 		if (actual_read != sizeof(struct client_msg)) {
 			cm->size = 0;
 			cm->ret = -EIO;
@@ -500,7 +500,7 @@ static SIZE_OR_ERROR From_Server( struct server_connection_state * scs, struct c
 		return 0;				// No payload, done.
 	}
 	rtry = cm->payload < (ssize_t) size ? (size_t) cm->payload : size;
-	tcp_read(scs->file_descriptor, (BYTE *) msg, rtry, &tv, &actual_read);	// read expected payload now.
+	tcp_read(scs->file_descriptor, (BYTE *) msg, rtry, &tv2, &actual_read);	// read expected payload now.
 	if (actual_read != rtry) {
 		LEVEL_DEBUG("Read only %d of %d\n",(int)actual_read,(int)rtry) ;
 		cm->ret = -EIO;
@@ -516,6 +516,7 @@ static SIZE_OR_ERROR From_Server( struct server_connection_state * scs, struct c
 static GOOD_OR_BAD To_Server( struct server_connection_state * scs, struct server_msg * sm, struct serverpackage *sp)
 {
 	struct connection_in * in = scs->in ; // for convenience
+	struct port_in * pin = in->pown ;
 	BYTE test_read[1] ;
 	int old_flags ;
 	ssize_t rcv_value ;
@@ -532,7 +533,7 @@ static GOOD_OR_BAD To_Server( struct server_connection_state * scs, struct serve
 	} else {
 		// Persistence desired
 		BUSLOCKIN(in);
-		switch ( SOC(in)->file_descriptor ) {
+		switch ( pin->file_descriptor ) {
 			case FILE_DESCRIPTOR_PERSISTENT_IN_USE:
 				// Currently in use, so make new non-persistent connection
 				scs->file_descriptor = ClientConnect(in);
@@ -542,14 +543,14 @@ static GOOD_OR_BAD To_Server( struct server_connection_state * scs, struct serve
 				// no conection currently, so make a new one
 				scs->file_descriptor = ClientConnect(in);
 				if ( FILE_DESCRIPTOR_VALID( scs->file_descriptor ) ) {
-					SOC(in)->file_descriptor = FILE_DESCRIPTOR_PERSISTENT_IN_USE ;
+					pin->file_descriptor = FILE_DESCRIPTOR_PERSISTENT_IN_USE ;
 				}	
 				break ;
 			default:
 				// persistent connection idle and waiting for use
 				// connection_in is locked so this is safe
-				scs->file_descriptor = SOC(in)->file_descriptor;
-				SOC(in)->file_descriptor = FILE_DESCRIPTOR_PERSISTENT_IN_USE;
+				scs->file_descriptor = pin->file_descriptor;
+				pin->file_descriptor = FILE_DESCRIPTOR_PERSISTENT_IN_USE;
 			break ;
 		}
 		BUSUNLOCKIN(in);
@@ -586,7 +587,7 @@ static GOOD_OR_BAD To_Server( struct server_connection_state * scs, struct serve
 			Close_Persistent( scs);
 			scs->file_descriptor = ClientConnect(in);
 			if ( FILE_DESCRIPTOR_VALID( scs->file_descriptor ) ) {
-				SOC(in)->file_descriptor = FILE_DESCRIPTOR_PERSISTENT_IN_USE ;
+				in->pown->file_descriptor = FILE_DESCRIPTOR_PERSISTENT_IN_USE ;
 			}
 			break ;
 		default:
@@ -645,7 +646,7 @@ static void Close_Persistent( struct server_connection_state * scs)
 	if (scs->persistence == persistent_yes) {
 		// no persistence wanted
 		BUSLOCKIN(scs->in);
-			SOC(scs->in)->file_descriptor = FILE_DESCRIPTOR_BAD ;
+			scs->in->pown->file_descriptor = FILE_DESCRIPTOR_BAD ;
 		BUSUNLOCKIN(scs->in);
 	}
 	
@@ -658,24 +659,38 @@ static SIZE_OR_ERROR WriteToServer(int file_descriptor, struct server_msg *sm, s
 {
 	int payload = 0;
 	int tokens = 0;
+	int server_type = Globals.program_type==program_type_server || Globals.program_type==program_type_external ;
+
+	// We use vector write -- several ranges
 	int nio = 0;
-	int traffic_counter ;
 	struct iovec io[5] = { {NULL, 0}, {NULL, 0}, {NULL, 0}, {NULL, 0}, {NULL, 0}, };
+
 	struct server_msg net_sm ;
 
+	// Set the version
+	sm->version = MakeServerprotocol(OWSERVER_PROTOCOL_VERSION);
+
 	// First block to send, the header
-	io[nio].iov_base = &net_sm;
-	io[nio].iov_len = sizeof(struct server_msg);
+	// We'll do this last since the header values (e.g. payload) change
 	nio++;
 
 	// Next block, the path
 	if (sp->path != 0) {	// send path (if not null)
-		io[nio].iov_base = sp->path ; // gives a spurious compiler error -- constant is OK HERE!
-		io[nio].iov_len = strlen(sp->path) + 1;
-		payload =  io[nio].iov_len ;
+		// writev should take const data pointers, but I can't fix the library
+#if ( __GNUC__ > 4 ) || (__GNUC__ == 4 && __GNUC_MINOR__ > 4 )
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+		io[nio].iov_base = (char *) sp->path ; // gives a spurious compiler error -- constant is OK HERE!
+#pragma GCC diagnostic pop
+#else
+		io[nio].iov_base = (char *) sp->path ; // gives a spurious compiler error -- constant is OK HERE!
+#endif
+		io[nio].iov_len = strlen(sp->path) + 1; // we're adding the null (though it isn't required post 2.9p3)
+		payload =  io[nio].iov_len; 
 		nio++;
 	}
-	// next block, data (only for writes)
+
+	// Next block, data (only for writes)
 	if ((sp->datasize>0) && (sp->data!=NULL)) {	// send data only for writes (if datasize not zero)
 		io[nio].iov_base = sp->data ;
 		io[nio].iov_len = sp->datasize ;
@@ -683,25 +698,35 @@ static SIZE_OR_ERROR WriteToServer(int file_descriptor, struct server_msg *sm, s
 		nio++;
 	}
 
-	sm->version = MakeServerprotocol(OWSERVER_PROTOCOL_VERSION);
-	if (Globals.program_type == program_type_server) {
+	if ( server_type ) {
 		tokens = sp->tokens;
-		sm->version |= MakeServermessage;
 
-		// next block prior tokens (if an owserver)
+		// Next block prior tokens (if an owserver)
 		if (tokens > 0) {		// owserver: send prior tags
 			io[nio].iov_base = sp->tokenstring;
-			io[nio].iov_len = tokens * sizeof(union antiloop);
+			io[nio].iov_len = tokens * sizeof(struct antiloop);
 			nio++;
 		}
-		// final block, new token (if an owserver)
+		// Final block, new token (if an owserver)
+		
 		++tokens;
-		sm->version |= MakeServertokens(tokens);
+		
+		if ( tokens == MakeServermessage ) {
+			LEVEL_DEBUG( "Too long a list of tokens -- %d owservers in the chain",tokens);
+			return -ELOOP ;
+		}
+		
 		io[nio].iov_base = &(Globals.Token);	// owserver: add our tag
-		io[nio].iov_len = sizeof(union antiloop);
+		io[nio].iov_len = sizeof(struct antiloop);
+
+		// put token information in header (lower 17 bits of version)
+		sm->version |= MakeServermessage; // bit 17
+		sm->version |= MakeServertokens(tokens); // lower 16 bits
 		nio++;
 	}
-	LEVEL_DEBUG("version=%u payload=%d size=%d type=%d SG=%X offset=%d",sm->version,payload,sm->size,sm->type,sm->control_flags,sm->offset);
+	
+	// First block to send, the header
+	// revisit now that the header values are set
 
 	// encode in network order (just the header)
 	net_sm.version       = htonl( sm->version       );
@@ -711,24 +736,37 @@ static SIZE_OR_ERROR WriteToServer(int file_descriptor, struct server_msg *sm, s
 	net_sm.control_flags = htonl( sm->control_flags );
 	net_sm.offset        = htonl( sm->offset        );
 
-	traffic_counter = 0 ;
-	TrafficOutFD("write header" ,io[traffic_counter].iov_base,io[traffic_counter].iov_len,file_descriptor);
-	++traffic_counter;
-	TrafficOutFD("write path"  ,io[traffic_counter].iov_base,io[traffic_counter].iov_len,file_descriptor);
-	if ((sp->datasize>0) && (sp->data!=NULL)) {	// send data only for writes (if datasize not zero)
-		++traffic_counter;
-		TrafficOutFD("write data" ,io[traffic_counter].iov_base,io[traffic_counter].iov_len,file_descriptor);
-	}
-	if ( Globals.program_type == program_type_server ) {
-		if (sp->tokens > 0) {		// owserver: send prior tags
-			++traffic_counter;
-			TrafficOutFD("write old tokens" ,io[traffic_counter].iov_base,io[traffic_counter].iov_len,file_descriptor);
-		}
-		++traffic_counter;
-		TrafficOutFD("write new tokens" ,io[traffic_counter].iov_base,io[traffic_counter].iov_len,file_descriptor);
-	}
+	// set the header into the first (index=0) block
+	io[0].iov_base = &net_sm;
+	io[0].iov_len = sizeof(struct server_msg);
 
-	return writev(file_descriptor, io, nio) != (ssize_t) (payload + sizeof(struct server_msg) + tokens * sizeof(union antiloop));
+	// debug data on packet
+	LEVEL_DEBUG("version=%u payload=%d size=%d type=%d SG=%X offset=%d",sm->version,payload,sm->size,sm->type,sm->control_flags,sm->offset);
+
+	// Here is some traffic display code
+	{
+		int traffic_counter ;
+		traffic_counter = 0 ;
+		TrafficOutFD("write header" ,io[traffic_counter].iov_base,io[traffic_counter].iov_len,file_descriptor);
+		++traffic_counter;
+		TrafficOutFD("write path"  ,io[traffic_counter].iov_base,io[traffic_counter].iov_len,file_descriptor);
+		if ((sp->datasize>0) && (sp->data!=NULL)) {	// send data only for writes (if datasize not zero)
+			++traffic_counter;
+			TrafficOutFD("write data" ,io[traffic_counter].iov_base,io[traffic_counter].iov_len,file_descriptor);
+		}
+		if ( server_type ) {
+			if (sp->tokens > 0) {		// owserver: send prior tags
+				++traffic_counter;
+				TrafficOutFD("write old tokens" ,io[traffic_counter].iov_base,io[traffic_counter].iov_len,file_descriptor);
+			}
+			++traffic_counter;
+			TrafficOutFD("write new tokens" ,io[traffic_counter].iov_base,io[traffic_counter].iov_len,file_descriptor);
+		}
+	}
+	// End traffic display code
+
+	// Actual write of data to owserver
+	return writev(file_descriptor, io, nio) != (ssize_t) (payload + sizeof(struct server_msg) + tokens * sizeof(struct antiloop));
 }
 
 /* flag the sg for "virtual root" -- the remote bus was specifically requested */
@@ -776,7 +814,7 @@ static void Release_Persistent( struct server_connection_state * scs, int grante
 
 	// mark as available
 	BUSLOCKIN(scs->in);
-	SOC(scs->in)->file_descriptor = scs->file_descriptor;
+	scs->in->pown->file_descriptor = scs->file_descriptor;
 	BUSUNLOCKIN(scs->in);
 	scs->persistence = persistent_no ; // we no longer own this connection
 	scs->file_descriptor = FILE_DESCRIPTOR_BAD ;
