@@ -1,9 +1,8 @@
 /*
-$Id: ow_simultaneous.c,v 1.89 2012/04/07 17:45:51 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -55,10 +54,14 @@ WRITE_FUNCTION(FS_w_convert_volt);
 READ_FUNCTION(FS_r_present);
 READ_FUNCTION(FS_r_single);
 
+/* Internal properties */
+Make_SlaveSpecificTag_exportable(S_T, fc_volatile);	// simultaneous temperature
+Make_SlaveSpecificTag_exportable(S_V, fc_volatile);	// simultaneous voltage
+
 /* -------- Structures ---------- */
 static struct filetype simultaneous[] = {
-	{"temperature", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_link, FS_r_convert, FS_w_convert_temp, VISIBLE, {i:simul_temp}, },
-	{"voltage", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_link, FS_r_convert, FS_w_convert_volt, VISIBLE, {i:simul_volt}, },
+	{"temperature", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_link, FS_r_convert, FS_w_convert_temp, VISIBLE, {v:SlaveSpecificTag(S_T)}, },
+	{"voltage", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_link, FS_r_convert, FS_w_convert_volt, VISIBLE, {v:SlaveSpecificTag(S_V)}, },
 	{"present", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_volatile, FS_r_present, NO_WRITE_FUNCTION, VISIBLE, {i:_1W_READ_ROM}, },
 	{"present_ds2400", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_volatile, FS_r_present, NO_WRITE_FUNCTION, VISIBLE, {i:_1W_OLD_READ_ROM}, },
 	{"single", 18, NON_AGGREGATE, ft_ascii, fc_volatile, FS_r_single, NO_WRITE_FUNCTION, VISIBLE, {i:_1W_READ_ROM}, },
@@ -67,25 +70,19 @@ static struct filetype simultaneous[] = {
 
 DeviceEntry(simultaneous, simultaneous, NO_GENERIC_READ, NO_GENERIC_WRITE);
 
-// in this order: enum sumul_type { simul_temp, simul_volt, } ;
-struct internal_prop ipSimul[] = {
-	{"temperature", fc_volatile},
-	{"voltage", fc_volatile},
-};
-
 #define _1W_CONVERT_T             0x44
 #define _1W_READ_POWERMODE        0xB4
 
 /* ------- Functions ------------ */
-static void OW_single2cache(BYTE * sn, const struct parsedname *pn2);
+//static void OW_single2cache(BYTE * sn, const struct parsedname *pn2);
 
-GOOD_OR_BAD FS_Test_Simultaneous( enum simul_type type, UINT delay, const struct parsedname * pn)
+GOOD_OR_BAD FS_Test_Simultaneous( const struct internal_prop *ip, UINT delay, const struct parsedname * pn)
 {
 	time_t dwell_time ;
 	time_t remaining_delay ;
 
 	//LEVEL_DEBUG("TEST Simultaneous valid?");
-	if( BAD( Cache_Get_Simul_Time(type, &dwell_time, pn)) ) {
+	if( BAD( Cache_Get_Simul_Time( ip, &dwell_time, pn)) ) {
 		LEVEL_DEBUG("No simultaneous conversion currently valid");
 		return gbBAD ; // No simultaneous valid
 	}
@@ -152,10 +149,10 @@ static ZERO_OR_ERROR FS_w_convert_temp(struct one_wire_query *owq)
 	FS_LoadDirectoryOnly(pn_directory, pn); // setup up for full directory message
 
 	// Get Power status
+	LEVEL_DEBUG("TEST if bus powered");
 	RETURN_BAD_IF_BAD(BUS_transaction(tpower, pn_directory)) ;
-	//LEVEL_DEBUG("TEST read power");
 	
-	Cache_Add_Simul(simul_temp, pn_directory);	// Mark start time
+	Cache_Add_Simul(pn->selected_filetype->data.v, pn_directory);	// Mark start time
 	if ( pow[0] != 0 ) {
 		// powered
 		// Send the conversion and let the timing work out when the actual
@@ -163,28 +160,15 @@ static ZERO_OR_ERROR FS_w_convert_temp(struct one_wire_query *owq)
 		if ( GOOD(BUS_transaction(t_powered_convert, pn_directory) ) ) {
 			return 0 ;
 		}
-	} else if ( in->Adapter == adapter_DS2482_800 ) {
-		// special case for the 8-channel DS2482-800
-		// Rather than block the whole chip (all 8 channels) by polling
-		GOOD_OR_BAD ret ;
-		_MUTEX_LOCK(in->bus_mutex); // channel
-		_MUTEX_LOCK(in->master.i2c.head->master.i2c.all_channel_lock); // master
-		ret = BUS_transaction_nolock(t_powered_convert, pn_directory) ;
-		_MUTEX_UNLOCK(in->master.i2c.head->master.i2c.all_channel_lock); //master
-		if ( GOOD(ret) ) {
-			UT_delay(1000) ; // wait for passive
-		}
-		_MUTEX_UNLOCK(in->bus_mutex); //channel
-		if ( GOOD(ret) ) {
-			return 0 ;
-		}
 	} else {
+		// Unpowered prohibits other bus traffic.
+		// This is at the port level, so could be all channels of the DS2482-800, etc
 		if ( GOOD(BUS_transaction(t_unpowered_convert, pn_directory) )) {
 			return 0 ;
 		}
 	}
 
-	Cache_Del_Simul(simul_temp, pn_directory);	// Clear start time
+	Cache_Del_Simul(pn->selected_filetype->data.v, pn_directory);	// Clear start time
 	LEVEL_DEBUG("Trouble setting simultaneous for %s",pn_directory->path);
 	return -EINVAL ;
 }
@@ -193,29 +177,43 @@ static ZERO_OR_ERROR FS_w_convert_volt(struct one_wire_query *owq)
 {
 	struct parsedname *pn = PN(owq);
 	struct parsedname pn_directory;
+	struct connection_in * in = pn->selected_connection ;
+
+	BYTE cmd_volt[] = { _1W_SKIP_ROM, 0x3C, 0x0F, 0x00, 0xFF, 0xFF };
+	struct transaction_log t[] = {
+		TRXN_START,
+		TRXN_WR_CRC16(cmd_volt, 4, 0),
+		TRXN_DELAY(5),
+		TRXN_END,
+	};
 	
 	if (OWQ_Y(owq) == 0) {
 		return 0;				// don't send convert
 	}
 	
 	FS_LoadDirectoryOnly(&pn_directory, pn);
-	Cache_Del_Internal(&ipSimul[simul_volt], &pn_directory);	// remove existing entry
+	Cache_Del_Internal(pn->selected_filetype->data.v, &pn_directory);	// remove existing entry
 	
-	/* Since writing to /simultaneous/temperature is done recursive to all
-	* adapters, we have to fake a successful write even if it's detected
-	* as a bad adapter. */
-	if (pn->selected_connection->Adapter != adapter_Bad) {
-		BYTE cmd_volt[] = { _1W_SKIP_ROM, 0x3C, 0x0F, 0x00, 0xFF, 0xFF };
-		struct transaction_log t[] = {
-			TRXN_START,
-			TRXN_WR_CRC16(cmd_volt, 4, 0),
-			TRXN_DELAY(5),
-			TRXN_END,
-		};
-		if ( GOOD(BUS_transaction(t, &pn_directory)) ) {
-			Cache_Add_SlaveSpecific(NULL, 0, &ipSimul[simul_volt], &pn_directory);
-		}
+	switch (in->Adapter) {
+		case adapter_Bad:
+		case adapter_w1_monitor:
+		case adapter_browse_monitor:
+		case adapter_usb_monitor:
+		case adapter_fake:
+		case adapter_tester:
+		case adapter_mock:
+			/* Since writing to /simultaneous/voltage is done recursive to all
+			* adapters, we have to fake a successful write even if it's detected
+			* as an unsupported adapter. */
+			return gbGOOD ;
+		default:
+			break ;
 	}
+
+	if ( GOOD(BUS_transaction(t, &pn_directory)) ) {
+		Cache_Add_SlaveSpecific(NULL, 0, pn->selected_filetype->data.v, &pn_directory);
+	}
+
 	return 0;
 }
 
@@ -225,7 +223,7 @@ static ZERO_OR_ERROR FS_r_convert(struct one_wire_query *owq)
 	struct parsedname pn_directory;
 
 	FS_LoadDirectoryOnly(&pn_directory, pn);
-	OWQ_Y(owq) = GOOD (Cache_Get_SlaveSpecific(NULL, 0, &ipSimul[pn->selected_filetype->data.i], &pn_directory) );
+	OWQ_Y(owq) = GOOD (Cache_Get_SlaveSpecific(NULL, 0, pn->selected_filetype->data.v, &pn_directory) );
 	return 0;
 }
 
@@ -235,9 +233,11 @@ static ZERO_OR_ERROR FS_r_present(struct one_wire_query *owq)
 
 	switch (pn->selected_connection->Adapter) {
 		case adapter_fake:
+		case adapter_mock:
 		case adapter_tester:
-			// fake adapter -- simple memory look
+			// fake adapter -- simple memory look-up
 			OWQ_Y(owq) = (DirblobElements(&(pn->selected_connection->master.fake.main)) > 0);
+			/* fall through */
 		default:
 		{
 			struct parsedname pn_directory;
@@ -264,7 +264,7 @@ static ZERO_OR_ERROR FS_r_present(struct one_wire_query *owq)
 				if (CRC8(resp, SERIAL_NUMBER_SIZE)) {
 					return 0;		// crc8 error -- more than one device
 				}
-				OW_single2cache(resp, &pn_directory);
+//				OW_single2cache(resp, &pn_directory);
 			} else {				// no devices
 				OWQ_Y(owq) = 0;
 			}
@@ -309,7 +309,7 @@ static ZERO_OR_ERROR FS_r_single(struct one_wire_query *owq)
 			RETURN_ERROR_IF_BAD(BUS_transaction(t, &pn_directory)) ;
 			LEVEL_DEBUG("dat=" SNformat " crc8=%02x", SNvar(resp), CRC8(resp, 7));
 			if ((memcmp(resp, collisions, SERIAL_NUMBER_SIZE) != 0) && (memcmp(resp, match, SERIAL_NUMBER_SIZE) != 0) && (CRC8(resp, SERIAL_NUMBER_SIZE) == 0)) {	// non-empty, and no CRC error
-				OW_single2cache(resp, &pn_directory);
+//				OW_single2cache(resp, &pn_directory);
 				/* Return device id. */
 				FS_devicename(ad, sizeof(ad), resp, pn);
 			} else {
@@ -327,15 +327,3 @@ static ZERO_OR_ERROR FS_r_single(struct one_wire_query *owq)
 	}
 	return OWQ_format_output_offset_and_size_z(ad, owq);
 }
-
-// Do cache for single item
-static void OW_single2cache(BYTE * sn, const struct parsedname *pn)
-{
-	struct dirblob db;
-	DirblobInit(&db);
-	DirblobAdd(sn, &db);
-	if (DirblobPure(&db)) {
-		Cache_Add_Dir(&db, pn);	// Directory cache
-	}
-	Cache_Add_Device(pn->selected_connection->index, sn);	// Device cache
-}
