@@ -1,9 +1,9 @@
 /*
-$Id: ow_system.c,v 1.68 2010/05/27 02:20:52 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -55,38 +55,39 @@ READ_FUNCTION(FS_pid);
 READ_FUNCTION(FS_in);
 READ_FUNCTION(FS_out);
 READ_FUNCTION(FS_define);
+READ_FUNCTION(FS_trim);
+READ_FUNCTION(FS_version);
+
+#define VERSION_LENGTH 20
 
 /* -------- Structures ---------- */
 /* special entry -- picked off by parsing before filetypes tried */
-struct filetype sys_process[] = {
+static struct filetype sys_process[] = {
 	{"pidfile", 128, NON_AGGREGATE, ft_vascii, fc_static, FS_pidfile, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA,},
 	// variable length
-	{"pid", PROPERTY_LENGTH_UNSIGNED, NON_AGGREGATE, ft_unsigned, fc_static, FS_pid, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA,},
+	{"pid", PROPERTY_LENGTH_UNSIGNED, NON_AGGREGATE, ft_unsigned, fc_static, FS_pid, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA, },
 };
 struct device d_sys_process = { "process", "process", ePN_system, COUNT_OF_FILETYPES(sys_process),
 	sys_process, NO_GENERIC_READ, NO_GENERIC_WRITE
 };
 
-struct filetype sys_connections[] = {
-	{"count_current_buses", PROPERTY_LENGTH_UNSIGNED, NON_AGGREGATE, ft_unsigned, fc_static, FS_in, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA,},
-	{"count_outbound_connections", PROPERTY_LENGTH_UNSIGNED, NON_AGGREGATE, ft_unsigned, fc_static, FS_out, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA,},
+static struct filetype sys_connections[] = {
+	{"count_current_buses", PROPERTY_LENGTH_UNSIGNED, NON_AGGREGATE, ft_unsigned, fc_static, FS_in, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA, },
+	{"count_outbound_connections", PROPERTY_LENGTH_UNSIGNED, NON_AGGREGATE, ft_unsigned, fc_static, FS_out, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA, },
 };
 struct device d_sys_connections = { "connections", "connections", ePN_system,
 	COUNT_OF_FILETYPES(sys_connections),
 	sys_connections, NO_GENERIC_READ, NO_GENERIC_WRITE
 };
 
-struct filetype sys_configure[] = {
-	{"threaded", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_MT},},
-	{"tai8570", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_TAI8570},},
-	{"thermocouples", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_THERMOCOUPLE},},
-	{"parport", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_PARPORT},},
-	{"USB", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_USB},},
-	{"i2c", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_I2C},},
-	{"cache", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_CACHE},},
-	{"HA7Net", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_HA7},},
-	{"DebugInfo", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_DEBUG},},
-	{"zeroconf", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:1},},
+static struct filetype sys_configure[] = {
+	{"parport", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_PARPORT}, },
+	{"USB", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_USB}, },
+	{"i2c", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_I2C}, },
+	{"DebugInfo", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:OW_DEBUG}, },
+	{"zeroconf", PROPERTY_LENGTH_INTEGER, NON_AGGREGATE, ft_integer, fc_static, FS_define, NO_WRITE_FUNCTION, VISIBLE, {i:1}, },
+	{"trim", PROPERTY_LENGTH_YESNO, NON_AGGREGATE, ft_yesno, fc_static, FS_trim, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA, },
+	{"version", VERSION_LENGTH, NON_AGGREGATE, ft_ascii, fc_static, FS_version, NO_WRITE_FUNCTION, VISIBLE, NO_FILETYPE_DATA, },
 };
 struct device d_sys_configure = { "configuration", "configuration", ePN_system,
 	COUNT_OF_FILETYPES(sys_configure),
@@ -110,6 +111,20 @@ static ZERO_OR_ERROR FS_pid(struct one_wire_query *owq)
 	return 0;
 }
 
+static ZERO_OR_ERROR FS_version( struct one_wire_query *owq)
+{
+	char ver[VERSION_LENGTH+1] ;
+	memset(ver,0,VERSION_LENGTH+1);
+	strncpy(ver,VERSION,VERSION_LENGTH) ;
+	return OWQ_format_output_offset_and_size_z(ver, owq);
+}
+
+static ZERO_OR_ERROR FS_trim(struct one_wire_query *owq)
+{
+	OWQ_Y(owq) = ShouldTrim(PN(owq));
+	return 0;
+}
+
 static ZERO_OR_ERROR FS_in(struct one_wire_query *owq)
 {
 	OWQ_U(owq) = Inbound_Control.active;
