@@ -1,5 +1,5 @@
 /*
-$Id: ow_w1_browse.c,v 1.32 2010/09/24 02:10:31 alfille Exp $
+$Id: ow_w1_browse.c,v 1.33 2013/01/01 16:03:36 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -18,7 +18,7 @@ $Id: ow_w1_browse.c,v 1.32 2010/09/24 02:10:31 alfille Exp $
 #include "owfs_config.h"
 #include "ow_connection.h"
 
-#if OW_W1 && OW_MT
+#if OW_W1
 
 #include "ow_w1.h"
 
@@ -34,4 +34,4 @@ GOOD_OR_BAD W1_Browse( void )
 	return gbGOOD ;
 }
 
-#endif /* OW_W1 && OW_MT */
+#endif /* OW_W1 */
