@@ -1,5 +1,4 @@
 /*
-$Id: ow_w1_parse.c,v 1.47 2011/12/17 22:00:42 alfille Exp $
 W1 Announce -- daemon  for showing w1 busmasters using Avahi
 Written 2008 Paul H Alfille
 email: paul.alfille@gmail.com
@@ -24,13 +23,13 @@ This file itself  is a modestly modified version of w1d by Evgeniy Polyakov
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <config.h>
 #include "owfs_config.h"
 
-#if OW_W1 && OW_MT
+#if OW_W1
 
 #include "ow_w1.h"
 #include "ow_connection.h"
@@ -111,7 +110,7 @@ GOOD_OR_BAD Netlink_Parse_Get( struct netlink_parse * nlp )
 
 	// first peek at message to get length and details
 	LEVEL_DEBUG("Wait to peek at message");
-	int recv_len = recv( SOC(Inbound_Control.w1_monitor)->file_descriptor, &peek_nlm, W1_NLM_LENGTH, MSG_PEEK );
+	int recv_len = recv( Inbound_Control.w1_monitor->pown->file_descriptor, &peek_nlm, W1_NLM_LENGTH, MSG_PEEK );
 
 	// Set time of last read
 	_MUTEX_LOCK(Inbound_Control.w1_monitor->master.w1_monitor.read_mutex) ;
@@ -132,7 +131,7 @@ GOOD_OR_BAD Netlink_Parse_Get( struct netlink_parse * nlp )
 	}
 
 	// read whole packet
-	recv_len = recv( SOC(Inbound_Control.w1_monitor)->file_descriptor, &(nlp->nlm[0]), peek_nlm.nlmsg_len, 0 );
+	recv_len = recv( Inbound_Control.w1_monitor->pown->file_descriptor, &(nlp->nlm[0]), peek_nlm.nlmsg_len, 0 );
 	if (recv_len == -1) {
 		ERROR_DEBUG("Netlink (w1) recv body error");
 		return gbBAD ;
@@ -172,6 +171,11 @@ static GOOD_OR_BAD Get_and_Parse_Pipe( FILE_DESCRIPTOR_OR_ERROR file_descriptor,
 
 	memcpy( nlm, &peek_nlm, W1_NLM_LENGTH ) ; // copy header
 	// read rest of packet
+	if ( peek_nlm.nlmsg_len <= W1_NLM_LENGTH ) {
+		owfree(nlm) ;
+		LEVEL_DEBUG( "W1 packet error. Length is too short." ) ;
+		return gbBAD ;
+	}
 	payload_length = peek_nlm.nlmsg_len - W1_NLM_LENGTH ;
 	if ( read( file_descriptor, NLMSG_DATA(nlm), payload_length ) != (ssize_t) payload_length ) {
 		owfree(nlm) ;
@@ -259,4 +263,4 @@ enum Netlink_Read_Status W1_Process_Response( void (* nrs_callback)( struct netl
 	return nrs_timeout ;
 }
 
-#endif /* OW_W1 && OW_MT */
+#endif /* OW_W1 */
