@@ -1,5 +1,5 @@
 /*
-$Id: ow_w1_scan.c,v 1.22 2011/12/19 00:07:04 alfille Exp $
+$Id: ow_w1_scan.c,v 1.23 2013/01/01 16:03:37 alfille Exp $
 W1 Announce -- daemon  for showing w1 busmasters using Avahi
 Written 2008 Paul H Alfille
 email: paul.alfille@gmail.com
@@ -32,7 +32,7 @@ This file itself  is amodestly modified version of w1d by Evgeniy Polyakov
 #include <config.h>
 #include "owfs_config.h"
 
-#if OW_W1 && OW_MT 
+#if OW_W1
 
 #include "ow_w1.h"
 #include "ow_connection.h"
@@ -79,4 +79,4 @@ void * w1_master_command(void * v)
 	return VOID_RETURN ;
 }
 
-#endif /* OW_W1 && OW_MT */
+#endif /* OW_W1 */
