@@ -1,5 +1,5 @@
 /*
-$Id: ow_w1_send.c,v 1.33 2011/12/14 02:40:25 alfille Exp $
+$Id: ow_w1_send.c,v 1.35 2012/09/28 20:43:34 alfille Exp $
 W1 Announce -- daemon  for showing w1 busmasters using Avahi
 Written 2008 Paul H Alfille
 email: paul.alfille@gmail.com
@@ -132,7 +132,7 @@ SEQ_OR_ERROR W1_send_msg( struct connection_in * in, struct w1_netlink_msg *msg,
 	LEVEL_DEBUG("Netlink send -----------------");
 	Netlink_Print( nlm, cn, w1m, w1c, pdata, data_size ) ;
 	
-	if ( send( SOC(Inbound_Control.w1_monitor)->file_descriptor, nlm, NLMSG_SPACE(nlm_payload),  0) == -1 ) {
+	if ( send( Inbound_Control.w1_monitor->pown->file_descriptor, nlm, NLMSG_SPACE(nlm_payload),  0) == -1 ) {
 		//err = COM_write( nlm, nlm_size, Inbound_Control.w1.monitor ) ;
 		owfree(nlm);
 		ERROR_CONNECT("Failed to send w1 netlink message");
