@@ -1,9 +1,8 @@
 /*
-$Id: ow_write.c,v 1.173 2011/12/22 03:08:32 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -269,29 +268,64 @@ static ZERO_OR_ERROR FS_write_real(int depth, struct one_wire_query *owq)
 	return FS_write_real(depth+1,owq);
 }
 
-#if OW_MT
 struct simultaneous_struct {
-	struct connection_in *in;
-	const struct one_wire_query *owq ;
+	struct port_in * pin ;
+	struct connection_in * cin;
+	struct one_wire_query owq ;
 };
 
-static void * Simultaneous_write(void * v)
+static void * Simultaneous_write_callback_conn(void * v)
 {
 	struct simultaneous_struct *ss = (struct simultaneous_struct *) v;
-	struct simultaneous_struct ss_next = { ss->in->next, ss->owq };
+	struct simultaneous_struct ss_next ;
 	pthread_t thread;
-	int threadbad = 1;
-	OWQ_allocate_struct_and_pointer(owq_copy);
+	int threadbad = 0;
 	
+	if ( ss->cin == NULL ) {
+		return VOID_RETURN;
+	}
 	
-	threadbad = (ss->in->next == NULL)
-	|| pthread_create(&thread, DEFAULT_THREAD_ATTR, Simultaneous_write, (void *) (&ss_next));
+	ss_next.cin = ss->cin->next ;
+	if ( ss_next.cin == NO_CONNECTION ) {
+		threadbad = 1 ;
+	} else {
+		ss_next.pin = ss->pin ;
+		memcpy( &(ss_next.owq), &(ss->owq), sizeof(struct one_wire_query));	// shallow copy
+		threadbad = pthread_create(&thread, DEFAULT_THREAD_ATTR, Simultaneous_write_callback_conn, (void *) (&ss_next)) ;
+	}
 	
-	memcpy(owq_copy, ss->owq, sizeof(struct one_wire_query));	// shallow copy
+	SetKnownBus(ss->cin->index, PN( &(ss->owq)) );
 	
-	SetKnownBus(ss->in->index, PN(owq_copy));
+	FS_w_given_bus( &(ss->owq) );
+
+	if (threadbad == 0) {		/* was a thread created? */
+		pthread_join(thread, NULL) ;
+	}
+	return VOID_RETURN ;
+}
+
+static void * Simultaneous_write_callback_port(void * v)
+{
+	struct simultaneous_struct *ss = (struct simultaneous_struct *) v;
+	struct simultaneous_struct ss_next ;
+	pthread_t thread;
+	int threadbad = 0;
+
+	if ( ss->pin == NULL ) {
+		return VOID_RETURN;
+	}
 	
-	FS_w_given_bus(owq_copy);
+	ss_next.pin = ss->pin->next ;
+	if ( ss_next.pin == NULL ) {
+		threadbad = 1 ;
+	} else {
+		memcpy( &(ss_next.owq), &(ss->owq), sizeof(struct one_wire_query));	// shallow copy
+		threadbad = pthread_create(&thread, DEFAULT_THREAD_ATTR, Simultaneous_write_callback_port, (void *) (&ss_next)) ;
+	}
+	
+	ss->cin = ss->pin->first ;
+	
+	Simultaneous_write_callback_conn(v) ;
 
 	if (threadbad == 0) {		/* was a thread created? */
 		pthread_join(thread, NULL) ;
@@ -304,36 +338,15 @@ static ZERO_OR_ERROR FS_w_simultaneous(struct one_wire_query *owq)
 {
 	if (SpecifiedBus(PN(owq))) {
 		return FS_w_given_bus(owq);
-	} else if (Inbound_Control.head) {
-		struct simultaneous_struct ss = { Inbound_Control.head, owq };
-		Simultaneous_write( (void *) (&ss) ) ;
+	} else {
+		struct simultaneous_struct ss ;
+		ss.pin = Inbound_Control.head_port ; 
+		memcpy( &(ss.owq), owq, sizeof(struct one_wire_query));	// shallow copy
+		Simultaneous_write_callback_port( (void *) (&ss) ) ;
 	}
 	return 0;
 }
 
-#else /* not OW_MT */
-
-/* This function is only used by "Simultaneous" */
-static ZERO_OR_ERROR FS_w_simultaneous(struct one_wire_query *owq)
-{
-	if (SpecifiedBus(PN(owq))) {
-		return FS_w_given_bus(owq);
-	} else if (Inbound_Control.head) {
-		struct connection_in * in;
-		OWQ_allocate_struct_and_pointer(owq_given);
-		
-		memcpy(owq_given, owq, sizeof(struct one_wire_query));	// shallow copy
-		
-		for( in=Inbound_Control.head; in ; in=in->next ) {
-			SetKnownBus(in->index, PN(owq_given));
-			FS_w_given_bus(owq_given);
-		}
-	}
-	return 0;
-}
-
-#endif /* OW_MT */
-
 /* Write to interface dir */
 static ZERO_OR_ERROR FS_w_interface(struct one_wire_query *owq)
 {
@@ -359,7 +372,7 @@ static ZERO_OR_ERROR FS_w_settings(struct one_wire_query *owq)
 		return -ECONNABORTED;
 	} else if (KnownBus(pn) && BusIsServer(pn->selected_connection)) {
 		return ServerWrite(owq);
-	} else if ( pn->selected_connection == NO_CONNECTION ) {
+	} else if ( pn->selected_connection != NO_CONNECTION ) {
 		LEVEL_DEBUG("Attempt to write to local bus for /settings");
 		return -ENODEV ;
 	} else {
@@ -429,7 +442,7 @@ static ZERO_OR_ERROR FS_w_local(struct one_wire_query *owq)
 	switch ( ft->ag->combined ) {
 		case ag_sparse:
 			// avoid cache
-			return (ft->write) (owq);;
+			return (ft->write) (owq);
 		case ag_aggregate:
 			switch (pn->extension) {
 				case EXTENSION_BYTE:
@@ -535,7 +548,7 @@ static ZERO_OR_ERROR FS_write_a_part( struct one_wire_query *owq_part )
 			buffer_pointer = buffer_pointer + OWQ_array_length(owq_all, extension); // current start next element
 
 			// move rest of elements to new locations
-			for (extension_index = extension + 1; extension_index < elements; ++extension) {
+			for (extension_index = extension + 1; extension_index < elements; ++extension_index ) {
 				size_t this_length = OWQ_array_length(owq_all, extension_index);
 				memmove(target_pointer, buffer_pointer, this_length);
 				target_pointer += this_length;
@@ -640,7 +653,7 @@ static ZERO_OR_ERROR FS_write_as_bits( struct one_wire_query *owq_byte )
 	for ( extension = 0 ; extension < elements ; ++extension ) {
 		ZERO_OR_ERROR z ;
 		OWQ_pn(owq_bit).extension = extension ;
-		OWQ_Y(owq_bit) = UT_getbit( (BYTE *) &OWQ_U(owq_byte), extension ) ;
+		OWQ_Y(owq_bit) = UT_getbit_U( OWQ_U(owq_byte), extension ) ;
 		z = FS_write_owq( owq_bit ) ;
 		if ( z != 0 ) {
 			z_or_e = z ;
@@ -674,7 +687,7 @@ static ZERO_OR_ERROR FS_write_a_bit(struct one_wire_query *owq_bit)
 	
 	if ( owq_byte != NO_ONE_WIRE_QUERY ) {
 		if ( FS_read_local( owq_byte ) >= 0 ) {
-			UT_setbit( (BYTE *) &OWQ_U( owq_byte ), OWQ_pn(owq_bit).extension, OWQ_Y(owq_bit) ) ;
+			UT_setbit_U( &OWQ_U( owq_byte ), OWQ_pn(owq_bit).extension, OWQ_Y(owq_bit) ) ;
 			z_or_e = FS_write_owq( owq_byte ) ;
 		}
 		OWQ_destroy( owq_byte ) ;
