@@ -1,5 +1,5 @@
 /*
-$Id: ow.h,v 1.321 2011/05/11 01:53:30 alfille Exp $
+$Id$
     OW -- One-Wire filesystem
     version 0.4 7/2/2003
 
@@ -79,6 +79,11 @@ $Id: ow.h,v 1.321 2011/05/11 01:53:30 alfille Exp $
 
 #define _FILE_OFFSET_BITS   64
 
+#ifdef __FreeBSD__
+/* from Johan Ström: needed for sys/param.h if sys/types.h sees it */
+#define __BSD_VISIBLE 1 
+#endif /* __FreeBSD__ */
+
 #ifdef HAVE_FEATURES_H
 #include <features.h>
 #endif							/* HAVE_FEATURES_H */
@@ -89,7 +94,7 @@ $Id: ow.h,v 1.321 2011/05/11 01:53:30 alfille Exp $
 
 #ifdef HAVE_SYS_TYPES_H
 #ifdef __FreeBSD__
-#define __BSD_VISIBLE 1
+#include <sys/param.h>
 #endif /* __FreeBSD__ */
 #include <sys/types.h>			/* for stat */
 #endif							/* HAVE_SYS_TYPES_H */
@@ -197,6 +202,11 @@ $Id: ow.h,v 1.321 2011/05/11 01:53:30 alfille Exp $
 ssize_t getline (char **lineptr, size_t *n, FILE *stream) ;
 #endif /* HAVE_GETLINE */
 
+/* If no timegm, use our version */
+#if (!defined _BSD_SOURCE && !defined _SVID_SOURCE)
+#include <time.h>
+time_t timegm(struct tm *tm);
+#endif
 
 /* Parport enabled uses two flags (one a holdover from the embedded work) */
 #ifdef USE_NO_PARPORT
@@ -327,6 +337,9 @@ enum deviceformat { fdi, fi, fdidc, fdic, fidc, fic };
 /* Separated out into ow_global.h for readability */
 #include "ow_global.h"
 
+/* Allow detail debugging of individual slave */
+#include "ow_detail.h"
+
 /* State information for the program */
 /* Separated out into ow_stateinfo.h for readability */
 #include "ow_stateinfo.h"
@@ -346,4 +359,7 @@ enum deviceformat { fdi, fi, fdidc, fdic, fidc, fic };
 /* Program control */
 #include "ow_programs.h"
 
+/* Return and error codes */
+#include "ow_return_code.h"
+
 #endif							/* OW_H */
