@@ -1,5 +1,5 @@
 /*
-$Id: ow_alloc.h,v 1.8 2012/05/20 20:06:39 alfille Exp $
+$Id$
     OW -- One-Wire filesystem
     version 0.4 7/2/2003
 
@@ -28,7 +28,12 @@ $Id: ow_alloc.h,v 1.8 2012/05/20 20:06:39 alfille Exp $
     ---------------------------------------------------------------------------
 */
 
-// Define for full memmory allocation debugging
+//* Special routines for tracking memory leakage */
+/* Turned on by
+ * ./configure --enable-owmalloc
+ * */
+
+// Can test also by uncommenting the following line:
 // #define OW_ALLOC_DEBUG
 
 #ifndef OW_ALLOC_H			/* tedious wrapper */
