@@ -1,5 +1,5 @@
 /*
-$Id: ow_bitfield.h,v 1.1 2011/07/21 18:16:36 alfille Exp $
+$Id$
     OW -- One-Wire filesystem
     version 0.4 7/2/2003
 
@@ -42,5 +42,7 @@ struct bitfield {
 
 ZERO_OR_ERROR FS_r_bitfield(struct one_wire_query *owq) ;
 ZERO_OR_ERROR FS_w_bitfield(struct one_wire_query *owq) ;
+ZERO_OR_ERROR FS_r_bit_array(struct one_wire_query *owq) ;
+ZERO_OR_ERROR FS_w_bit_array(struct one_wire_query *owq) ;
 
 #endif							/* OW_BITFIELD_H */
