@@ -1,5 +1,5 @@
 /*
-$Id: ow_connection.h,v 1.244 2011/09/08 03:34:04 alfille Exp $
+$Id: ow_connection.h,v 1.252 2013/01/16 01:08:51 alfille Exp $
     OW -- One-Wire filesystem
     version 0.4 7/2/2003
 
@@ -42,11 +42,18 @@ See: http://www.iana.org/assignments/port-numbers
 #define DEFAULT_FTP_PORT          "21"
 #define DEFAULT_HA7_PORT          "80"
 #define DEFAULT_ENET_PORT         "8080"
+#define DEFAULT_ENET2_PORT        "8080"
 #define DEFAULT_LINK_PORT         "10001"
 #define DEFAULT_XPORT_CONTROL_PORT "9999"
 #define DEFAULT_XPORT_PORT        "10001"
 #define DEFAULT_ETHERWEATHER_PORT "15862"
 
+#define HA7_DISCOVERY_PORT        "4567"
+#define HA7_DISCOVERY_ADDRESS     "224.1.2.3"
+
+#define ENET2_DISCOVERY_PORT      30303
+#define ENET2_DISCOVERY_ADDRESS   "255.255.255.255"
+
 #include "ow.h"
 #include "ow_counters.h"
 #include <sys/ioctl.h>
@@ -101,6 +108,8 @@ See: http://www.iana.org/assignments/port-numbers
 #define W1_FIFO_SIZE 128
 #define LINK_FIFO_SIZE UART_FIFO_SIZE
 #define HA7E_FIFO_SIZE UART_FIFO_SIZE
+#define ENET_FIFO_SIZE 128
+#define ENET2_FIFO_SIZE 128
 #define HA5_FIFO_SIZE UART_FIFO_SIZE
 #define LINKE_FIFO_SIZE 1500
 #define I2C_FIFO_SIZE 1
@@ -123,6 +132,7 @@ See: http://www.iana.org/assignments/port-numbers
 
 /* -------------------------------------------- */
 /* Inbound connections (bus masters) ---------- */
+#include "ow_port_in.h"
 #include "ow_connection_in.h"
 
 /* -------------------------------------------- */
@@ -132,7 +142,6 @@ See: http://www.iana.org/assignments/port-numbers
 /* This bug-fix/workaround function seem to be fixed now... At least on
  * the platforms I have tested it on... printf() in owserver/src/c/owserver.c
  * returned very strange result on c->busmode before... but not anymore */
-enum bus_mode get_busmode(struct connection_in *c);
 int BusIsServer(struct connection_in *in);
 
 // mode bit flags for level
@@ -169,6 +178,8 @@ GOOD_OR_BAD COM_change( struct connection_in *connection) ;
 GOOD_OR_BAD serial_change(struct connection_in *connection) ;
 GOOD_OR_BAD telnet_change(struct connection_in *in) ;
 
+GOOD_OR_BAD serial_powercycle(struct connection_in *connection) ;
+
 GOOD_OR_BAD COM_write( const BYTE * data, size_t length, struct connection_in *connection);
 GOOD_OR_BAD COM_write_simple( const BYTE * data, size_t length, struct connection_in *connection);
 GOOD_OR_BAD telnet_write_binary( const BYTE * buf, const size_t size, struct connection_in *in);
@@ -188,12 +199,10 @@ void RemoveIn( struct connection_in * conn ) ;
 void FreeOutAll(void);
 void DelIn(struct connection_in *in);
 
-struct connection_in *AllocIn(const struct connection_in *in);
-struct connection_in *LinkIn(struct connection_in *in);
-struct connection_in *NewIn(const struct connection_in *in);
+struct connection_in *LinkIn(struct connection_in *in, struct port_in * head);
 
-void Add_InFlight( GOOD_OR_BAD (*nomatch)(struct connection_in * trial,struct connection_in * existing), struct connection_in * new_in );
-void Del_InFlight( GOOD_OR_BAD (*nomatch)(struct connection_in * trial,struct connection_in * existing), struct connection_in * new_in );
+void Add_InFlight( GOOD_OR_BAD (*nomatch)(struct port_in * trial,struct port_in * existing), struct port_in * new_pin );
+void Del_InFlight( GOOD_OR_BAD (*nomatch)(struct port_in * trial,struct port_in * existing), struct port_in * new_pin );
 
 struct connection_in *find_connection_in(int nr);
 int SetKnownBus( int bus_number, struct parsedname * pn) ;
