@@ -1,4 +1,4 @@
-/* $Id: ow_debug.h,v 1.30 2010/09/07 11:31:45 alfille Exp $
+/* $Id$
    Debug and error messages
    Meant to be included in ow.h
    Separated just for readability
@@ -90,7 +90,7 @@ extern int log_available;
 #define ERROR_DETAIL(...)     do { } while (0)
 #define ERROR_DEBUG(...)      do { } while (0)
 
-#define FATAL_ERROR(...)      do { } while (0)
+#define FATAL_ERROR(...)      do { exit(EXIT_FAILURE); } while (0)
 
 #define Debug_Bytes(title,buf,length)    do { } while (0)
 #define Debug_OWQ(owq)        do { } while (0)
@@ -104,7 +104,18 @@ extern int log_available;
 #define SNformat	"%.2X %.2X %.2X %.2X %.2X %.2X %.2X %.2X"
 #define SNvar(sn)	(sn)[0],(sn)[1],(sn)[2],(sn)[3],(sn)[4],(sn)[5],(sn)[6],(sn)[7]
 
-#if OW_MT
+extern const char  sem_init_failed[];
+// FreeBSD note: if we continue with failed semaphores, we WILL segfault later on.
+#define my_sem_init(sem, shared, value)              do {\
+	int mrc = sem_init(sem, shared, value);	            \
+	if(mrc != 0) { fprintf(stderr, sem_init_failed,        errno, strerror(errno)); \
+		if(errno == 28) { \
+			LEVEL_DEFAULT("Too many semaphores running, please run fewer apps utilizing semaphores (owfs apps for example), or tweak sysctl kern.ipc.sem*. Exiting.\n"); \
+		} \
+		FATAL_ERROR(sem_init_failed, mrc, strerror(mrc));			\
+	}} while(0)
+ 
+ 
 /* Need to define those functions to get FILE and LINE information */
 
 extern const char  mutex_init_failed[];
@@ -167,25 +178,4 @@ extern const char  cond_destroy_failed[];
 	int mrc = pthread_cond_destroy(cond);	                \
 	if(mrc != 0) { FATAL_ERROR( cond_destroy_failed,      mrc, strerror(mrc)); }} while (0)
 
-//extern const char rwlock_init_failed[];
-//extern const char rwlock_read_lock_failed[];
-//extern const char rwlock_read_unlock_failed[];
-
-#else /* not OW_MT */
-
-#define my_pthread_mutex_init(mutex, attr)              do {} while(0)
-#define my_pthread_mutex_destroy(mutex)                 do {} while(0)
-#define my_pthread_mutex_lock(mutex)                    do {} while (0)
-#define my_pthread_mutex_unlock(mutex)                  do {} while (0)
-#define my_pthread_mutexattr_init(attr)                 do {} while (0)
-#define my_pthread_mutexattr_destroy(attr)              do {} while (0)
-#define my_pthread_mutexattr_settype(attr, typ)         do {} while (0)
-#define my_pthread_cond_timedwait(cond, mutex, abstime)	do {} while (0)
-#define my_pthread_cond_wait(cond, mutex)	            do {} while (0)
-#define my_pthread_cond_signal(cond)	                do {} while (0)
-#define my_pthread_cond_init(cond, attr)                do {} while (0)
-#define my_pthread_cond_destroy(cond)	                do {} while (0)
-
-#endif /* OW_MT */
-
 #endif							/* OW_DEBUG_H */
