@@ -1,5 +1,5 @@
 /*
-$Id: ow_functions.h,v 1.93 2011/04/22 02:48:31 alfille Exp $
+$Id$
     OW -- One-Wire filesystem
     version 0.4 7/2/2003
 
@@ -50,7 +50,7 @@ $Id: ow_functions.h,v 1.93 2011/04/22 02:48:31 alfille Exp $
 #define  WRITE_FUNCTION( fname )  static int fname( struct one_wire_query * owq )
 
 /* Prototypes for owlib.c -- libow overall control */
-void LibSetup(enum opt_program op);
+void LibSetup(enum enum_program_type op);
 GOOD_OR_BAD LibStart(void);
 void HandleSignals(void);
 void LibStop(void);
@@ -64,6 +64,7 @@ void DeviceDestroy(void);
 int filetype_cmp(const void *name, const void *ex);
 ZERO_OR_ERROR FS_ParsedNamePlus(const char *path, const char *file, struct parsedname *pn);
 ZERO_OR_ERROR FS_ParsedNamePlusExt(const char *path, const char *file, int extension, enum ag_index alphanumeric, struct parsedname *pn);
+ZERO_OR_ERROR FS_ParsedNamePlusText(const char *path, const char *file, const char *extension, struct parsedname *pn);
 ZERO_OR_ERROR FS_ParsedName(const char *fn, struct parsedname *pn);
 ZERO_OR_ERROR FS_ParsedName_BackFromRemote(const char *fn, struct parsedname *pn);
 void FS_ParsedName_destroy(struct parsedname *pn);
@@ -92,10 +93,16 @@ char num2char(const BYTE n);
 void num2string(char *s, const BYTE n);
 void string2bytes(const char *str, BYTE * b, const int bytes);
 void bytes2string(char *str, const BYTE * b, const int bytes);
-int UT_getbit(const BYTE * buf, const int loc);
-int UT_get2bit(const BYTE * buf, const int loc);
-void UT_setbit(BYTE * buf, const int loc, const int bit);
-void UT_set2bit(BYTE * buf, const int loc, const int bits);
+
+int UT_getbit(const BYTE * buf, int loc);
+void UT_setbit(BYTE * buf, int loc, int bit);
+
+int UT_getbit_U(UINT U, int loc);
+void UT_setbit_U(UINT * U, int loc, int bit);
+
+int UT_get2bit(const BYTE * buf, int loc);
+void UT_set2bit(BYTE * buf, int loc, int bits);
+
 void UT_fromDate(const _DATE D, BYTE * data);
 _DATE UT_toDate(const BYTE * date);
 
@@ -107,8 +114,7 @@ void Init_Pipe( FILE_DESCRIPTOR_OR_ERROR * pipe_fd ) ;
 #include "ow_cache.h"
 void FS_LoadDirectoryOnly(struct parsedname *pn_directory, const struct parsedname *pn_original);
 
-GOOD_OR_BAD FS_Test_Simultaneous( enum simul_type type, UINT delay, const struct parsedname * pn) ;
-GOOD_OR_BAD FS_poll_convert(const struct parsedname *pn);
+GOOD_OR_BAD FS_Test_Simultaneous( const struct internal_prop *ip, UINT delay, const struct parsedname * pn) ;
 
 // ow_locks.c
 void LockSetup(void);
@@ -165,6 +171,9 @@ SIZE_OR_ERROR FS_read_local( struct one_wire_query *owq);
 
 size_t FileLength_vascii(struct one_wire_query *owq);
 
+ZERO_OR_ERROR FS_r_external( struct one_wire_query * owq ) ;
+ZERO_OR_ERROR FS_w_external( struct one_wire_query * owq ) ;
+
 #include "ow_sibling.h"
 
 ZERO_OR_ERROR FS_fstat(const char *path, struct stat *stbuf);
@@ -191,10 +200,16 @@ void BUS_lock(const struct parsedname *pn);
 void BUS_unlock(const struct parsedname *pn);
 void BUS_lock_in(struct connection_in *in);
 void BUS_unlock_in(struct connection_in *in);
+void CHANNEL_lock_in(struct connection_in *in);
+void CHANNEL_unlock_in(struct connection_in *in);
+void PORT_lock_in(struct connection_in *in);
+void PORT_unlock_in(struct connection_in *in);
 
 /* API wrappers for swig and owcapi */
-void API_setup(enum opt_program opt);
-GOOD_OR_BAD API_init(const char *command_line);
+enum restart_init { restart_if_repeat, continue_if_repeat, } ; // behavior if init called a second time
+void API_setup(enum enum_program_type opt);
+GOOD_OR_BAD API_init(const char *command_line, enum restart_init repeat);
+GOOD_OR_BAD API_init_args(int argc, char **argv, enum restart_init repeat) ;
 void API_set_error_level(const char *params);
 void API_set_error_print(const char *params);
 void API_finish(void);
