@@ -1,5 +1,5 @@
 /*
-$Id: ow_global.h,v 1.35 2012/03/30 02:54:28 alfille Exp $
+$Id$
     OW -- One-Wire filesystem
     version 0.4 7/2/2003
 
@@ -44,7 +44,7 @@ $Id: ow_global.h,v 1.35 2012/03/30 02:54:28 alfille Exp $
     25-05-2003 iButtonLink device
 */
 
-/* CAnn stand alone -- separated out of ow.h for clarity */
+/* Can stand alone -- separated out of ow.h for clarity */
 
 #ifndef OW_GLOBAL_H				/* tedious wrapper */
 #define OW_GLOBAL_H
@@ -56,6 +56,7 @@ $Id: ow_global.h,v 1.35 2012/03/30 02:54:28 alfille Exp $
 #define GLOBAL_UNTOUCHED_TEMP_LIMIT	(-999.)
 
 #define DEFAULT_USB_SCAN_INTERVAL 10 /* seconds */
+#define DEFAULT_ENET_SCAN_INTERVAL 60 /* seconds */
 
 enum zero_support { zero_unknown, zero_none, zero_bonjour, zero_avahi, } ;
 
@@ -65,6 +66,18 @@ enum enum_program_type {
 	program_option_sensor, program_option_property,
 };
 
+/* Daemon status
+ * can be running as foreground or background
+ * this is for a state machine implementation
+ * sd is systemd which is foreground
+ * sd cannot be changed
+ * want_bg is default state
+ * goes to bg if proper program type and can daemonize
+ * */
+enum enum_daemon_status {
+	e_daemon_want_bg, e_daemon_bg, e_daemon_sd, e_daemon_sd_done, e_daemon_fg, e_daemon_unknown,
+} ;
+
 /* Globals information (for local control) */
 struct global {
 	int announce_off;			// use zeroconf?
@@ -73,15 +86,15 @@ struct global {
 	enum pressure_type pressure_scale ;
 	enum deviceformat format ;
 	enum enum_program_type program_type;
+	enum enum_daemon_status daemon_status ;
 	int allow_external ; // allow this program to call external programs for read/write -- dangerous
 	int allow_other ;
 	ASCII *progname;
-	union antiloop Token;
+	struct antiloop Token;
 	int uncached ; // all requests are from /uncached directory
 	int unaliased ; // all requests are from /unaliased (no alias substitution on results)
-	int want_background;
-	int now_background;
 	int error_level;
+	int error_level_restore;
 	int error_print;
 	int fatal_debug;
 	int concurrent_connections;
@@ -113,11 +126,13 @@ struct global {
 	int clients_persistent_low;
 	int clients_persistent_high;
 	int usb_scan_interval ;
+	int enet_scan_interval ;
 	int pingcrazy;
 	int no_dirall;
 	int no_get;
 	int no_persistence;
 	int eightbit_serial;
+	int trim;
 	enum zero_support zero ;
 	int i2c_APU ;
 	int i2c_PPM ;
