@@ -1,5 +1,5 @@
 /*
-$Id: ow_mutexes.h,v 1.25 2012/04/14 03:52:46 alfille Exp $
+$Id$
     OW -- One-Wire filesystem
     version 0.4 7/2/2003
 
@@ -51,11 +51,8 @@ $Id: ow_mutexes.h,v 1.25 2012/04/14 03:52:46 alfille Exp $
 #ifndef OW_MUTEXES_H			/* tedious wrapper */
 #define OW_MUTEXES_H
 
-
-#if OW_MT
 #include <pthread.h>
 #define DEFAULT_THREAD_ATTR	NULL
-#endif							/* OW_MT */
 
 extern struct mutexes {
 	pthread_mutex_t stat_mutex;
@@ -70,6 +67,8 @@ extern struct mutexes {
 	pthread_mutex_t aliasfind_mutex;
 	pthread_mutex_t aliaslist_mutex;
 	pthread_mutex_t externalcount_mutex;
+	pthread_mutex_t timegm_mutex;
+	pthread_mutex_t detail_mutex;
 	
 	pthread_mutexattr_t mattr; // mutex attribute -- used for all mutexes
 	my_rwlock_t lib;
@@ -82,7 +81,6 @@ extern struct mutexes {
 
 
 
-#if OW_MT
 #define _MUTEX_ATTR_INIT(at)	my_pthread_mutexattr_init(    &(at) )
 #define _MUTEX_ATTR_SET(at,typ)	my_pthread_mutexattr_settype( &(at) , typ )
 #define _MUTEX_ATTR_DESTROY(at)	my_pthread_mutexattr_destroy( &(at) )
@@ -96,6 +94,8 @@ extern struct mutexes {
 #define RWLOCK_INIT(rw)		my_rwlock_init(    &(rw) )
 #define RWLOCK_DESTROY(rw)	my_rwlock_destroy( &(rw) )
 
+#define _SEM_INIT(sem, shared, value)	my_sem_init(  &(sem) , (shared), (value) )
+
 #define RWLOCK_WLOCK(mut)	my_rwlock_write_lock(   &(mut) )
 #define RWLOCK_WUNLOCK(mut)	my_rwlock_write_unlock( &(mut) )
 
@@ -163,10 +163,20 @@ extern struct mutexes {
 #define EXTERNALCOUNTLOCK   _MUTEX_LOCK(  Mutex.externalcount_mutex)
 #define EXTERNALCOUNTUNLOCK _MUTEX_UNLOCK(Mutex.externalcount_mutex)
 
+#define TIMEGMLOCK   		_MUTEX_LOCK(  Mutex.timegm_mutex)
+#define TIMEGMUNLOCK 		_MUTEX_UNLOCK(Mutex.timegm_mutex)
+
+#define DETAILLOCK   		_MUTEX_LOCK(  Mutex.detail_mutex)
+#define DETAILUNLOCK 		_MUTEX_UNLOCK(Mutex.detail_mutex)
+
 #define BUSLOCK(pn)       	BUS_lock(pn)
 #define BUSUNLOCK(pn)     	BUS_unlock(pn)
 #define BUSLOCKIN(in)     	BUS_lock_in(in)
 #define BUSUNLOCKIN(in)   	BUS_unlock_in(in)
+#define CHANNELLOCKIN(in)   CHANNEL_lock_in(in)
+#define CHANNELUNLOCKIN(in) CHANNEL_unlock_in(in)
+#define PORTLOCKIN(in)     	PORT_lock_in(in)
+#define PORTUNLOCKIN(in)   	PORT_unlock_in(in)
 #ifdef __UCLIBC__
 #define UCLIBCLOCK        	_MUTEX_LOCK(  Mutex.uclibc_mutex)
 #define UCLIBCUNLOCK      	_MUTEX_UNLOCK(Mutex.uclibc_mutex)
@@ -177,88 +187,4 @@ extern struct mutexes {
 
 #define DETACH_THREAD		pthread_detach(pthread_self())
 
-#else							/* OW_MT */
-
-#define _MUTEX_ATTR_DESTROY(at)	return_ok()
-
-#define _MUTEX_INIT(mut)	(void) mut
-#define _MUTEX_DESTROY(mut)	(void) mut
-#define _MUTEX_LOCK(mut)	return_ok()
-#define _MUTEX_UNLOCK(mut)	return_ok()
-
-#define LIB_WLOCK			return_ok()
-#define LIB_WUNLOCK			return_ok()
-#define LIB_RLOCK			return_ok()
-#define LIB_RUNLOCK			return_ok()
-
-#define CACHE_WLOCK			return_ok()
-#define CACHE_WUNLOCK		return_ok()
-#define CACHE_RLOCK			return_ok()
-#define CACHE_RUNLOCK		return_ok()
-
-#define PERSISTENT_WLOCK	return_ok()
-#define PERSISTENT_WUNLOCK	return_ok()
-#define PERSISTENT_RLOCK	return_ok()
-#define PERSISTENT_RUNLOCK	return_ok()
-
-#define CONNIN_WLOCK		return_ok()
-#define CONNIN_WUNLOCK		return_ok()
-#define CONNIN_RLOCK		return_ok()
-#define CONNIN_RUNLOCK		return_ok()
-
-#define MONITOR_WLOCK      	return_ok()
-#define MONITOR_WUNLOCK    	return_ok()
-#define MONITOR_RLOCK      	return_ok()
-#define MONITOR_RUNLOCK    	return_ok()
-
-#define STATLOCK			return_ok()
-#define STATUNLOCK			return_ok()
-
-#define CONTROLFLAGSLOCK	return_ok()
-#define CONTROLFLAGSUNLOCK	return_ok()
-
-#define FSTATLOCK			return_ok()
-#define FSTATUNLOCK			return_ok()
-
-#define SIMULLOCK			return_ok()
-#define SIMULUNLOCK			return_ok()
-
-#define DIRLOCK				return_ok()
-#define DIRUNLOCK			return_ok()
-
-#define UCLIBCLOCK			return_ok()
-#define UCLIBCUNLOCK		return_ok()
-
-#define LIBUSBLOCK			return_ok()
-#define LIBUSBUNLOCK		return_ok()
-
-#define TYPEDIRLOCK			return_ok()
-#define TYPEDIRUNLOCK		return_ok()
-
-#define EXTERNALDIRLOCK		return_ok()
-#define EXTERNALDIRUNLOCK	return_ok()
-
-#define NAMEFINDLOCK		return_ok()
-#define NAMEFINDUNLOCK		return_ok()
-
-#define ALIASFINDLOCK		return_ok()
-#define ALIASFINDUNLOCK		return_ok()
-
-#define ALIASLISTLOCK		return_ok()
-#define ALIASLISTUNLOCK		return_ok()
-
-#define EXTERNALCOUNTLOCK	return_ok()
-#define EXTERNALCOUNTUNLOCK	return_ok()
-
-#define UCLIBCLOCK			return_ok()
-#define UCLIBCUNLOCK		return_ok()
-#define BUSLOCK(pn)			return_ok()
-#define BUSUNLOCK(pn)		return_ok()
-#define BUSLOCKIN(in)		return_ok()
-#define BUSUNLOCKIN(in)		return_ok()
-
-#define DETACH_THREAD		do {} while(0)
-
-#endif							/* OW_MT */
-
 #endif							/* OW_MUTEXES_H */
