@@ -1,5 +1,4 @@
 /*
-$Id: ow_parsedname.h,v 1.39 2011/07/08 20:53:02 alfille Exp $
     OW -- One-Wire filesystem
     version 0.4 7/2/2003
 
@@ -60,7 +59,7 @@ Parsed name has several components:
 sn is the serial number of the device
 dev and ft are pointers to device and filetype
   members corresponding to the element
-buspath and pathlength interpret the route through
+ds2409_hubs and ds2409_depth interpret the route through
   DS2409 branch controllers
 filetype and extension correspond to property
   (filetype) details
@@ -80,11 +79,12 @@ extern int32_t LocalControlFlags;
 // flag to pn->selected_connection->branch.branch to force select from root dir
 enum eBranch {
 	eBranch_bad =  0xFF,
+	eBranch_cleared =  0xFE,
 	eBranch_main = 0x00,
 	eBranch_aux =  0x01,
 };
 
-struct buspath {
+struct ds2409_hubs {
 	BYTE sn[SERIAL_NUMBER_SIZE];
 	BYTE branch;
 };
@@ -124,11 +124,13 @@ enum ePS_state {
 	ePS_busveryremote = 0x0080,
 	ePS_reconnection  = 0x0100,
 	ePS_unaliased     = 0x0200,
+	ePS_json          = 0x0400,
 };
 
 struct parsedname {
-	char *path;				// text-more device name
-	char *path_to_server;			// pointer to path without first bus
+	char path[2*PATH_MAX+2];				// full device name
+	char path_to_server[PATH_MAX+2];			// path without first bus
+	char * device_name ;		// for external name
 	struct connection_in *known_bus;	// where this device is located
 	enum ePN_type type;			// real? settings? ...
 	enum ePS_state state;			// alarm?
@@ -136,13 +138,16 @@ struct parsedname {
 	struct device *selected_device;		// 1-wire device
 	struct filetype *selected_filetype;	// device property
 	int extension;				// numerical extension (for array values) or -1
+	char * sparse_name;         // text extension for a spare array value
 	struct filetype *subdir;		// in-device grouping
 	int dirlength ;				// Length of just directory part of path
-	UINT pathlength;			// DS2409 branching depth
-	struct buspath *bp;			// DS2409 branching route
+	UINT ds2409_depth;			// DS2409 branching depth
+	struct ds2409_hubs *bp;			// DS2409 branching route
 	struct connection_in *selected_connection;	// which bus is assigned to this item
 	uint32_t control_flags;				// more state info, packed for network transmission
 	struct devlock *lock;			// pointer to a device-specific lock
+	int return_code ; // return (error) code
+	int detail_flag ; // matches a detail request
 	int tokens;				// for anti-loop work
 	BYTE *tokenstring;			// List of tokens from owservers passed
 };
@@ -154,6 +159,7 @@ struct parsedname {
 #define ALIAS_REQUEST               ( (UINT) 0x00000008 )
 #define SAFEMODE                    ( (UINT) 0x00000010 )
 #define UNCACHED                    ( (UINT) 0x00000020 )
+#define TRIM                        ( (UINT) 0x00000040 )
 #define OWNET                       ( (UINT) 0x00000100 )
 #define TEMPSCALE_MASK              ( (UINT) 0x00030000 )
 #define TEMPSCALE_BIT      16
@@ -188,6 +194,8 @@ struct parsedname {
 
 #define     InSafeMode(pn)    ( (((pn)->control_flags) & SAFEMODE ) != 0 )
 
+#define     ShouldTrim(pn)    ( (((pn)->control_flags) & TRIM ) != 0 )
+
 #define KnownBus(pn)          ((((pn)->state) & ePS_bus) != 0 )
 #define UnsetKnownBus(pn)           do { (pn)->state &= ~ePS_bus; \
                                         (pn)->known_bus=NULL; \
@@ -202,7 +210,7 @@ struct parsedname {
 
 #define SpecifiedBus(pn)          ( SpecifiedLocalBus(pn) || SpecifiedRemoteBus(pn) )
 
-#define RootNotBranch(pn)         (((pn)->pathlength)==0)
+#define RootNotBranch(pn)         (((pn)->ds2409_depth)==0)
 
 enum parse_enum {
 	parse_first,
