@@ -1,5 +1,5 @@
 /* From Paul Alfille */
-/* $Id: rwlock.h,v 1.10 2010/09/07 11:31:45 alfille Exp $ */
+/* $Id$ */
 /* Implementation of Reader/Writer locks using semaphores and mutexes */
 /* based on "The Little Book of Semaphores" bu Allen B. Downey */
 /* http://www.greenteapress.com */
@@ -13,8 +13,6 @@
 
 #include "owfs_config.h"
 
-#if OW_MT
-
 #include <pthread.h>
 #include "sem.h"
 
@@ -26,16 +24,10 @@ typedef struct {
 } my_rwlock_t;
 
 void my_rwlock_init(my_rwlock_t * rwlock);
-inline void my_rwlock_write_lock(my_rwlock_t * rwlock);
-inline void my_rwlock_write_unlock(my_rwlock_t * rwlock);
-inline void my_rwlock_read_lock(my_rwlock_t * rwlock);
-inline void my_rwlock_read_unlock(my_rwlock_t * rwlock);
+void my_rwlock_write_lock(my_rwlock_t * rwlock);
+void my_rwlock_write_unlock(my_rwlock_t * rwlock);
+void my_rwlock_read_lock(my_rwlock_t * rwlock);
+void my_rwlock_read_unlock(my_rwlock_t * rwlock);
 void my_rwlock_destroy(my_rwlock_t * rwlock);
 
-#else /* not OW_MT */
-
-typedef int my_rwlock_t ;
-
-#endif							/* OW_MT */
-
 #endif							/* RWLOCK */
