@@ -1,5 +1,5 @@
 /* From Paul Alfille */
-/* $Id: rwlock.h,v 1.5 2009/01/05 12:04:06 d1mag Exp $ */
+/* $Id$ */
 /* Implementation of Reader/Writer locks using semaphores and mutexes */
 /* based on "The Little Book of Semaphores" bu Allen B. Downey */
 /* http://www.greenteapress.com */
@@ -13,8 +13,6 @@
 
 #include "owfs_config.h"
 
-#if OW_MT
-
 #include <pthread.h>
 #include "sem.h"
 
@@ -32,6 +30,4 @@ inline void my_rwlock_read_lock(my_rwlock_t * rwlock);
 inline void my_rwlock_read_unlock(my_rwlock_t * rwlock);
 void my_rwlock_destroy(my_rwlock_t * rwlock);
 
-#endif							/* OW_MT */
-
 #endif							/* RWLOCK */
