@@ -1,7 +1,7 @@
 package OWNet ;
 
 # OWNet module for perl
-# $Id: OWNet.pm,v 1.22 2011/03/24 01:40:39 alfille Exp $
+# $Id$
 #
 # Paul H Alfille -- copyright 2007
 # Part of the OWFS suite:
@@ -97,17 +97,17 @@ Pressure scale can also be specified in the I<address>. Same syntax as the other
 
 =over
 
-=item --mbar     millibar (default)
+=item -mbar     millibar (default)
 
-=item --atm      atmosphere
+=item -atm      atmosphere
 
-=item --mmHg     mm Mercury
+=item -mmHg     mm Mercury
 
-=item --inHg     inch Mercury
+=item -inHg     inch Mercury
 
-=item --psi      pounds per inch^2
+=item -psi      pounds per inch^2
 
-=item --Pa       pascal
+=item -Pa       pascal
 
 =back
 
@@ -133,7 +133,15 @@ Show directories that are themselves directories with a '/' suffix ( e.g. /10.67
 
 =over
 
-=item -slash  show directory elements
+=item --slash  show directory elements
+
+=back
+
+Trim whitespace from numeric values
+
+=over
+
+=item -trim  trim spaces
 
 =back
 
@@ -204,7 +212,7 @@ my $PERSISTENCE_BIT = 0x04 ;
 my $DEFAULT_SG = 0x100 + 0x2 + 0x8 ;
 my $DEFAULT_BLOCK_LENGTH = 33000 ;
 
-our $VERSION=(split(/ /,q[$Revision: 1.22 $]))[1] ;
+our $VERSION=(split(/ /,q[$Revision$]))[1] ;
 
 sub _new($$) {
 	my ($self,$addr) = @_ ;
@@ -245,6 +253,11 @@ sub _new($$) {
 		$format = 0x1000000 , last FORMAT if $addr =~ /-fi/ ;
 	}
 
+	my $trim = 0 ;
+	TRIM: {
+		$trim = 0x00000040 , last TRIM if $addr =~ /-trim/ ;
+	}
+
 	# Verbose flag
 	$self->{VERBOSE} = 1 if $addr =~ /-v/i ;
 	
@@ -267,7 +280,7 @@ sub _new($$) {
 
 	$self->{ADDR} = $addr ;
 	$self->{PORT} = $port ;
-	$self->{SG} = $DEFAULT_SG + $tempscale + $presscale + $format ;
+	$self->{SG} = $DEFAULT_SG + $tempscale + $presscale + $format + $trim ;
 	$self->{VER} = 0 ;
 }
 
@@ -346,8 +359,8 @@ sub _BonjourLookup($) {
 		$self->{SOCK} = undef ;
 	return ;
 	} ;
-	$self->{ADDR} = $self->{SOCK}->peeraddr ;
-	$self->{PORT} = $self->{SOCK}->peerport ;
+	$self->{ADDR} = $self->{SOCK}->peeraddr || return ;
+	$self->{PORT} = $self->{SOCK}->peerport || return ;
 	return 1 ;
 }
 
@@ -378,7 +391,7 @@ sub _FromServerBinaryParse($$) {
 	my $length_wanted = shift ;
 	return '' if $length_wanted == 0 ;
 
-	my $fileno = $self->{SOCK}->fileno ;
+	my $fileno = $self->{SOCK}->fileno  or return undef ;
 	my $selectreadbits = '' ;
 	vec($selectreadbits,$fileno,1) = 1 ;
 
@@ -615,6 +628,7 @@ sub dir($$) {
 	# new MSG_DIRALL method -- single packet
 	_ToServer($self,$MSG_DIRALL,$DEFAULT_BLOCK_LENGTH,$NO_OFFSET,$path) || return ;
 	my @r = _FromServer($self) ;
+	return if !@r ;
 	if (@r) {
 		$self->{SOCK} = undef if $self->{PERSIST} == 0 ;
 		return $r[6] ;
