@@ -1,5 +1,5 @@
 /*
-$Id: data.c,v 1.51 2012/03/27 03:42:48 alfille Exp $
+$Id: data.c,v 1.52 2013/01/01 16:03:37 alfille Exp $
     OW_HTML -- OWFS used for the web
     OW -- One-Wire filesystem
 
@@ -214,14 +214,13 @@ void *DataHandler(void *v)
 		ErrorToClient(hd, &cm) ;
 	}
 
-#if OW_MT
 	// Signal to PingLoop that we're done.
 	hd->toclient = toclient_complete ;
 	if ( hd->ping_pipe[fd_pipe_write] != FILE_DESCRIPTOR_BAD ) {
 		ignore_result = write( hd->ping_pipe[fd_pipe_write],"X",1) ; //dummy payload
 	}
-#endif /* OW_MT */
 	TOCLIENTUNLOCK(hd);
+
 	if (retbuffer) {
 		owfree(retbuffer);
 	}
