@@ -1,5 +1,5 @@
 /*
-$Id: owserver.c,v 1.127 2011/05/11 01:53:30 alfille Exp $
+$Id: owserver.c,v 1.131 2013/01/01 16:03:37 alfille Exp $
     OW_HTML -- OWFS used for the web
     OW -- One-Wire filesystem
 
@@ -43,13 +43,16 @@ static void SetupAntiloop(void);
 int main(int argc, char **argv)
 {
 	int c;
-
+	
 	/* Set up owlib */
-	LibSetup(opt_server);
+	LibSetup(program_type_server);
 
 	/* grab our executable name */
 	if (argc > 0) {
 		Globals.progname = owstrdup(argv[0]);
+		if ( strcasecmp( Globals.progname, "owexternal" ) == 0 ) {
+			Globals.allow_external = 1 ; // only if program named "owexternal"
+		}
 	}
 
 	while ((c = getopt_long(argc, argv, OWLIB_OPT, owopts_long, NULL)) != -1) {
@@ -90,17 +93,15 @@ int main(int argc, char **argv)
 	set_exit_signal_handlers(exit_handler);
 	set_signal_handlers(NULL);
 
-#if OW_MT
 	_MUTEX_INIT(persistence_mutex);
-#endif
 
 	/* Set up "Antiloop" -- a unique token */
 	SetupAntiloop();
-	ServerProcess(Handler);
+	ServerProcess( Handler );
 	LEVEL_DEBUG("ServerProcess done");
-#if OW_MT
+
 	_MUTEX_DESTROY(persistence_mutex);
-#endif
+
 	ow_exit(0);
 	return 0;
 }
