@@ -1,5 +1,5 @@
 /*
-$Id: to_client.c,v 1.20 2010/08/03 01:07:55 alfille Exp $
+$Id: to_client.c,v 1.24 2012/12/26 01:19:25 alfille Exp $
     OW_HTML -- OWFS used for the web
     OW -- One-Wire filesystem
 
@@ -40,18 +40,29 @@ $Id: to_client.c,v 1.20 2010/08/03 01:07:55 alfille Exp $
 /* Send fully configured message back to client.
    data is optional and length depends on "payload"
  */
-int ToClient(int file_descriptor, struct client_msg *machine_order_cm, char *data)
+int ToClient(int file_descriptor, struct client_msg *machine_order_cm, const char *data)
 {
 	struct client_msg s_cm;
 	struct client_msg *network_order_cm = &s_cm;
 	
-	// note data should be (const char *) but iovec complains about const arguments
 	int nio = 1;
+
+#if ( __GNUC__ > 4 ) || (__GNUC__ == 4 && __GNUC_MINOR__ > 4 )
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+	// note data should be (const char *) but iovec complains about const arguments
 	struct iovec io[] = {
 		{network_order_cm, sizeof(struct client_msg),},
-		{data, machine_order_cm->payload,},
+		{ (char *) data, machine_order_cm->payload,},
 	};
-
+#pragma GCC diagnostic pop
+#else
+	// note data should be (const char *) but iovec complains about const arguments
+	struct iovec io[] = {
+		{network_order_cm, sizeof(struct client_msg),},
+		{ (char *) data, machine_order_cm->payload,},
+	};
+#endif
 	LEVEL_DEBUG("payload=%d size=%d, ret=%d, sg=0x%X offset=%d ", machine_order_cm->payload, machine_order_cm->size, machine_order_cm->ret,
 		     machine_order_cm->control_flags, machine_order_cm->offset);
 
