@@ -1,12 +1,12 @@
 /*
-$Id: owserver.h,v 1.32 2012/03/27 17:34:27 alfille Exp $
+$Id$
    OWFS and OWHTTPD
    one-wire file system and
    one-wire web server
 
     By Paul H Alfille
     {c} 2003 GPL
-    palfille@earthlink.net
+    paul.alfille@gmail.com
 */
 
 /* OWSERVER - specific header */
@@ -48,7 +48,7 @@ struct handlerdata {
 int FromClient(struct handlerdata *hd);
 
 /* Send fully configured message back to client */
-int ToClient(int file_descriptor, struct client_msg *cm, char *data);
+int ToClient(int file_descriptor, struct client_msg *cm, const char *data);
 
 /* Read from 1-wire bus and return file contents */
 void *ReadHandler(struct handlerdata *hd, struct client_msg *cm, struct one_wire_query *owq);
@@ -74,14 +74,13 @@ void Handler(FILE_DESCRIPTOR_OR_ERROR file_descriptor);
 /* Send a response to client of an error */
 void ErrorToClient(struct handlerdata *hd, struct client_msg * cm ) ;
 
-#if OW_MT
-
 /* Send a timeout ping */
 void PingClient(struct handlerdata *hd);
 
 /* Loop waiting for finish sending pings */
 void PingLoop(struct handlerdata *hd) ;
 
-#endif /* OW_MT */
+/* Create a md5 hash (for the token) */
+void md5(const uint8_t *initial_msg, size_t initial_len, uint8_t *digest) ;
 
 #endif							/* OWSERVER_H */
