@@ -1,9 +1,9 @@
 /*
-$Id: ow_net.c,v 1.12 2009/08/04 12:12:38 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -94,8 +94,9 @@ int ClientConnect(void)
 	if (ai) {
 		file_descriptor = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
 		if (file_descriptor >= 0) {
-			if (connect(file_descriptor, ai->ai_addr, ai->ai_addrlen) == 0)
+			if (connect(file_descriptor, ai->ai_addr, ai->ai_addrlen) == 0) {
 				return file_descriptor;
+			}
 			close(file_descriptor);
 		}
 	}
