@@ -1,9 +1,9 @@
 /*
-$Id: ow_server.c,v 1.34 2012/03/20 23:09:41 alfille Exp $
+$Id$
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
-    email: palfille@earthlink.net
+    email: paul.alfille@gmail.com
     Released under the GPL
     See the header file: ow.h for full attribution
     1wire/iButton system from Dallas Semiconductor
@@ -335,7 +335,7 @@ static int ToServer(int file_descriptor, struct server_msg *sm, struct serverpac
 	sm->offset = htonl(sm->offset);
 
 	//Debug_Writev(io, nio);
-	return writev(file_descriptor, io, nio) != (ssize_t) (payload + sizeof(struct server_msg) + sp->tokens * sizeof(union antiloop));
+	return writev(file_descriptor, io, nio) != (ssize_t) (payload + sizeof(struct server_msg) + sp->tokens * sizeof(struct antiloop));
 }
 
 /* flag the sg for "virtual root" -- the remote bus was specifically requested */
@@ -354,6 +354,8 @@ static uint32_t SetupSemi(void)
 	sg |= uncached ? UNCACHED : 0 ;
 	// Unaliased
 	sg |= unaliased ? 0 : ALIAS_REQUEST ;
+	// Trim
+	sg |= trim ? TRIM : 0 ;
 	// OWNet flag or Presence check
 	sg |= OWNET ;
 
