@@ -1,5 +1,5 @@
 /*
-$Id: owexist.c,v 1.1 2012/03/20 23:09:41 alfille Exp $
+$Id: owexist.c,v 1.3 2012/07/10 14:16:32 alfille Exp $
      OW -- One-Wire filesystem
     version 0.4 7/2/2003
 
@@ -22,6 +22,8 @@ $Id: owexist.c,v 1.1 2012/03/20 23:09:41 alfille Exp $
 /* ---------------------------------------------- */
 int main(int argc, char *argv[])
 {
+	int fd ;
+	
 	Setup();
 
 	/* process command line arguments */
@@ -35,7 +37,15 @@ int main(int argc, char *argv[])
 
 	DefaultOwserver();
 	Server_detect(); // Exit(1) if problem.
-
+	
+	fd = ClientConnect() ;
+	if ( fd == -1 ) {
+		PRINT_ERROR("Cannot connect\n");
+		Exit(1);
+	}
+	
+	// Success!
+	close(fd);
 	Exit(0) ;
-
+	return 0 ;
 }
