@@ -1,5 +1,5 @@
 /*
-$Id: owshell.h,v 1.33 2011/08/10 01:54:07 alfille Exp $
+$Id$
 
     Written 2006 Paul H Alfille
 
@@ -119,7 +119,7 @@ typedef int INT;
 
 /* command line options */
 /* These are the owlib-specific options */
-#define OWLIB_OPT "m:c:f:p:s:hu::d:t:CFRKVP:"
+#define OWLIB_OPT "m:c:f:p:s:hqu::d:t:CFRKVP:"
 extern const struct option owopts_long[];
 void owopt(const int c, const char *arg);
 
@@ -170,11 +170,7 @@ extern int count_inbound_connections;
 #define OW_DEFAULT_LENGTH (128)
 
 /* Unique token for owserver loop checks */
-union antiloop {
-	struct {
-		pid_t pid;
-		clock_t clock;
-	} simple;
+struct antiloop {
 	BYTE uuid[16];
 };
 
@@ -183,12 +179,12 @@ struct global {
 	int announce_off;			// use zeroconf?
 	ASCII *announce_name;
 	ASCII *progname;
-	union antiloop Token;
-	int want_background;
-	int now_background;
+	struct antiloop Token;
 	int readonly;
 	int max_clients;			// for ftp
 	int autoserver;
+	int quiet ;
+	int trim ;
 	/* Special parameter to trigger William Robison <ibutton@n952.dyndns.ws> timings */
 	int timeout_network ;
 
@@ -262,6 +258,7 @@ extern int size_of_data ;
 extern int offset_into_data ;
 extern int uncached ;
 extern int unaliased ;
+extern int trim ;
 extern enum temp_type temperature_scale ;
 extern enum pressure_type pressure_scale ;
 extern enum deviceformat device_format ;
@@ -291,8 +288,12 @@ int ServerPresence(ASCII * path);
 #define DEVFORMAT_MASK              ( (UINT) 0xFF000000 )
 #define DEVFORMAT_BIT  24
 #define UNCACHED                    ( (UINT) 0x00000020 )
+#define TRIM                        ( (UINT) 0x00000040 )
 #define OWNET                       ( (UINT) 0x00000100 )
 
+#define PRINT_ERROR(...)		while ( ! Globals.quiet ) { fprintf( stderr, __VA_ARGS__ ) ; break ; }
+#define PERROR(...)				while ( ! Globals.quiet ) { perror( __VA_ARGS__ ) ; break ; }
+
 struct connection_in;
 
 struct device_search {
