@@ -1,5 +1,5 @@
 /*
-$Id: owfs_config.h.in,v 1.29 2010/07/20 04:41:05 alfille Exp $
+$Id$
     OW -- One-Wire filesystem
 
     Function naming scheme:
@@ -53,23 +53,18 @@ $Id: owfs_config.h.in,v 1.29 2010/07/20 04:41:05 alfille Exp $
 
 /* This file describes the version and features of the owfs-installation. */
 
-#define OWFS_VERSION    "2.8p14"
+#define OWFS_VERSION    "2.9p5"
 #define OWFS_MAJOR      "2"
-#define OWFS_MINOR      "8"
-#define OWFS_PATCHLEVEL "14"
+#define OWFS_MINOR      "9"
+#define OWFS_PATCHLEVEL "5"
 
-#define OW_MT           1
 #define OW_ALLOC_DEBUG  0
 #define OW_SHOW_TRAFFIC 0
-#define OW_CACHE        1
 #define OW_USB          1
 #define OW_PARPORT      1
 #define OW_DEBUG        1
-#define OW_TAI8570      1
-#define OW_HA7          1
 #define OW_W1           1
 #define OW_I2C          1
-#define OW_THERMOCOUPLE 1
 #define OW_ZERO         1
 #define OW_CYGWIN       0
 #define OW_DARWIN       0
